----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:15:55 12/30/2011 
-- Design Name: 
-- Module Name:    A10KHzCounter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity A10KHzCounter is
    Port (
		
		TheCounter 		: out STD_LOGIC_VECTOR(15 downto 0);
		
		-- Usual Ports 
		Reset 			: in  STD_LOGIC;
		Clk 				: in  STD_LOGIC	 
	 );
end A10KHzCounter;

architecture Behavioral of A10KHzCounter is

	-- The 95+ MHZ Counter
	signal Counter : unsigned (29 downto 0) := (others => '0');
	
	-- The 10kHz Counter
	signal SlowCounter : unsigned (15 downto 0) := (others => '0');
	
	-- 95.8333MHz => 10.000 Hz Clock
	constant The10thOf1MS : unsigned (29 downto 0) := to_unsigned(9583,30);

begin

	TheCounter <= std_logic_vector(SlowCounter);
	
	FastClock : process(Reset,Clk)
	begin
		if (Reset = '1') then
			
			-- Default:
			Counter <= (others => '0');
			SlowCounter <= (others => '0');
			
		elsif rising_edge(Clk) then
			
			-- Default:
			SlowCounter <= SlowCounter;
			Counter <= Counter + 1;
			
			-- Waited for 1/10 ms ?
			if Counter = The10thOf1MS then
				-- Yes
				-- Then Restart Counter
				Counter <= (others => '0');
				SlowCounter <= SlowCounter + 1;
			else
				-- No
				NULL;
			end if;
			
		end if;
		
	end process FastClock;

end Behavioral;

