----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:15:55 12/30/2011 
-- Design Name: 
-- Module Name:    A10KHzCounter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity A32KHzCounter is
    Port (
		
		TheCounter 					: out STD_LOGIC_VECTOR(15 downto 0);
		ExternalADCIsEnabled 	: in STD_LOGIC;
		ADC_NOT_DATAREADY			: in STD_LOGIC;
		
		-- Usual Ports 
		Reset 						: in  STD_LOGIC;
		Clk 							: in  STD_LOGIC	 
	 );
end A32KHzCounter;

architecture Behavioral of A32KHzCounter is

	-- The 95+ MHZ Counter
	signal Counter : unsigned (29 downto 0) := (others => '0');
	
	-- The 10kHz Counter
	signal SlowCounter : unsigned (15 downto 0) := (others => '0');
	
	-- 95.8333MHz => 32.000 Hz Clock
	constant The10thOf1MS : unsigned (29 downto 0) := to_unsigned(2995,30);

	signal ADC_NOT_DATAREADY_Last : std_logic := '0';
	
begin

	TheCounter <= std_logic_vector(SlowCounter);
	
	FastClock : process(Reset,Clk)
	begin
		if (Reset = '1') then
			
			-- Default:
			Counter <= (others => '0');
			SlowCounter <= (others => '0');
			ADC_NOT_DATAREADY_Last <= '0';
			
		elsif rising_edge(Clk) then
			
			-- Default:
			SlowCounter <= SlowCounter;
			ADC_NOT_DATAREADY_Last <= ADC_NOT_DATAREADY;
			
			if ExternalADCIsEnabled = '0' then
				Counter <= Counter + 1;
			else
				Counter <= (others => '0');
			end if;
			
			if ExternalADCIsEnabled = '0' then
				-- Waited for 1/10 ms ?
				if Counter = The10thOf1MS then
					-- Yes
					-- Then Restart Counter
					Counter <= (others => '0');
					SlowCounter <= SlowCounter + 1;
				else
					-- No
					NULL;
				end if;
			else
				if ADC_NOT_DATAREADY_Last = '1' AND ADC_NOT_DATAREADY = '0' then
					SlowCounter <= SlowCounter + 1;
				else
					SlowCounter <= SlowCounter;
				end if;
			end if;
			
		end if;
		
	end process FastClock;

end Behavioral;

