----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:20:44 01/25/2012 
-- Design Name: 
-- Module Name:    AD5162_SPI - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity AD5162_SPI is
    Port ( 
		
		Values 			: in  STD_LOGIC_VECTOR (7 downto 0);
		Valid				: in  STD_LOGIC;
		Ready  			: out STD_LOGIC;
		
		POTI_SDI			: out STD_LOGIC;
		POTI_CLK			: out STD_LOGIC;
		POTI_NOT_CS		: out STD_LOGIC;
		
		Reset 			: in STD_LOGIC;
		Clk 				: in  STD_LOGIC
	 
	 );
end AD5162_SPI;

architecture Behavioral of AD5162_SPI is

	-- The states of the state machine
	type STATE_Type is (
		FSM_RESET,
		FSM_IDLE,
		FSM_START_A,
		FSM_START_B,
		FSM_BITA_A,
		FSM_BITA_B,
		FSM_BIT7_A,
		FSM_BIT7_B,
		FSM_BIT6_A,
		FSM_BIT6_B,
		FSM_BIT5_A,
		FSM_BIT5_B,
		FSM_BIT4_A,
		FSM_BIT4_B,
		FSM_BIT3_A,
		FSM_BIT3_B,
		FSM_BIT2_A,
		FSM_BIT2_B,
		FSM_BIT1_A,
		FSM_BIT1_B,
		FSM_BIT0_A,
		FSM_BIT0_B,
		FSM_COOLDOWN_A,
		FSM_COOLDOWN_B,
		FSM_PAUSE_A,
		FSM_PAUSE_B,
		FSM_PAUSE_C,
		FSM_PAUSE_D,
		FSM_START_C,
		FSM_START_D,
		FSM_BITA_C,
		FSM_BITA_D,
		FSM_BIT7_C,
		FSM_BIT7_D,
		FSM_BIT6_C,
		FSM_BIT6_D,
		FSM_BIT5_C,
		FSM_BIT5_D,
		FSM_BIT4_C,
		FSM_BIT4_D,
		FSM_BIT3_C,
		FSM_BIT3_D,
		FSM_BIT2_C,
		FSM_BIT2_D,
		FSM_BIT1_C,
		FSM_BIT1_D,
		FSM_BIT0_C,
		FSM_BIT0_D,
		FSM_COOLDOWN_C,
		FSM_COOLDOWN_D,
		FSM_PAUSE_E
		);
	signal SPIMACHINE_STATE : STATE_TYPE;

	signal Value_Copy : std_logic_vector(7 downto 0) := (others => '0');
begin

	FSM_PROCESS : process (Clk, Reset)
	begin
		if Reset = '1' then
			SPIMACHINE_STATE <= FSM_RESET;
			Ready <= '0';
			Value_Copy <= (others => '0');
			POTI_NOT_CS <= '1';
			POTI_SDI <= '0';
			POTI_CLK <= '0';			
		elsif rising_edge(Clk) then
			
			-- Default: 
			Ready <= '0';
			POTI_NOT_CS <= '0';
			POTI_SDI <= '0';
			POTI_CLK <= '0';
			Value_Copy <= Value_Copy;
			
			case SPIMACHINE_STATE is

				when FSM_RESET =>
					SPIMACHINE_STATE <= FSM_IDLE;
					POTI_NOT_CS <= '1';
					
				when FSM_IDLE =>
					
					Ready <= '1';
					if Valid = '1' then
						SPIMACHINE_STATE <= FSM_START_A;
					else
						SPIMACHINE_STATE <= FSM_IDLE;
					end if;
					Value_Copy <= Values;
					POTI_NOT_CS <= '1';
					
				when FSM_START_A =>
					SPIMACHINE_STATE <= FSM_START_B;
					
				when FSM_START_B =>
					SPIMACHINE_STATE <= FSM_BITA_A;

-- BIT A = '0'					
				when FSM_BITA_A =>
					SPIMACHINE_STATE <= FSM_BITA_B;
					POTI_SDI <= '0';
					
				when FSM_BITA_B =>
				
					SPIMACHINE_STATE <= FSM_BIT7_A;				
					POTI_CLK <= '1';
					POTI_SDI <= '0';
					
				when FSM_BIT7_A =>
					SPIMACHINE_STATE <= FSM_BIT7_B;				
					POTI_SDI <= Value_Copy(7);
					
				when FSM_BIT7_B =>
					SPIMACHINE_STATE <= FSM_BIT6_A;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(7);
					
				when FSM_BIT6_A =>
					SPIMACHINE_STATE <= FSM_BIT6_B;				
					POTI_SDI <= Value_Copy(6);
					
				when FSM_BIT6_B =>
					SPIMACHINE_STATE <= FSM_BIT5_A;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(6);
					
				when FSM_BIT5_A =>
					SPIMACHINE_STATE <= FSM_BIT5_B;				
					POTI_SDI <= Value_Copy(5);
					
				when FSM_BIT5_B =>
					SPIMACHINE_STATE <= FSM_BIT4_A;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(5);
					
				when FSM_BIT4_A =>
					SPIMACHINE_STATE <= FSM_BIT4_B;				
					POTI_SDI <= Value_Copy(4);
					
				when FSM_BIT4_B =>
					SPIMACHINE_STATE <= FSM_BIT3_A;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(4);
					
				when FSM_BIT3_A =>
					SPIMACHINE_STATE <= FSM_BIT3_B;				
					POTI_SDI <= Value_Copy(3);
					
				when FSM_BIT3_B =>
					SPIMACHINE_STATE <= FSM_BIT2_A;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(3);
					
				when FSM_BIT2_A =>
					SPIMACHINE_STATE <= FSM_BIT2_B;				
					POTI_SDI <= Value_Copy(2);
					
				when FSM_BIT2_B =>
					SPIMACHINE_STATE <= FSM_BIT1_A;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(2);
					
				when FSM_BIT1_A =>
					SPIMACHINE_STATE <= FSM_BIT1_B;				
					POTI_SDI <= Value_Copy(1);
					
				when FSM_BIT1_B =>
					SPIMACHINE_STATE <= FSM_BIT0_A;	
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(1);
					
				when FSM_BIT0_A =>
					SPIMACHINE_STATE <= FSM_BIT0_B;
					POTI_SDI <= Value_Copy(0);
					
				when FSM_BIT0_B =>
					SPIMACHINE_STATE <= FSM_COOLDOWN_A;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(0);
					
				when FSM_COOLDOWN_A =>
					SPIMACHINE_STATE <= FSM_COOLDOWN_B;				
					
				when FSM_COOLDOWN_B =>
					SPIMACHINE_STATE <= FSM_PAUSE_A;				
					
				when FSM_PAUSE_A =>
					POTI_NOT_CS <= '1';
					SPIMACHINE_STATE <= FSM_PAUSE_B;	
					
				when FSM_PAUSE_B =>
					SPIMACHINE_STATE <= FSM_PAUSE_C;
					POTI_NOT_CS <= '1';					
					
				when FSM_PAUSE_C =>
					SPIMACHINE_STATE <= FSM_PAUSE_D;				
					POTI_NOT_CS <= '1';
					
				when FSM_PAUSE_D =>
					SPIMACHINE_STATE <= FSM_START_C;				
					POTI_NOT_CS <= '1';
					
				when FSM_START_C =>
					SPIMACHINE_STATE <= FSM_START_D;	
					
				when FSM_START_D =>
					SPIMACHINE_STATE <= FSM_BITA_C;

-- Bit A = '1'					
				when FSM_BITA_C =>
					SPIMACHINE_STATE <= FSM_BITA_D;
					POTI_SDI <= '1';							
					
				when FSM_BITA_D => 
					SPIMACHINE_STATE <= FSM_BIT7_C;	
					POTI_CLK <= '1';
					POTI_SDI <= '1';
					
				when FSM_BIT7_C =>
					SPIMACHINE_STATE <= FSM_BIT7_D;				
					POTI_SDI <= Value_Copy(7);
					
				when FSM_BIT7_D =>
					SPIMACHINE_STATE <= FSM_BIT6_C;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(7);
					
				when FSM_BIT6_C =>
					SPIMACHINE_STATE <= FSM_BIT6_D;				
					POTI_SDI <= Value_Copy(6);
					
				when FSM_BIT6_D =>
					SPIMACHINE_STATE <= FSM_BIT5_C;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(6);
					
				when FSM_BIT5_C =>
					SPIMACHINE_STATE <= FSM_BIT5_D;				
					POTI_SDI <= Value_Copy(5);
					
				when FSM_BIT5_D =>
					SPIMACHINE_STATE <= FSM_BIT4_C;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(5);
					
				when FSM_BIT4_C =>
					SPIMACHINE_STATE <= FSM_BIT4_D;				
					POTI_SDI <= Value_Copy(4);
					
				when FSM_BIT4_D =>
					SPIMACHINE_STATE <= FSM_BIT3_C;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(4);
					
				when FSM_BIT3_C =>
					SPIMACHINE_STATE <= FSM_BIT3_D;				
					POTI_SDI <= Value_Copy(3);
					
				when FSM_BIT3_D =>
					SPIMACHINE_STATE <= FSM_BIT2_C;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(3);
					
				when FSM_BIT2_C =>
					SPIMACHINE_STATE <= FSM_BIT2_D;
					POTI_SDI <= Value_Copy(2);
					
				when FSM_BIT2_D =>
					SPIMACHINE_STATE <= FSM_BIT1_C;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(2);
					
				when FSM_BIT1_C =>
					SPIMACHINE_STATE <= FSM_BIT1_D;				
					POTI_SDI <= Value_Copy(1);
					
				when FSM_BIT1_D =>
					SPIMACHINE_STATE <= FSM_BIT0_C;				
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(1);
					
				when FSM_BIT0_C =>
					SPIMACHINE_STATE <= FSM_BIT0_D;				
					POTI_SDI <= Value_Copy(0);
					
				when FSM_BIT0_D =>
					SPIMACHINE_STATE <= FSM_COOLDOWN_C;
					POTI_CLK <= '1';
					POTI_SDI <= Value_Copy(0);
					
				when FSM_COOLDOWN_C =>
					SPIMACHINE_STATE <= FSM_COOLDOWN_D;				
				when FSM_COOLDOWN_D =>
					SPIMACHINE_STATE <= FSM_PAUSE_E;
				when FSM_PAUSE_E =>	
					POTI_NOT_CS <= '1';
					SPIMACHINE_STATE <= FSM_IDLE;
				when others =>
					SPIMACHINE_STATE <= FSM_RESET;
			end case;
		end if;
	end process FSM_PROCESS;

end Behavioral;

