----------------------------------------------------------------------------------
-- Company: ITP - Uni Bremen
-- Engineer: David Rotermund
-- 
-- Create Date:    17:59:22 03/05/2011 
-- Design Name: 
-- Module Name:    ADC_24Bit_Datacollector - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity ADC_24Bit_Datacollector is
    Port ( 
				
				-- FROM THE ADC
				DATA_IN 									: in STD_LOGIC;
				NOT_DATAREADY 							: in STD_LOGIC;
				
				-- TO THE OUTPUT FIFO
				-- Data from ADC to Output Fifo is valid (=1)
				OUTPUT_FIFO_WRITEENABLE				: out STD_LOGIC;										
				-- Data from ADC to Output Fifo
				OUTPUT_FIFO_DATAOUT					: out STD_LOGIC_VECTOR(16 downto 0);			
				
				Clk_OUT 									: out  STD_LOGIC;

				Reset 									: in STD_LOGIC;
				-- (32000 Samples pro sec * 24 bit *16 Channel + Time for Sampling > ~14 MHz)
				Clk 										: in  STD_LOGIC 
	 );
	 
end ADC_24Bit_Datacollector;

architecture Behavioral of ADC_24Bit_Datacollector is


	-- The states of the state machine
	type STATE_Type is (
		SM_INIT,
		SM_IDLE,
		SM_NEXT_BIT_A,
		SM_NEXT_BIT_B
	);
	signal SPIMACHINE_STATE : STATE_TYPE;
	
	signal BitToTransmit : std_logic_vector(23 downto 0) := (others => '0');
	signal BitToTransmit_Buffer : std_logic_vector(23 downto 0) := (others => '0');
	signal NewSampleReady : std_logic := '0';
	signal Counter : unsigned (7 downto 0) := (others => '0');
	
	signal Last_NOT_DATAREADY : std_logic := '1';
	
	constant MaxChannels : unsigned (7 downto 0) := to_unsigned(16,8);
	signal ChannelCounter : unsigned (7 downto 0) := (others => '0');
	signal LastSample : std_logic := '0';
	signal LastSample_Buffer : std_logic := '0';
	signal FirstPart : std_logic := '0';
	
begin

	ADC_Readout_Machine : process (Clk,Reset)
	begin
		
		if Reset = '1' then

			SPIMACHINE_STATE <= SM_INIT;

			BitToTransmit <= (others => '0');
			BitToTransmit_Buffer <= (others => '0');
			Counter <= (others => '0');

			Last_NOT_DATAREADY <= '1';

			Clk_OUT <= '0';

			ChannelCounter <= (others => '0');
			NewSampleReady <= '0';
			
			LastSample_Buffer <= '0';
			
		elsif rising_edge(Clk) then
		
			Last_NOT_DATAREADY <= NOT_DATAREADY;

			BitToTransmit <= BitToTransmit;
			BitToTransmit_Buffer <= BitToTransmit_Buffer;
			Counter <= Counter;			

			Clk_OUT <= '0';						

			ChannelCounter <= ChannelCounter;
			NewSampleReady <= '0';
			LastSample_Buffer <= LastSample_Buffer;
			
			case SPIMACHINE_STATE is
				when SM_INIT =>
					SPIMACHINE_STATE <= SM_IDLE;
					BitToTransmit <= (others => '0');
					Counter <= (others => '0');
					LastSample_Buffer <= '0';
					
				when SM_IDLE =>
				
					if (Last_NOT_DATAREADY = '1') AND (NOT_DATAREADY = '0') then
						SPIMACHINE_STATE <= SM_NEXT_BIT_A;
					else
						SPIMACHINE_STATE <= SM_IDLE;
					end if;
					Counter <= (others => '0');
					BitToTransmit <= (others => '0');
					ChannelCounter <= (others => '0');
					
					
				when SM_NEXT_BIT_A =>
					Clk_OUT <= '1';
					if ChannelCounter < MaxChannels then
						SPIMACHINE_STATE <= SM_NEXT_BIT_B;
					else
						SPIMACHINE_STATE <= SM_IDLE;
					end if;
					
					Counter <= Counter;
					
				when SM_NEXT_BIT_B =>	
				
					if NOT_DATAREADY = '0' then		
						SPIMACHINE_STATE <= SM_NEXT_BIT_A;
					else
						SPIMACHINE_STATE <= SM_IDLE;
					end if;
					
					if Counter < to_unsigned(23,8) then
						BitToTransmit <= BitToTransmit(22 downto 0) & DATA_IN;
						Counter <= Counter + 1;
					else
						BitToTransmit_Buffer <= BitToTransmit(22 downto 0) & DATA_IN;
						NewSampleReady <= '1';
						BitToTransmit <= (others => '0');
						Counter <= to_unsigned(0,8);
						ChannelCounter <= ChannelCounter + 1;
					end if;
					LastSample_Buffer <= LastSample;
					
				when others =>
					SPIMACHINE_STATE <= SM_INIT;
					BitToTransmit <= (others => '0');
					Counter <= (others => '0');
					ChannelCounter <= (others => '0');
					LastSample_Buffer <= '0';
			end case;
			
		end if;
		
	end process ADC_Readout_Machine;

	ShippmentProcess : process (Clk, Reset)
	begin 
		if Reset = '1' then
			OUTPUT_FIFO_DATAOUT <= (others => '0');
			OUTPUT_FIFO_WRITEENABLE <= '0';
			FirstPart <= '0';

		elsif rising_edge(Clk) then
		
			if NewSampleReady = '1' then
				OUTPUT_FIFO_DATAOUT <= "0" 
					& BitToTransmit_Buffer(23) &BitToTransmit_Buffer(23) 
					& BitToTransmit_Buffer(23) &BitToTransmit_Buffer(23) 
					& BitToTransmit_Buffer(23) &BitToTransmit_Buffer(23) 
					& BitToTransmit_Buffer(23) &BitToTransmit_Buffer(23) 
					& BitToTransmit_Buffer(23 downto 16);
					
				OUTPUT_FIFO_WRITEENABLE <= '1';
				FirstPart <= '1';
				
			elsif FirstPart = '1' then
			
				OUTPUT_FIFO_DATAOUT <= LastSample_Buffer & BitToTransmit_Buffer(15 downto 0);
				OUTPUT_FIFO_WRITEENABLE <= '1';
				FirstPart <= '0';
			else
				OUTPUT_FIFO_DATAOUT <= (others => '0');
				OUTPUT_FIFO_WRITEENABLE <= '0';
				FirstPart <= '0';
			end if;
			
		end if;
	
	end process ShippmentProcess;
	
	LastSampleProcess : process (ChannelCounter)
	begin
		if ChannelCounter = to_unsigned(15,8) then
			LastSample <= '1';
		else
			LastSample <= '0';
		end if;
	end process LastSampleProcess;

end Behavioral;

