library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

 
ENTITY ADC_24Bit_Datacollector_UUT IS
END ADC_24Bit_Datacollector_UUT;
 
ARCHITECTURE behavior OF ADC_24Bit_Datacollector_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT ADC_24Bit_Datacollector
    PORT(
         DATA_IN : IN  std_logic;
         NOT_DATAREADY : IN  std_logic;
         OUTPUT_FIFO_WRITEENABLE : OUT  std_logic;
         OUTPUT_FIFO_DATAOUT : OUT  std_logic_vector(16 downto 0);
         Clk_OUT : OUT  std_logic;
         Reset : IN  std_logic;
         Clk : IN  std_logic
        );
    END COMPONENT;
    

   --Inputs
   signal DATA_IN : std_logic := '1';
   signal NOT_DATAREADY : std_logic := '0';
   signal Reset : std_logic := '0';
   signal Clk : std_logic := '0';

 	--Outputs
   signal OUTPUT_FIFO_WRITEENABLE : std_logic;
   signal OUTPUT_FIFO_DATAOUT : std_logic_vector(16 downto 0);
   signal Clk_OUT : std_logic;

  	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 
	signal SlowClock : std_logic := '0';

	signal Clk_Greater14MHz : std_logic := '0';
	signal Clock_Counter : unsigned (5 downto 0) := (others => '0');	
	
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: ADC_24Bit_Datacollector PORT MAP (
          DATA_IN => DATA_IN,
          NOT_DATAREADY => NOT_DATAREADY,
          OUTPUT_FIFO_WRITEENABLE => OUTPUT_FIFO_WRITEENABLE,
          OUTPUT_FIFO_DATAOUT => OUTPUT_FIFO_DATAOUT,
          Clk_OUT => Clk_OUT,
          Reset => Reset,
          Clk => Clk
        );

     -- Clock process definitions
TheMasterClock : process
begin
	clockloop: for i in 0 to 50000 loop
		clk <= NOT clk;
		wait for 4ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

TheSlowClock : process(SlowClock) 
begin
	if rising_edge(SlowClock) then
		DATA_IN <= not DATA_IN;
	end if;
end process TheSlowClock;

	MyClock14MHz : process (Clk, Reset)
	begin 

		if Reset = '1' then
			Clk_Greater14MHz <= '0';
			Clock_Counter <= (others => '0');
			
		elsif rising_edge(Clk) then
			
			Clk_Greater14MHz <= Clk_Greater14MHz;
			
			if Clock_Counter < to_unsigned(5,6) then
				Clock_Counter <= Clock_Counter + 1;
			else
				Clock_Counter <= (others => '0');
				Clk_Greater14MHz <= not Clk_Greater14MHz;
			end if;
			
		end if;
	end process MyClock14MHz;

TheSecondClock : process(clk) 
begin
	if rising_edge(clk) then
		
		SlowClock <= not SlowClock;

		MasterCounter <= MasterCounter + 1;
		
		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
		else
			Reset <= '0';
		end if;
		
		NOT_DATAREADY <= '0';
		
		if MasterCounter = to_unsigned(10,128) then
			null;
		elsif MasterCounter = to_unsigned(11,128) then
			NOT_DATAREADY <= '1';
		elsif MasterCounter = to_unsigned(12,128) then
			NOT_DATAREADY <= '1';
		elsif MasterCounter = to_unsigned(13,128) then
			NOT_DATAREADY <= '1';
		elsif MasterCounter = to_unsigned(14,128) then
			NOT_DATAREADY <= '1';
		else
			null;
		end if;

	end if;
end process TheSecondClock;

END;
