library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity BiStableSwitchPolling is
    Port ( 		
	 
		-- Enable / Disable
		Enable 						: in  STD_LOGIC;			
		Disable						: in  STD_LOGIC;
		
		-- Result 
		PollingIsEnabled			: out STD_LOGIC;
		
		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	);
end BiStableSwitchPolling;

architecture Behavioral of BiStableSwitchPolling is

	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State (is disabled)
		STATE_IDLE,
		-- Is Enabled 
		STATE_ENABLED
		);
	signal FSM_STATE : STATE_TYPE;

	signal Enable_Copy : STD_LOGIC := '0';			
	signal Disable_Copy : STD_LOGIC := '0';	

begin
	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			FSM_STATE <= STATE_RESET;
			PollingIsEnabled <= '0';
			Enable_Copy <= '0';
			Disable_Copy <= '0';
			
		elsif rising_edge(Clk) then

				Enable_Copy <= Enable;
				Disable_Copy <= Disable;
			
				case FSM_STATE is
					when STATE_RESET => 		

						FSM_STATE <= STATE_IDLE;
						PollingIsEnabled <= '0';

					when STATE_IDLE =>

						if Enable_Copy = '1' then 
							FSM_STATE <= STATE_ENABLED;
							PollingIsEnabled <= '1';
						else
							FSM_STATE <= STATE_IDLE;
							PollingIsEnabled <= '0';
						end if;
						
					when STATE_ENABLED =>
					
						if Disable_Copy = '1' then 
							FSM_STATE <= STATE_IDLE;
							PollingIsEnabled <= '0';
						else
							FSM_STATE <= STATE_ENABLED;
							PollingIsEnabled <= '1';
						end if;
					
					when others => 
						FSM_STATE <= STATE_RESET;
						PollingIsEnabled <= '0';
				end case;
		end if;
	end process FSM;

end Behavioral;

