library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity CRC_Checker is
	Port ( 
		Network_Data				: in STD_LOGIC_VECTOR(15 downto 0); 
		Network_Valid				: in STD_LOGIC; 

		CRC_Reset					: in STD_LOGIC;
		CRC_00						: out 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_01						: out 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_10						: out 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_11						: out 	STD_LOGIC_VECTOR(15 downto 0);			
		
		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	);
		
end CRC_Checker;

architecture Behavioral of CRC_Checker is

	-- CRC 
	signal CRC_00_Internal : unsigned (15 downto 0) := (others => '0');
	signal CRC_01_Internal : unsigned (15 downto 0) := (others => '0');
	signal CRC_10_Internal : unsigned (15 downto 0) := (others => '0');
	signal CRC_11_Internal : unsigned (15 downto 0) := (others => '0');

	-- CRC SubRoutine Calculations -- Detect: 
	signal CRC_00_Sub : unsigned (7 downto 0) := (others => '0');
	signal CRC_01_Sub : unsigned (7 downto 0) := (others => '0');
	signal CRC_10_Sub : unsigned (7 downto 0) := (others => '0');
	signal CRC_11_Sub : unsigned (7 downto 0) := (others => '0');

	-- CRC SubRoutine Calculations -- Count: 
	signal CRC_00_Count : unsigned (3 downto 0) := (others => '0');
	signal CRC_01_Count : unsigned (3 downto 0) := (others => '0');
	signal CRC_10_Count : unsigned (3 downto 0) := (others => '0');
	signal CRC_11_Count : unsigned (3 downto 0) := (others => '0');

begin

	CRC_00 <= std_logic_vector(CRC_00_Internal);
	CRC_01 <= std_logic_vector(CRC_01_Internal);	
	CRC_10 <= std_logic_vector(CRC_10_Internal);
	CRC_11 <= std_logic_vector(CRC_11_Internal);	

	CRC_Module : process(Reset,Clk)
	begin
		if (Reset = '1') then
			CRC_00_Internal <= (others => '0');
			CRC_01_Internal <= (others => '0');
			CRC_10_Internal <= (others => '0');
			CRC_11_Internal <= (others => '0');
		
		elsif rising_edge(Clk) then

			CRC_00_Internal <= CRC_00_Internal;
			CRC_01_Internal <= CRC_01_Internal;
			CRC_10_Internal <= CRC_10_Internal;
			CRC_11_Internal <= CRC_11_Internal;

			if CRC_Reset = '1' then
			
				CRC_00_Internal <= (others => '0');
				CRC_01_Internal <= (others => '0');
				CRC_10_Internal <= (others => '0');
				CRC_11_Internal <= (others => '0');
			else
				CRC_00_Internal <= CRC_00_Internal + resize(CRC_00_Count,16);
				CRC_01_Internal <= CRC_01_Internal + resize(CRC_01_Count,16);
				CRC_10_Internal <= CRC_10_Internal + resize(CRC_10_Count,16);
				CRC_11_Internal <= CRC_11_Internal + resize(CRC_11_Count,16);				
			end if;
		end if;
	end process CRC_Module;
	
	MyCRC_SubRoutine : process(Reset,Clk)
	begin
		if (Reset = '1') then		

			-- Default: 
			CRC_00_Sub <= (others => '0');
			CRC_01_Sub <= (others => '0');
			CRC_10_Sub <= (others => '0');
			CRC_11_Sub <= (others => '0');

		elsif rising_edge(Clk) then
			-- Default: 
			
			CRC_00_Sub <= (others => '0');
			CRC_01_Sub <= (others => '0');
			CRC_10_Sub <= (others => '0');
			CRC_11_Sub <= (others => '0');
			
			if Network_Valid = '1' then
				-- Bit 0 & 1 
				case Network_Data(1 downto 0) is
					when "00" =>
						CRC_00_Sub(0) <= '1';
					when "01" =>
						CRC_01_Sub(0) <= '1';
					when "10" =>
						CRC_10_Sub(0) <= '1';
					when "11" =>
						CRC_11_Sub(0) <= '1';
					when others => 
						null;
				end case;

				-- Bit 2 & 3
				case Network_Data(3 downto 2) is
					when "00" =>
						CRC_00_Sub(1) <= '1';
					when "01" =>
						CRC_01_Sub(1) <= '1';
					when "10" =>
						CRC_10_Sub(1) <= '1';
					when "11" =>
						CRC_11_Sub(1) <= '1';
					when others => 
						null;
				end case;

				-- Bit 4 & 5
				case Network_Data(5 downto 4) is
					when "00" =>
						CRC_00_Sub(2) <= '1';
					when "01" =>
						CRC_01_Sub(2) <= '1';
					when "10" =>
						CRC_10_Sub(2) <= '1';
					when "11" =>
						CRC_11_Sub(2) <= '1';
					when others => 
						null;
				end case;

				-- Bit 6 & 7
				case Network_Data(7 downto 6) is
					when "00" =>
						CRC_00_Sub(3) <= '1';
					when "01" =>
						CRC_01_Sub(3) <= '1';
					when "10" =>
						CRC_10_Sub(3) <= '1';
					when "11" =>
						CRC_11_Sub(3) <= '1';
					when others => 
						null;
				end case;

				-- Bit 8 & 9
				case Network_Data(9 downto 8) is
					when "00" =>
						CRC_00_Sub(4) <= '1';
					when "01" =>
						CRC_01_Sub(4) <= '1';
					when "10" =>
						CRC_10_Sub(4) <= '1';
					when "11" =>
						CRC_11_Sub(4) <= '1';
					when others => 
						null;
				end case;

				-- Bit 10 & 11
				case Network_Data(11 downto 10) is
					when "00" =>
						CRC_00_Sub(5) <= '1';
					when "01" =>
						CRC_01_Sub(5) <= '1';
					when "10" =>
						CRC_10_Sub(5) <= '1';
					when "11" =>
						CRC_11_Sub(5) <= '1';
					when others => 
						null;
				end case;

				-- Bit 12 & 13
				case Network_Data(13 downto 12) is
					when "00" =>
						CRC_00_Sub(6) <= '1';
					when "01" =>
						CRC_01_Sub(6) <= '1';
					when "10" =>
						CRC_10_Sub(6) <= '1';
					when "11" =>
						CRC_11_Sub(6) <= '1';
					when others => 
						null;
				end case;

				-- Bit 14 & 15
				case Network_Data(15 downto 14) is
					when "00" =>
						CRC_00_Sub(7) <= '1';
					when "01" =>
						CRC_01_Sub(7) <= '1';
					when "10" =>
						CRC_10_Sub(7) <= '1';
					when "11" =>
						CRC_11_Sub(7) <= '1';
					when others => 
						null;
				end case;
			end if;
		end if;
	end process MyCRC_SubRoutine;

	MyCRC_CountRoutine : process(Reset,Clk)
		variable CRC_00_Var : unsigned (3 downto 0) := (others => '0');
		variable CRC_01_Var : unsigned (3 downto 0) := (others => '0');
		variable CRC_10_Var : unsigned (3 downto 0) := (others => '0');
		variable CRC_11_Var : unsigned (3 downto 0) := (others => '0');	

	begin
		if (Reset = '1') then		

			-- Default: 
			CRC_00_Var := (others => '0');
			CRC_01_Var := (others => '0');
			CRC_10_Var := (others => '0');
			CRC_11_Var := (others => '0');
			
			-- Export the CRC values calculation
			CRC_00_Count <= CRC_00_Var;
			CRC_01_Count <= CRC_01_Var;
			CRC_10_Count <= CRC_10_Var;
			CRC_11_Count <= CRC_11_Var;
		elsif rising_edge(Clk) then

			-- CRC : In the beginning was nothing... 
			CRC_00_Var := (others => '0');
			CRC_01_Var := (others => '0');
			CRC_10_Var := (others => '0');
			CRC_11_Var := (others => '0');
			
			if CRC_Reset = '1' then
				NULL;
			else
			
				-- 00
				for i in 7 downto 0 loop
					if CRC_00_Sub(i) = '1' then
						CRC_00_Var := CRC_00_Var + 1;
					else
						null;
					end if;
				end loop;

				-- 01
				for i in 7 downto 0 loop
					if CRC_01_Sub(i) = '1' then
						CRC_01_Var := CRC_01_Var + 1;
					else
						null;
					end if;
				end loop;

				-- 10
				for i in 7 downto 0 loop
					if CRC_10_Sub(i) = '1' then
						CRC_10_Var := CRC_10_Var + 1;
					else
						null;
					end if;
				end loop;

				-- 11
				for i in 7 downto 0 loop
					if CRC_11_Sub(i) = '1' then
						CRC_11_Var := CRC_11_Var + 1;
					else
						null;
					end if;
				end loop;
			end if;
			
			-- Export the CRC values calculation
			CRC_00_Count <= CRC_00_Var;
			CRC_01_Count <= CRC_01_Var;
			CRC_10_Count <= CRC_10_Var;
			CRC_11_Count <= CRC_11_Var;
		end if;
	end process MyCRC_CountRoutine;


end architecture Behavioral;

