----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:48:56 01/01/2012 
-- Design Name: 
-- Module Name:    ChannelMaskCount - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity ChannelMaskCount is
	Generic (
		-- Upper Limit of the Implant_ChannelMask array
		MaskLength : integer := 127;
		-- Upper Limit of the Count array
		CountLength : integer := 7
	);
    Port ( 
	 
		Count							: out STD_LOGIC_VECTOR(CountLength downto 0);
		Ready							: out STD_LOGIC;
		
		Implant_ChannelMask 		: in std_logic_vector(MaskLength downto 0);	 
		
		
		Clk 							: in  STD_LOGIC;
		Reset 						: in  STD_LOGIC
	 );
end ChannelMaskCount;

architecture Behavioral of ChannelMaskCount is

	signal Implant_ChannelMask_Copy 	: std_logic_vector(MaskLength downto 0) := (others => '0');
	signal Implant_ChannelMask_Temp 	: std_logic_vector(MaskLength downto 0) := (others => '0');
	signal Counter : unsigned(CountLength downto 0) := (others => '0');
	signal Count_Internal : unsigned(CountLength downto 0) := (others => '0');
	

	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		STATE_COUNT
	);
	signal FSM_STATE : STATE_TYPE;	
	
	
begin
	Count <= std_logic_vector(Count_Internal);

	FSM : process(Reset,Clk)
	begin
	
		if (Reset = '1') then
		
			Implant_ChannelMask_Copy <= (others => '0');
			Implant_ChannelMask_Temp <= (others => '0');
			Counter <= (others => '0');
			Count_Internal <= (others => '0');
			Ready <= '0';
			
		elsif rising_edge(Clk) then
		
			-- DEFAULT: 
			Implant_ChannelMask_Copy <= Implant_ChannelMask_Copy;
			Implant_ChannelMask_Temp <= Implant_ChannelMask_Temp;
			Counter <= Counter;
			Count_Internal <= Count_Internal;
			Ready <= '1';
			
			case FSM_STATE is
				-- After reset...
				when STATE_RESET =>
					FSM_STATE <= STATE_IDLE;
					Ready <= '0';
					
				when STATE_IDLE =>
					if Implant_ChannelMask = Implant_ChannelMask_Copy then
						FSM_STATE <= STATE_IDLE;
					else
						FSM_STATE <= STATE_COUNT;
						Implant_ChannelMask_Copy <= Implant_ChannelMask;
						Count_Internal <= (others => '0');
						Ready <= '0';
					end if;
					Implant_ChannelMask_Temp <= Implant_ChannelMask;
					Counter <= to_unsigned(MaskLength+1,CountLength+1);
					
				when STATE_COUNT => 

					Ready <= '0';
					FSM_STATE <= STATE_COUNT;
					Counter <= Counter - 1;

					Implant_ChannelMask_Temp <= "0" & Implant_ChannelMask_Temp(MaskLength downto 1);
					
					if Counter = to_unsigned(0,CountLength+1) then
						FSM_STATE <= STATE_IDLE;
					elsif Implant_ChannelMask_Temp(0) = '1' then
						Count_Internal <= Count_Internal + 1;
					else
						NULL;
					end if;
					
			end case;
		end if;
	end process FSM;
	
end Behavioral;

