library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity ClockForPolling is
    Port ( 
	 
		-- Constant with maximum counter value
		Zarlink_CheckBuffer 		: in  STD_LOGIC_VECTOR(31 downto 0);

		-- Start Counting Process
		RestartClock				: in	STD_LOGIC;
		
		-- The results of the counting process 
		ClockEvent					: out STD_LOGIC;
		
		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	);
end ClockForPolling;

architecture Behavioral of ClockForPolling is

	signal Zarlink_CheckBuffer_Counter : unsigned (32 downto 0) := (others => '0');
	
begin
	
	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
		
			Zarlink_CheckBuffer_Counter <= (others => '0');
			ClockEvent <= '0';
			
		elsif rising_edge(Clk) then
		
			if RestartClock = '1' then
				Zarlink_CheckBuffer_Counter <= (others => '0');
				ClockEvent <= '0';
			elsif Zarlink_CheckBuffer_Counter < unsigned("0" & Zarlink_CheckBuffer) then
				Zarlink_CheckBuffer_Counter <= Zarlink_CheckBuffer_Counter + 1;
				ClockEvent <= '0';
			else
				Zarlink_CheckBuffer_Counter <= Zarlink_CheckBuffer_Counter;
				ClockEvent <= '1';
			end if;
			
		end if;
	end process FSM;	

end architecture Behavioral;

