library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
-- Errors:
-- 0: No Error
-- 1: Input Data too long
-- 2: Input Data too short
-- 3: Strange state related to the input data 
-- 4: CRC Error
-- 5: Zarlink Timeout Error
-- 10 : TimeOut - Had problems with the tx buffer
-- 11 : TimeOut - Had problems with the rx buffer
-- 5XX: RF Connection Timeout 
-- 500: STATE_SPI_SPEED_A
-- 501: STATE_WATCHDOG_OFF
-- 502: STATE_SPI_SPEED_B
-- 503: STATE_WAIT_IRQ_RADIOREADY_A_REQ
--	504: STATE_WAIT_IRQ_RADIOREADY_A_WAIT,
--	505: STATE_REG_MAC_MODUSER,
--	506: STATE_REG_MAC_CHANNEL,
--	507: STATE_REG_REG_TXBUFFER_BSIZE_A,
--	508: STATE_REG_REG_RXBUFFER_BSIZE_A, 
--	509: STATE_REG_TXBUFF_MAXPACKSIZE_A, 
--	510: STATE_IMD_ID_A,
--	511: STATE_IMD_ID_B,
--	512: STATE_IMD_ID_C,
--	513: STATE_IRQ_ENABLESET1,
--	514: STATE_REG_MAC_INITCOM_A, 
--	515: STATE_WAIT_IRQ_RXNOTEMPTY_REQ,
--	516: STATE_WAIT_IRQ_RXNOTEMPTY_WAIT,
--	517: STATE_READ_BLOCK_REQ,
--	518: STATE_READ_BLOCK_WAIT,
--	519: STATE_READ_BLOCK_WAIT - Zarlink Timeout,
--	520: STATE_REG_MAC_INITCOM_B, 
--	521: STATE_REG_MAC_CTRL_A, 
--	522: STATE_WAIT_IRQ_RADIOREADY_B_REQ, 
--	523: STATE_WAIT_IRQ_RADIOREADY_B_WAIT,
--	524: STATE_REG_MAX_RESENDTIME,
--	525: STATE_REG_MAC_CTRL_B, 
--	526: STATE_REG_MAC_CTRL_C, 
--	527: STATE_WAIT_IRQ_COMMANDDONE_REQ, 
--	528: STATE_WAIT_IRQ_COMMANDDONE_WAIT,
--	529: STATE_REG_REG_TXBUFFER_BSIZE_B,
--	530: STATE_REG_REG_RXBUFFER_BSIZE_B, 
--	531: STATE_REG_TXBUFF_MAXPACKSIZE_B, 
--	532: STATE_REG_MAC_INITCOM_C, 
--	533: STATE_WAIT_IRQ_LINKREADY_REQ,
--	534: STATE_WAIT_IRQ_LINKREADY_WAIT,
--	535: STATE_IMD_COPY_ID_A,
--	536: STATE_IMD_COPY_ID_B,
--	537: STATE_IMD_COPY_ID_C,
-- 538: STATE_REG_MAC_INITCOM_A_IMD,

entity Command_ZarlinkConnect is
	 Generic (
		Module_Name : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
		Module_HasMemory : STD_LOGIC := '1';
		-- Needs to be at least 1 => 
		Module_LengthInWord : integer := 1;
		Module_UsesZarlink : STD_LOGIC := '1'
	 );
    Port ( 
		-- Connection to the first Input-Filter-Layer 
		In_Data						: in STD_LOGIC_VECTOR(15 downto 0);
		In_Valid 					: in STD_LOGIC;
		In_Wait						: out STD_LOGIC;
		
		-- More information about the packets 
		Packet_Start				: in STD_LOGIC;
		Packet_End					: in STD_LOGIC;
		Packet_Valid				: in STD_LOGIC;
		Packet_Unknown				: in STD_LOGIC;
		
		-- Connection to the Zarlink 
		Zarlink_Data_Out			: out STD_LOGIC_VECTOR(119 downto 0); -- Tri-State
		Zarlink_WriteEn			: out STD_LOGIC; -- Tri-State
		Zarlink_Data_In			: in 	STD_LOGIC_VECTOR(111 downto 0);
		Zarlink_Valid_In			: in 	STD_LOGIC;
		Zarlink_BlockMode			: out	STD_LOGIC; -- Tri-State 
		Zarlink_Wait				: in  STD_LOGIC;
		Zarlink_RequestChannel  : out STD_LOGIC;
		Zarlink_GetChannel		: in  STD_LOGIC;
		Zarlink_Name				: out STD_LOGIC_VECTOR (15 downto 0); -- Tri-State
		Zarlink_Error 				: in	STD_LOGIC;
				
		-- Connection to the Network
		Network_Data				: out STD_LOGIC_VECTOR(15 downto 0); -- Tri-State 
		Network_WriteEn			: out STD_LOGIC; -- Tri-State
		Network_Wait				: in  STD_LOGIC;
		Network_RequestChannel  : out STD_LOGIC; 
		Network_GetChannel		: in  STD_LOGIC;
		CRC_Reset					: out STD_LOGIC;
		CRC_00						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_01						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_10						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_11						: in 	STD_LOGIC_VECTOR(15 downto 0);

		Zarlink_Implant_Mode		: in STD_LOGIC;
		
		Zarlink_Connection_Mode : in STD_LOGIC_VECTOR(3 downto 0);
		Zarlink_SPISpeedFlags 	: in STD_LOGIC_VECTOR(1 downto 0);
		-- Constants 
		Zarlink_ConnectionTimeout : in STD_LOGIC_VECTOR (47 downto 0);
		Zarlink_WaitForReset 	: in STD_LOGIC_VECTOR(27 downto 0);
		Zarlink_Retry 				: in STD_LOGIC_vector(18 downto 0);

		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	 );
end Command_ZarlinkConnect;

architecture Behavioral of Command_ZarlinkConnect is

-- Handle the Tri-States 	
	-- Zarlink Outputs
	signal Zarlink_Data_Out_Internal	: STD_LOGIC_VECTOR(119 downto 0) := (others => '0');
	signal Zarlink_WriteEn_Internal : STD_LOGIC := '0';
	signal Zarlink_BlockMode_Internal : STD_LOGIC := '0';
	
	-- Network Outputs
	signal Network_Data_Internal : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal Network_WriteEn_Internal : STD_LOGIC := '0';
	signal Network_GetChannel_Internal : STD_LOGIC := '0';
	signal CRC_Reset_Internal : STD_LOGIC := '0';
	-- Internal Memory 
	signal Input_Memory : STD_LOGIC_VECTOR ((Module_LengthInWord*16)-1 downto 0) := (others => '0');
	signal Input_WordCounter : unsigned (15 downto 0) := (others => '0');
	
	signal ErrorState : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');

	signal CRC_01_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_10_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_11_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
	-- 0x55AA
	constant NET_HEADER : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(21930,16));
	-- 0x8000
	constant NET_RESPONSEID : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(32768,16));
	-- Not constant because the length can change 
	signal NET_LENGTH : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		-- The Packet Starts and we acquire the data from the stream
		STATE_GETDATA,
		-- Wait for the valid check 
		STATE_INPUTDATA_VALID,
		-- Wait until the power up reset is done
		STATE_FIRST_RESET,
		-- Request the Zalink Channel
		STATE_ZARLINK_PREPARE,
		-- Comunicate with the Zarlink
		STATE_ZARLINK_COMMUNICATE,
		-- Wait until the software reset is done
		STATE_SECOND_RESET,
		-- SET SPI SPEED
		STATE_SPI_SPEED_A,
		-- Let's disable the watchdog
		STATE_WATCHDOG_OFF, 
		-- SET SPI SPEED (again) We don't trust the watchdog... 
		STATE_SPI_SPEED_B,		
		-- Wait for the radio ready flag 
		STATE_WAIT_IRQ_RADIOREADY_A_REQ,
		STATE_WAIT_IRQ_RADIOREADY_A_WAIT,
		STATE_WAIT_IRQ_RADIOREADY_A_PAUSE,
		STATE_REG_MAC_MODUSER,
		STATE_REG_MAC_CHANNEL,
		STATE_REG_REG_TXBUFFER_BSIZE_A,
		STATE_REG_REG_RXBUFFER_BSIZE_A, 
		STATE_REG_TXBUFF_MAXPACKSIZE_A, 
		STATE_IMD_COPY_ID_A,
		STATE_IMD_COPY_ID_B,
		STATE_IMD_COPY_ID_C,
		STATE_IMD_ID_A,
		STATE_IMD_ID_B,
		STATE_IMD_ID_C,
		STATE_REG_MAC_INITCOM_A_IMD,
		STATE_IRQ_ENABLESET1,
		STATE_REG_MAC_INITCOM_A, 
		STATE_WAIT_IRQ_RXNOTEMPTY_REQ,
		STATE_WAIT_IRQ_RXNOTEMPTY_WAIT,
		STATE_WAIT_IRQ_RXNOTEMPTY_PAUSE,
		STATE_READ_BLOCK_REQ,
		STATE_READ_BLOCK_WAIT,
		STATE_REG_MAC_INITCOM_B, 
		STATE_REG_MAC_CTRL_A, 
		STATE_WAIT_IRQ_RADIOREADY_B_REQ, 
		STATE_WAIT_IRQ_RADIOREADY_B_WAIT,
		STATE_WAIT_IRQ_RADIOREADY_B_PAUSE,
		STATE_REG_MAX_RESENDTIME,
		STATE_REG_MAC_CTRL_B, 
		STATE_REG_MAC_CTRL_C, 
		STATE_WAIT_IRQ_COMMANDDONE_REQ, 
		STATE_WAIT_IRQ_COMMANDDONE_WAIT,
		STATE_WAIT_IRQ_COMMANDDONE_PAUSE,
		STATE_REG_REG_TXBUFFER_BSIZE_B,
		STATE_REG_REG_RXBUFFER_BSIZE_B, 
		STATE_REG_TXBUFF_MAXPACKSIZE_B, 
		STATE_REG_MAC_INITCOM_C, 
		STATE_WAIT_IRQ_LINKREADY_REQ,
		STATE_WAIT_IRQ_LINKREADY_WAIT,
		STATE_WAIT_IRQ_LINKREADY_PAUSE,
		-- Prepare the network for transmission
		STATE_NETWORK_PREPARE,
		-- Communicate with the network (HEADER)
		STATE_NETWORK_COMMUNICATE_HEADER,
		-- Communicate with the network (LENGTH)
		STATE_NETWORK_COMMUNICATE_LENGTH,
		-- Communicate with the network (RESPONSEID)
		STATE_NETWORK_COMMUNICATE_RESPONSEID,
		-- Communicate with the network (ID)
		STATE_NETWORK_COMMUNICATE_ID,
		-- Communicate with the network (ERROR)
		STATE_NETWORK_COMMUNICATE_ERROR,
		-- Communicate with the network (BODY)
		STATE_NETWORK_COMMUNICATE_BODY,
		-- Communicate with the network - WAIT A
		STATE_NETWORK_COMMUNICATE_WAIT_A,
		-- Communicate with the network - WAIT B
		STATE_NETWORK_COMMUNICATE_WAIT_B,		
		-- Communicate with the network - WAIT C
		STATE_NETWORK_COMMUNICATE_WAIT_C,		
		-- Communicate with the network (CRC 00)
		STATE_NETWORK_COMMUNICATE_CRC00,
		-- Communicate with the network (CRC 01)
		STATE_NETWORK_COMMUNICATE_CRC01,
		-- Communicate with the network (CRC 10)
		STATE_NETWORK_COMMUNICATE_CRC10,
		-- Communicate with the network (CRC 11)
		STATE_NETWORK_COMMUNICATE_CRC11
	);
	signal FSM_STATE : STATE_TYPE;

	signal ZDOR	: STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
	signal Z_RD : STD_LOGIC := '0';
	signal Z_Address : STD_LOGIC_VECTOR(6 downto 0) := (others => '0');
	signal Z_BM : STD_LOGIC := '0';
	signal Z_WE : STD_LOGIC := '0';
	signal Z_Done : STD_LOGIC := '0';
	
	signal Zarlink_ResetCounter : unsigned (28 downto 0) := (others => '0');
	signal Zarlink_ConnectionTimeoutCounter : unsigned (48 downto 0) := (others => '0');
	signal Zarlink_Retry_Counter : unsigned(18 downto 0) := (others => '0');
	
begin
-- Resort the Zarlink_Data_Out_Internal in Zarlink-Fashion
Zarlink_Data_Out_Internal <= 
		 ZDOR(104)&ZDOR(105)&ZDOR(106)&ZDOR(107)&ZDOR(108)&ZDOR(109)&ZDOR(110)&ZDOR(111)
		&ZDOR(96)&ZDOR(97)&ZDOR(98)&ZDOR(99)&ZDOR(100)&ZDOR(101)&ZDOR(102)&ZDOR(103)
		&ZDOR(88)&ZDOR(89)&ZDOR(90)&ZDOR(91)&ZDOR(92)&ZDOR(93)&ZDOR(94)&ZDOR(95)
		&ZDOR(80)&ZDOR(81)&ZDOR(82)&ZDOR(83)&ZDOR(84)&ZDOR(85)&ZDOR(86)&ZDOR(87)
		&ZDOR(72)&ZDOR(73)&ZDOR(74)&ZDOR(75)&ZDOR(76)&ZDOR(77)&ZDOR(78)&ZDOR(79)
		&ZDOR(64)&ZDOR(65)&ZDOR(66)&ZDOR(67)&ZDOR(68)&ZDOR(69)&ZDOR(70)&ZDOR(71)
		&ZDOR(56)&ZDOR(57)&ZDOR(58)&ZDOR(59)&ZDOR(60)&ZDOR(61)&ZDOR(62)&ZDOR(63)
		&ZDOR(48)&ZDOR(49)&ZDOR(50)&ZDOR(51)&ZDOR(52)&ZDOR(53)&ZDOR(54)&ZDOR(55)
		&ZDOR(40)&ZDOR(41)&ZDOR(42)&ZDOR(43)&ZDOR(44)&ZDOR(45)&ZDOR(46)&ZDOR(47)
		&ZDOR(32)&ZDOR(33)&ZDOR(34)&ZDOR(35)&ZDOR(36)&ZDOR(37)&ZDOR(38)&ZDOR(39)
		&ZDOR(24)&ZDOR(25)&ZDOR(26)&ZDOR(27)&ZDOR(28)&ZDOR(29)&ZDOR(30)&ZDOR(31)
		&ZDOR(16)&ZDOR(17)&ZDOR(18)&ZDOR(19)&ZDOR(20)&ZDOR(21)&ZDOR(22)&ZDOR(23)
		&ZDOR(8)&ZDOR(9)&ZDOR(10)&ZDOR(11)&ZDOR(12)&ZDOR(13)&ZDOR(14)&ZDOR(15)
		&ZDOR(0)&ZDOR(1)&ZDOR(2)&ZDOR(3)&ZDOR(4)&ZDOR(5)&ZDOR(6)&ZDOR(7)
		&Z_Address(0)&Z_Address(1)&Z_Address(2)&Z_Address(3)&Z_Address(4)&Z_Address(5)&Z_Address(6)&Z_RD;
Zarlink_BlockMode_Internal	<= Z_BM;
Zarlink_WriteEn_Internal <= Z_WE;

-- Tri - State Buffers: 
-- Zarlink Tri-State Output
Zarlink_Data_Out <= Zarlink_Data_Out_Internal when Zarlink_GetChannel = '1' else (others => 'Z');
Zarlink_WriteEn <= Zarlink_WriteEn_Internal when Zarlink_GetChannel = '1' else 'Z';
Zarlink_BlockMode <= Zarlink_BlockMode_Internal when Zarlink_GetChannel = '1' else 'Z';
Zarlink_Name <= Module_Name when Zarlink_GetChannel = '1' else (others => 'Z');

-- Network Tri-State Output
Network_Data <= Network_Data_Internal when Network_GetChannel_Internal = '1' else (others => 'Z');
Network_WriteEn <= Network_WriteEn_Internal when Network_GetChannel_Internal = '1' else 'Z';
CRC_Reset <= CRC_Reset_Internal when Network_GetChannel_Internal = '1' else 'Z';

	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			-- Default:
				ZDOR <= (others => '0');
				Z_RD <= '0';
				Z_Address <= (others => '0');						
				Z_BM <= '0';
				Z_WE <= '0';
				Z_Done <= '0';
				Zarlink_RequestChannel <= '0';
				
				Network_Data_Internal <= (others => '0');
				Network_WriteEn_Internal <= '0';
				Network_RequestChannel <= '0';
				CRC_Reset_Internal <= '0';
				
				ErrorState <= (others => '0');
				
				In_Wait <= '0';
				Input_Memory <= (others => '0');
				FSM_STATE <= STATE_RESET;
				Input_WordCounter <= (others => '0');

				CRC_01_Copy <= (others => '0');
				CRC_10_Copy <= (others => '0');
				CRC_11_Copy <= (others => '0');
				NET_LENGTH <= std_logic_vector(to_unsigned(18,16));
				Network_GetChannel_Internal <= '0';
				
				Zarlink_ResetCounter <= (others => '0');
				Zarlink_ConnectionTimeoutCounter <= (others => '0');
				Zarlink_Retry_Counter <= (others => '0');
				
		elsif rising_edge(Clk) then
				Network_GetChannel_Internal <= Network_GetChannel;
			-- Default:
				ZDOR <= (others => '0');
				Z_RD <= '0';
				Z_Address <= (others => '0');						
				Z_BM <= '0';
				Z_WE <= '0';
				Z_Done <= Z_Done;
				Zarlink_RequestChannel <= '0';
				
				Network_Data_Internal <= (others => '0');
				Network_WriteEn_Internal <= '0';
				Network_RequestChannel <= '0';
				CRC_Reset_Internal <= '0';
				
				ErrorState <= ErrorState;
				In_Wait <= '0';
				Input_Memory <= Input_Memory;
				Input_WordCounter <= (others => '0');
				CRC_01_Copy <= (others => '0');
				CRC_10_Copy <= (others => '0');
				CRC_11_Copy <= (others => '0');
				
				if std_logic_vector(Zarlink_ResetCounter) < "0" & Zarlink_WaitForReset then
					Zarlink_ResetCounter <= Zarlink_ResetCounter +1;
				else
					Zarlink_ResetCounter <= Zarlink_ResetCounter;
				end if;

				if std_logic_vector(Zarlink_ConnectionTimeoutCounter) < "0" & Zarlink_ConnectionTimeout then
					Zarlink_ConnectionTimeoutCounter <= Zarlink_ConnectionTimeoutCounter +1;
				else
					Zarlink_ConnectionTimeoutCounter <= Zarlink_ConnectionTimeoutCounter;
				end if;
				
				NET_LENGTH <= NET_LENGTH;
				
				Zarlink_Retry_Counter <= (others => '0');
				
				case FSM_STATE is
					when STATE_RESET => 
						FSM_STATE <= STATE_IDLE;
						Input_Memory <= (others => '0');	
						ErrorState <= (others => '0');
					when STATE_IDLE => 
						
						-- Is there packet with my name? 
						if (Packet_Start = '1') AND (In_Valid = '1') AND (In_Data = Module_Name) then
							-- yes 
							FSM_STATE <= STATE_GETDATA;
						else
							-- no
							FSM_STATE <= STATE_IDLE;
						end if;
						Input_Memory <= (others => '0');	
						ErrorState <= (others => '0');
					-- There is a putative packet with my name... 
					-- Store my data !!!
					when STATE_GETDATA => 
						-- Is there valid data? 
						if In_Valid = '1' then
							-- Yes
							Input_WordCounter <= Input_WordCounter + 1;
							-- Can we still store data? 
							if (Input_WordCounter < to_unsigned(Module_LengthInWord,16)) AND (Module_HasMemory = '1') then
								-- Yes
								-- Then store the data !!!!
								Input_Memory <= In_Data & Input_Memory((Module_LengthInWord*16)-1 downto 16) ;
							else
								-- No
								Input_WordCounter <= Input_WordCounter;
								Input_Memory <= (others => '0');
							end if;
						else
							-- No
							Input_WordCounter <= Input_WordCounter;
						end if;
						
						-- Has the Packet ended? 
						if Packet_End = '0' then
							-- No
							FSM_STATE <= STATE_GETDATA;
						else
							In_Wait <= '1';
							-- Yes
							-- Was it valid?
							if (Packet_End = '1') AND (Packet_Valid = '1') AND 
								((Input_WordCounter = to_unsigned(Module_LengthInWord,16) ) OR Module_HasMemory = '0') then 
								-- Yes => Continue
								FSM_STATE <= STATE_INPUTDATA_VALID;
							else
								-- No
								-- I will report this!!!!
								if (Packet_Valid = '0') AND ((Input_WordCounter = to_unsigned(Module_LengthInWord,16))  OR Module_HasMemory = '0' ) then
									ErrorState <= std_logic_vector(to_unsigned(4,16));
								elsif Input_WordCounter > to_unsigned(Module_LengthInWord,16) then 
									ErrorState <= std_logic_vector(to_unsigned(1,16));
								elsif Input_WordCounter < to_unsigned(Module_LengthInWord,16) then
									ErrorState <= std_logic_vector(to_unsigned(2,16));
								else
									ErrorState <= std_logic_vector(to_unsigned(3,16));
								end if;
								FSM_STATE <= STATE_NETWORK_PREPARE;
							end if;
						end if;
					
					-- We have a valid packet!!!
					-- Lets do something with it. 
					-- Do Network or Zarlink...
					when STATE_INPUTDATA_VALID	=>
					
						----------------------------------------
						-- Here we can assign some variables... 
						----------------------------------------
						
						if Module_UsesZarlink = '1' then
							FSM_STATE <= STATE_FIRST_RESET;
						else
							FSM_STATE <= STATE_NETWORK_PREPARE;
						end if;
						
						In_Wait <= '1';

					-- Wait until the power on reset is done... 
					when STATE_FIRST_RESET => 
						-- (500ms wait, 23 Bit Counter) 
						if std_logic_vector(Zarlink_ResetCounter) < "0" & Zarlink_WaitForReset then
							FSM_STATE <= STATE_FIRST_RESET;
						else
							FSM_STATE <= STATE_ZARLINK_PREPARE;
							Zarlink_RequestChannel <= '1';
						end if;
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';

					--	We need the Zarlink
					-- So... Lets get it. 
					when STATE_ZARLINK_PREPARE =>
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if Zarlink_GetChannel = '1' then
							-- We have the channel to the zarlink. 
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE;
						else
							-- Continue waiting
							FSM_STATE <= STATE_ZARLINK_PREPARE;
						end if;
						Z_Done <= '0';
						
					-- We have the link to the Zarlink
					-- Lets comunicate...
					when STATE_ZARLINK_COMMUNICATE =>
						-- (dec95 <= dec82) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_SECOND_RESET;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE;
							Z_Address <= std_logic_vector(to_unsigned(95,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(82,8));	
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE;						
						end if;

						Zarlink_ResetCounter <= (others => '0');
						
					when STATE_SECOND_RESET =>
						-- (500ms wait, 23 Bit Counter, Count to 6250000) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ResetCounter) < "0" & Zarlink_WaitForReset then
							FSM_STATE <= STATE_SECOND_RESET;
						else
							FSM_STATE <= STATE_SPI_SPEED_A;
							Zarlink_ConnectionTimeoutCounter <= (others => '0');
						end if;					
						Z_Done <= '0';
						
						
					when STATE_SPI_SPEED_A =>
						-- (dec43 <= "01100000") 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(500,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WATCHDOG_OFF;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_SPI_SPEED_A;
							Z_Address <= std_logic_vector(to_unsigned(43,7));	
							ZDOR (7 downto 0) <= "0" & Zarlink_SPISpeedFlags & "00000";	
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_SPI_SPEED_A;						
						end if;

					when STATE_WATCHDOG_OFF =>
						-- (dec26 <= 0x0F) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(501,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_SPI_SPEED_B;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_WATCHDOG_OFF;
							Z_Address <= std_logic_vector(to_unsigned(26,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(15,8));
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_WATCHDOG_OFF;						
						end if;

					when STATE_SPI_SPEED_B =>
						-- (dec43 <= "01100000") 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(502,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_A_REQ;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_SPI_SPEED_B;
							Z_Address <= std_logic_vector(to_unsigned(43,7));	
							ZDOR (7 downto 0) <= "0" & Zarlink_SPISpeedFlags & "00000";	
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_SPI_SPEED_B;						
						end if;

					when STATE_WAIT_IRQ_RADIOREADY_A_REQ =>
						-- (dec50 => [3]) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(503,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_A_WAIT;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_A_REQ;
							Z_Address <= std_logic_vector(to_unsigned(50,7));	
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_A_REQ;						
						end if;
				
					when STATE_WAIT_IRQ_RADIOREADY_A_WAIT =>
						-- (dec50 => [3]) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';		
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(504,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif Zarlink_Valid_In = '0' then
							-- Wait 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_A_WAIT;
						else
							if Zarlink_Data_In(3) = '1' then
								-- Continue
								FSM_STATE <= STATE_REG_MAC_MODUSER;
							else
								-- Return
								FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_A_PAUSE;
							end if;
						end if;
						Z_Done <= '0';

					when STATE_WAIT_IRQ_RADIOREADY_A_PAUSE => 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';	
						Zarlink_Retry_Counter <= Zarlink_Retry_Counter +1;
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(504,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif std_logic_vector(Zarlink_Retry_Counter) >= Zarlink_Retry then 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_A_REQ;
						else
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_A_PAUSE;
						end if;
						Z_Done <= '0';
						
					when STATE_REG_MAC_MODUSER =>
						-- (dec23 <= "00001111")
						-- User Bits = "0000"
						-- RX 4FSK = "11"
						-- TX 4FSK = "11" 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(505,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_MAC_CHANNEL;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_MAC_MODUSER;
							Z_Address <= std_logic_vector(to_unsigned(23,7));	
							ZDOR (7 downto 0) <= "0000" & Zarlink_Connection_Mode;		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_MAC_MODUSER;						
						end if;

					
					when STATE_REG_MAC_CHANNEL =>
						-- (dec25 <= dec5) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(506,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_REG_TXBUFFER_BSIZE_A;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_MAC_CHANNEL;
							Z_Address <= std_logic_vector(to_unsigned(25,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(5,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_MAC_CHANNEL;						
						end if;

						
					when STATE_REG_REG_TXBUFFER_BSIZE_A =>
						-- (dec4 <= dec14) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(507,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_REG_RXBUFFER_BSIZE_A;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_REG_TXBUFFER_BSIZE_A;
							Z_Address <= std_logic_vector(to_unsigned(4,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(14,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_REG_TXBUFFER_BSIZE_A;						
						end if;
						
					when STATE_REG_REG_RXBUFFER_BSIZE_A => 
						-- (dec2 <= dec30) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(508,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_TXBUFF_MAXPACKSIZE_A;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_REG_RXBUFFER_BSIZE_A;
							Z_Address <= std_logic_vector(to_unsigned(2,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(30,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_REG_RXBUFFER_BSIZE_A;						
						end if;
						
					when STATE_REG_TXBUFF_MAXPACKSIZE_A => 
						-- (dec5 <= dec 31) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(509,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							if Zarlink_Implant_Mode = '1' then
								FSM_STATE <= STATE_IMD_COPY_ID_A;
							else
								FSM_STATE <= STATE_IMD_ID_A;
							end if;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_TXBUFF_MAXPACKSIZE_A;
							Z_Address <= std_logic_vector(to_unsigned(5,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(31,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_TXBUFF_MAXPACKSIZE_A;						
						end if;
						
					---------------------------------------
					---- Only IMD
					---------------------------------------
					
					when STATE_IMD_COPY_ID_A =>
						-- (dec66 <= Input_Memory(7 downto 0)) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(535,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_IMD_COPY_ID_B;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_IMD_COPY_ID_A;
							Z_Address <= std_logic_vector(to_unsigned(66,7));	
							ZDOR (7 downto 0) <= Input_Memory(7 downto 0);		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_IMD_COPY_ID_A;						
						end if;
						
					when STATE_IMD_COPY_ID_B =>
						-- (dec67 <= Input_Memory(15 downto 8)) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(536,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_IMD_COPY_ID_C;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_IMD_COPY_ID_B;
							Z_Address <= std_logic_vector(to_unsigned(67,7));	
							ZDOR (7 downto 0) <= Input_Memory(15 downto 8);		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_IMD_COPY_ID_B;						
						end if;
						
					when STATE_IMD_COPY_ID_C =>
						-- (dec67 <= Input_Memory(23 downto 16))  
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(536,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_IMD_ID_A;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_IMD_COPY_ID_C;
							Z_Address <= std_logic_vector(to_unsigned(68,7));	
							ZDOR (7 downto 0) <= Input_Memory(23 downto 16);		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_IMD_COPY_ID_C;						
						end if;
					
					---------------------------------------	
					---------------------------------------
					---------------------------------------
					
					when STATE_IMD_ID_A =>
						-- (dec19 <= Input_Memory(7 downto 0)) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(510,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_IMD_ID_B;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_IMD_ID_A;
							Z_Address <= std_logic_vector(to_unsigned(19,7));	
							ZDOR (7 downto 0) <= Input_Memory(7 downto 0);		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_IMD_ID_A;						
						end if;
						
					when STATE_IMD_ID_B =>
						-- (dec20 <= Input_Memory(15 downto 8)) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(511,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_IMD_ID_C;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_IMD_ID_B;
							Z_Address <= std_logic_vector(to_unsigned(20,7));	
							ZDOR (7 downto 0) <= Input_Memory(15 downto 8);		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_IMD_ID_B;						
						end if;
						
					when STATE_IMD_ID_C =>
						-- (dec21 <= Input_Memory(23 downto 16))  
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(512,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							if Zarlink_Implant_Mode = '1' then
								FSM_STATE <= STATE_REG_MAC_INITCOM_A_IMD;
							else
								FSM_STATE <= STATE_IRQ_ENABLESET1;
							end if;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_IMD_ID_C;
							Z_Address <= std_logic_vector(to_unsigned(21,7));	
							ZDOR (7 downto 0) <= Input_Memory(23 downto 16);		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_IMD_ID_C;						
						end if;
					
					---------------------------------------------------------
					-- This is only for the IMD (and the last IMD command!)
					---------------------------------------------------------
					
					when STATE_REG_MAC_INITCOM_A_IMD => 
						-- (dec42 <= dec0) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(538,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_NETWORK_PREPARE;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_MAC_INITCOM_A_IMD;
							Z_Address <= std_logic_vector(to_unsigned(42,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(0,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_MAC_INITCOM_A_IMD;						
						end if;
						
					---------------------------------------
					-- FROM HERE ON ONLY BASE-STATION 
					---------------------------------------
					
					when STATE_IRQ_ENABLESET1 =>
						-- (dec48 <= dec16) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(513,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_MAC_INITCOM_A;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_IRQ_ENABLESET1;
							Z_Address <= std_logic_vector(to_unsigned(48,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(16,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_IRQ_ENABLESET1;						
						end if;
					
					when STATE_REG_MAC_INITCOM_A => 
						-- (dec42 <= dec138) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(514,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WAIT_IRQ_RXNOTEMPTY_REQ;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_MAC_INITCOM_A;
							Z_Address <= std_logic_vector(to_unsigned(42,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(138,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_MAC_INITCOM_A;						
						end if;
						
					when STATE_WAIT_IRQ_RXNOTEMPTY_REQ =>
						-- (dec46 => [4]) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(515,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WAIT_IRQ_RXNOTEMPTY_WAIT;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_WAIT_IRQ_RXNOTEMPTY_REQ;
							Z_Address <= std_logic_vector(to_unsigned(46,7));	
							ZDOR (7 downto 0) <= (others => '0');		
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_WAIT_IRQ_RXNOTEMPTY_REQ;						
						end if;
						
					when STATE_WAIT_IRQ_RXNOTEMPTY_WAIT =>
						-- (dec46 => [4]) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';		
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(516,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif Zarlink_Valid_In = '0' then
							-- Wait 
							FSM_STATE <= STATE_WAIT_IRQ_RXNOTEMPTY_WAIT;
						else
							if Zarlink_Data_In(4) = '1' then
								-- Continue
								FSM_STATE <= STATE_READ_BLOCK_REQ;
							else
								-- Return
								FSM_STATE <= STATE_WAIT_IRQ_RXNOTEMPTY_PAUSE;
							end if;
						end if;
						Z_Done <= '0';

					when STATE_WAIT_IRQ_RXNOTEMPTY_PAUSE => 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';	
						Zarlink_Retry_Counter <= Zarlink_Retry_Counter +1;
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(516,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif std_logic_vector(Zarlink_Retry_Counter) >= Zarlink_Retry then 
							FSM_STATE <= STATE_WAIT_IRQ_RXNOTEMPTY_REQ;
						else
							FSM_STATE <= STATE_WAIT_IRQ_RXNOTEMPTY_PAUSE;
						end if;
						Z_Done <= '0';
						
					when STATE_READ_BLOCK_REQ =>
						-- (dec45 => XXX 14 Byte) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(517,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_READ_BLOCK_WAIT;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_READ_BLOCK_REQ;
							Z_Address <= std_logic_vector(to_unsigned(45,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '1';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_READ_BLOCK_REQ;						
						end if;
						
					when STATE_READ_BLOCK_WAIT =>
						-- (dec45 => XXX 14 Byte) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';	
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(518,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif Zarlink_Error = '1' then
							ErrorState <= std_logic_vector(to_unsigned(519,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif Zarlink_Valid_In = '0' then
							FSM_STATE <= STATE_READ_BLOCK_WAIT;
						else
							if (Zarlink_Data_In(111 downto 104) = Input_Memory(23 downto 16)) AND 
								(Zarlink_Data_In(103 downto 96) = Input_Memory(15 downto 8)) AND 
								(Zarlink_Data_In(95 downto 88) = Input_Memory(7 downto 0)) then
								
								FSM_STATE <= STATE_REG_MAC_INITCOM_B;
							else
								FSM_STATE <= STATE_READ_BLOCK_REQ;
							end if;
						end if;
						Z_Done <= '0';
						
					when STATE_REG_MAC_INITCOM_B => 
						-- (dec42 <= dec8) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(520,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_MAC_CTRL_A;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_MAC_INITCOM_B;
							Z_Address <= std_logic_vector(to_unsigned(42,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(8,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_MAC_INITCOM_B;						
						end if;
						
					when STATE_REG_MAC_CTRL_A =>
						-- (dec41 <= dec2) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(521,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_B_REQ;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_MAC_CTRL_A;
							Z_Address <= std_logic_vector(to_unsigned(41,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(2,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_MAC_CTRL_A;						
						end if;
						
					when STATE_WAIT_IRQ_RADIOREADY_B_REQ => 
						-- (dec50 => [3]) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(522,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_B_WAIT;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_B_REQ;
							Z_Address <= std_logic_vector(to_unsigned(50,7));	
							ZDOR (7 downto 0) <= (others => '0');		
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_B_REQ;						
						end if;
						
					when STATE_WAIT_IRQ_RADIOREADY_B_WAIT =>
						-- (dec50 => [3]) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';		
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(523,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif Zarlink_Valid_In = '0' then
							-- Wait 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_B_WAIT;
						else
							if Zarlink_Data_In(3) = '1' then
								-- Continue
								FSM_STATE <= STATE_REG_MAX_RESENDTIME;
							else
								-- Return

								FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_B_PAUSE;
							end if;
						end if;
						Z_Done <= '0';

					when STATE_WAIT_IRQ_RADIOREADY_B_PAUSE => 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';	
						Zarlink_Retry_Counter <= Zarlink_Retry_Counter +1;
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(523,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif std_logic_vector(Zarlink_Retry_Counter) >= Zarlink_Retry then 
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_B_REQ;
						else
							FSM_STATE <= STATE_WAIT_IRQ_RADIOREADY_B_PAUSE;
						end if;
						Z_Done <= '0';
						
					when STATE_REG_MAX_RESENDTIME =>
						-- (dec37 <= dec23) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(524,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_MAC_CTRL_B;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_MAX_RESENDTIME;
							Z_Address <= std_logic_vector(to_unsigned(37,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(23,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_MAX_RESENDTIME;						
						end if;
						
					when STATE_REG_MAC_CTRL_B =>
						-- (dec41 <= dec0) 					
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(525,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_MAC_CTRL_C;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_MAC_CTRL_B;
							Z_Address <= std_logic_vector(to_unsigned(41,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(0,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_MAC_CTRL_B;						
						end if;
						
					when STATE_REG_MAC_CTRL_C => 
						-- (dec41 <= dec64) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(526,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WAIT_IRQ_COMMANDDONE_REQ;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_MAC_CTRL_C;
							Z_Address <= std_logic_vector(to_unsigned(41,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(64,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_MAC_CTRL_C;						
						end if;
						
					when STATE_WAIT_IRQ_COMMANDDONE_REQ => 
						-- (dec50 => [0]) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(527,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WAIT_IRQ_COMMANDDONE_WAIT;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_WAIT_IRQ_COMMANDDONE_REQ;
							Z_Address <= std_logic_vector(to_unsigned(50,7));	
							ZDOR (7 downto 0) <= (others => '0');		
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_WAIT_IRQ_COMMANDDONE_REQ;						
						end if;
						
					when STATE_WAIT_IRQ_COMMANDDONE_WAIT =>
						-- (dec50 => [0]) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';		
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(528,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif Zarlink_Valid_In = '0' then
							-- Wait 
							FSM_STATE <= STATE_WAIT_IRQ_COMMANDDONE_WAIT;
						else
							if Zarlink_Data_In(0) = '1' then
								-- Continue
								FSM_STATE <= STATE_REG_REG_TXBUFFER_BSIZE_B;
							else
								-- Return

								FSM_STATE <= STATE_WAIT_IRQ_COMMANDDONE_PAUSE;
							end if;
						end if;
						Z_Done <= '0';

					when STATE_WAIT_IRQ_COMMANDDONE_PAUSE => 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';	
						Zarlink_Retry_Counter <= Zarlink_Retry_Counter +1;
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(528,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif std_logic_vector(Zarlink_Retry_Counter) >= Zarlink_Retry then 
							FSM_STATE <= STATE_WAIT_IRQ_COMMANDDONE_REQ;
						else
							FSM_STATE <= STATE_WAIT_IRQ_COMMANDDONE_PAUSE;
						end if;						
						Z_Done <= '0';
						
					when STATE_REG_REG_TXBUFFER_BSIZE_B =>
						-- (dec4 <= dec14) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(529,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_REG_RXBUFFER_BSIZE_B;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_REG_TXBUFFER_BSIZE_B;
							Z_Address <= std_logic_vector(to_unsigned(4,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(14,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_REG_TXBUFFER_BSIZE_B;						
						end if;
						
					when STATE_REG_REG_RXBUFFER_BSIZE_B => 
						-- (dec2 <= dec30) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(530,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_TXBUFF_MAXPACKSIZE_B;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_REG_RXBUFFER_BSIZE_B;
							Z_Address <= std_logic_vector(to_unsigned(2,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(30,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_REG_RXBUFFER_BSIZE_B;						
						end if;
						
					when STATE_REG_TXBUFF_MAXPACKSIZE_B => 
						-- (dec5 <= dec 31) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(531,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_REG_MAC_INITCOM_C;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_TXBUFF_MAXPACKSIZE_B;
							Z_Address <= std_logic_vector(to_unsigned(5,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(31,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_TXBUFF_MAXPACKSIZE_B;						
						end if;
						
					when STATE_REG_MAC_INITCOM_C => 
						-- (dec42 <= dec12) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(532,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WAIT_IRQ_LINKREADY_REQ;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_REG_MAC_INITCOM_C;
							Z_Address <= std_logic_vector(to_unsigned(42,7));	
							ZDOR (7 downto 0) <= std_logic_vector(to_unsigned(12,8));		
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_REG_MAC_INITCOM_C;						
						end if;
						
					when STATE_WAIT_IRQ_LINKREADY_REQ =>
						-- (dec50 => [2]) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(533,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;							
						elsif (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_WAIT_IRQ_LINKREADY_WAIT;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_WAIT_IRQ_LINKREADY_REQ;
							Z_Address <= std_logic_vector(to_unsigned(50,7));	
							ZDOR (7 downto 0) <= (others => '0');		
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_WAIT_IRQ_LINKREADY_REQ;						
						end if;
					
					when STATE_WAIT_IRQ_LINKREADY_WAIT =>
						-- (dec50 => [2]) 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';		
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(534,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif Zarlink_Valid_In = '0' then
							-- Wait 
							FSM_STATE <= STATE_WAIT_IRQ_LINKREADY_WAIT;
						else
							if Zarlink_Data_In(2) = '1' then
								-- Continue
								FSM_STATE <= STATE_NETWORK_PREPARE;
							else
								-- Return
								FSM_STATE <= STATE_WAIT_IRQ_LINKREADY_PAUSE;
							end if;
						end if;
						Z_Done <= '0';

					when STATE_WAIT_IRQ_LINKREADY_PAUSE => 
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';	
						Zarlink_Retry_Counter <= Zarlink_Retry_Counter +1;
						if std_logic_vector(Zarlink_ConnectionTimeoutCounter) >= "0" & Zarlink_ConnectionTimeout then 
							-- It took too long for the connection. Let's end this! 
							ErrorState <= std_logic_vector(to_unsigned(534,16));
							FSM_STATE <= STATE_NETWORK_PREPARE;	
						elsif std_logic_vector(Zarlink_Retry_Counter) >= Zarlink_Retry then 
							FSM_STATE <= STATE_WAIT_IRQ_LINKREADY_REQ;
						else
							FSM_STATE <= STATE_WAIT_IRQ_LINKREADY_PAUSE;
						end if;
						Z_Done <= '0';
						
					--	We need the Network
					-- So... Lets get it. 
					when STATE_NETWORK_PREPARE => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';
						if Network_GetChannel_Internal = '1' then
							-- We have the channel to the network.
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_HEADER;
							CRC_Reset_Internal <= '1';
						else
							-- Continue waiting
							FSM_STATE <= STATE_NETWORK_PREPARE;
						end if;	
					
					-- Communicate with the network (HEADER)
					when STATE_NETWORK_COMMUNICATE_HEADER =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';					
						if Network_Wait = '0' then
							Network_Data_Internal <= NET_HEADER;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_LENGTH;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_HEADER;
						end if;
						
					-- Communicate with the network (LENGTH)
					when STATE_NETWORK_COMMUNICATE_LENGTH =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= NET_LENGTH;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_RESPONSEID;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_LENGTH;
						end if;						
					
					-- Communicate with the network (RESPONSEID)
					when STATE_NETWORK_COMMUNICATE_RESPONSEID => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= NET_RESPONSEID;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ID;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_RESPONSEID;
						end if;
						
					-- Communicate with the network (ID)
					when STATE_NETWORK_COMMUNICATE_ID =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= Module_Name;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ERROR;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ID;
						end if;
					
					-- Communicate with the network (ERROR)
					when STATE_NETWORK_COMMUNICATE_ERROR => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= ErrorState;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ERROR;
						end if;
						
					-- Communicate with the network (BODY)
					when STATE_NETWORK_COMMUNICATE_BODY =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_A;
						-- Do your worst... 
					
					-- Wait for the CRC - Values I.
					when STATE_NETWORK_COMMUNICATE_WAIT_A =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_B;

					-- Wait for the CRC - Values II.
					when STATE_NETWORK_COMMUNICATE_WAIT_B =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';	
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_C;						
					
					-- Wait for the CRC - Values III.
					when STATE_NETWORK_COMMUNICATE_WAIT_C =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
						
					-- Communicate with the network (CRC 00)
					when STATE_NETWORK_COMMUNICATE_CRC00 => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						CRC_01_Copy <= CRC_01;
						CRC_10_Copy <= CRC_10;
						CRC_11_Copy <= CRC_11;
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_00;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
						end if;
						
					-- Communicate with the network (CRC 01)
					when STATE_NETWORK_COMMUNICATE_CRC01 =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						CRC_01_Copy <= CRC_01_Copy;
						CRC_10_Copy <= CRC_10_Copy;
						CRC_11_Copy <= CRC_11_Copy;		
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_01_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
						end if;
						
					-- Communicate with the network (CRC 10)
					when STATE_NETWORK_COMMUNICATE_CRC10 => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						CRC_10_Copy <= CRC_10_Copy;
						CRC_11_Copy <= CRC_11_Copy;	
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_10_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
						end if;
						
					-- Communicate with the network (CRC 11)
					when STATE_NETWORK_COMMUNICATE_CRC11 => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						

						CRC_11_Copy <= CRC_11_Copy;		
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_11_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_IDLE;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
						end if;						
						
					when others => 
						FSM_STATE <= STATE_RESET;
				
				end case;
		

		end if;
	end process FSM;

end architecture Behavioral;
