----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:50:33 12/30/2011 
-- Design Name: 
-- Module Name:    DataPacketRefurbish - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity DataPacketRefurbish is
    Port (
	 
			-- Connection to the FIFO (Zarlink Side)
			Data_In						: in STD_LOGIC_VECTOR(143 downto 0);
			Valid_In						: in STD_LOGIC;
			ReadEn_In					: out STD_LOGIC;

			Implant_ChannelMask 		: in std_logic_vector(127 downto 0);
			Implant_ADCResolution 	: in std_logic_vector(3 downto 0);
			Implant_SampleRate		: in std_logic_vector(7 downto 0);
			Implant_Filter 			: in std_logic_vector(3 downto 0);
			Implant_DeviceID 			: in STD_LOGIC_VECTOR(7 downto 0);

			ChannelMask_Count			: in STD_LOGIC_VECTOR(7 downto 0);
			ChannelMask_Ready			: in STD_LOGIC;
		
			-- Output Data (Network Side)
			FIFO_PlusFullPacket		: out STD_LOGIC;
			FIFO_wr_en 					: out STD_LOGIC;
			FIFO_din 					: out STD_LOGIC_VECTOR(16 DOWNTO 0);
			FIFO_FULL					: in  STD_LOGIC;
			
			-- Information about error states 
			FIFO_Error_Overrun  		: in  STD_LOGIC;
			FIFO_Error_Underrun 		: in  STD_LOGIC;	 
			
			
			Debug_DumpMode				: in STD_LOGIC;
			
			-- The usual ports 
			Reset 						: in  STD_LOGIC;			
			Clk 							: in  STD_LOGIC
		);
end DataPacketRefurbish;

architecture Behavioral of DataPacketRefurbish is

	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		STATE_HEADER_A,
		STATE_HEADER_B,
		STATE_HEADER_C,
		STATE_PACKAGE_DEFECT,
		STATE_DATAMINING,
		STATE_PREPARE_HEADER,
		STATE_PREPARE_HEADER_ERROR,
		STATE_PREPARE_LENGTH,
		STATE_PREPARE_LENGTH_ERROR,
		STATE_PREPARE_RESPONSEID,
		STATE_PREPARE_RESPONSEID_ERROR,
		STATE_PREPARE_ID_ERROR,
		STATE_PREPARE_ERROR_ERROR,
		STATE_PREPARE_TIMESTAMP_A,
		STATE_PREPARE_TIMESTAMP_B,
		STATE_PREPARE_TRIGGER,
		STATE_PREPARE_DATA,
		STATE_PREPARE_DATA_FIRST_PRE,
		STATE_PREPARE_DATA_FIRST,
		STATE_PREPARE_DATA_PROCESS,
		STATE_PREPARE_DATA_GETDATA,
		STATE_PREPARE_FILLUP,
		STATE_PREPARE_VALID,
		STATE_COOLDOWN,
		STATE_DEBUG_DUMP_A,
		STATE_DEBUG_DUMP_B,
		STATE_DEBUG_DUMP_C,
		STATE_DEBUG_DUMP_D,
		STATE_DEBUG_DUMP_E,
		STATE_DEBUG_DUMP_F,
		STATE_DEBUG_DUMP_G,
		STATE_DEBUG_DUMP_H,
		STATE_DEBUG_DUMP_I,
		STATE_DEBUG_DUMP_J,
		STATE_DEBUG_DUMP_K,
		STATE_DEBUG_DUMP_L,
		STATE_DEBUG_DUMP_M,
		STATE_DEBUG_DUMP_N,
		STATE_DEBUG_DUMP_O,
		STATE_DEBUG_DUMP_P,
		STATE_DEBUG_DUMP_Q,
		STATE_DEBUG_DUMP_R,
		STATE_DEBUG_DUMP_S,
		STATE_DEBUG_DUMP_T,
		STATE_DEBUG_DUMP_U,
		STATE_DEBUG_DUMP_V,
		STATE_DEBUG_DUMP_W,
		STATE_DEBUG_DUMP_X,
		STATE_DEBUG_DUMP_Y,
		STATE_DEBUG_DUMP_Z,
		STATE_DEBUG_DUMP_NEARLYDONE,
		STATE_DEBUG_DUMP_EXIT
	);
	signal FSM_STATE : STATE_TYPE;

	signal ChannelMask_Count_Copy : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
	signal Data_In_Copy : STD_LOGIC_VECTOR(143 downto 0) := (others => '0');
	signal NumberOfSamples : unsigned (7 downto 0) := (others => '0');
	signal TriggerChannelsIn : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal The10KHzCounter : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal TimeStamp : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
	signal PacketsAreWrong : STD_LOGIC := '0';
	
	-- First 4 bit
	constant PacketHeader_0 : std_logic_vector(3 downto 0) := "0000";
	constant PacketHeader_1 : std_logic_vector(3 downto 0) := "0001";

	-- 0x55AA
	constant NET_HEADER : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(21930,16));
	-- 0x8000
	constant NET_RESPONSEID_ERROR : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(32768,16));		
	-- 0x8001
	constant NET_RESPONSEID_OK : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(32769,16));	

	signal ErrorState : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');

	-- Not constant because the length can change 
	signal NET_LENGTH_ERROR : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal NET_LENGTH_OK : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
	signal HadToFillUp : std_logic := '0';
	
	signal ThisIsALongHeader : std_logic := '0';

	signal DP_Input_Data : STD_LOGIC_VECTOR(110 downto 0) := (others => '0');
	signal DP_Input_IsHeader : STD_LOGIC := '0';
	signal DP_Input_Valid : STD_LOGIC := '0';
	signal DP_NewPacket : STD_LOGIC := '0';

	signal DP_Input_RequestNext : STD_LOGIC := '0';		
	signal DP_Output_Data : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal DP_Output_Valid : STD_LOGIC := '0';
	
	signal ReadEn_In_Copy : STD_LOGIC := '0';
	signal Data_In_BitReverse	: STD_LOGIC_VECTOR(143 downto 0) := (others => '0');
	
begin
	-- This is the real data part of the packet 
	DP_Input_Data <= Data_In_Copy(111 downto 1);
	ReadEn_In <= ReadEn_In_Copy;

	Data_In_BitReverse <= Data_In(143 downto 112)
		&Data_In(104)&Data_In(105)&Data_In(106)&Data_In(107)&Data_In(108)&Data_In(109)&Data_In(110)&Data_In(111)
		&Data_In(96)&Data_In(97)&Data_In(98)&Data_In(99)&Data_In(100)&Data_In(101)&Data_In(102)&Data_In(103)
		&Data_In(88)&Data_In(89)&Data_In(90)&Data_In(91)&Data_In(92)&Data_In(93)&Data_In(94)&Data_In(95)
		&Data_In(80)&Data_In(81)&Data_In(82)&Data_In(83)&Data_In(84)&Data_In(85)&Data_In(86)&Data_In(87)
		&Data_In(72)&Data_In(73)&Data_In(74)&Data_In(75)&Data_In(76)&Data_In(77)&Data_In(78)&Data_In(79)
		&Data_In(64)&Data_In(65)&Data_In(66)&Data_In(67)&Data_In(68)&Data_In(69)&Data_In(70)&Data_In(71)
		&Data_In(56)&Data_In(57)&Data_In(58)&Data_In(59)&Data_In(60)&Data_In(61)&Data_In(62)&Data_In(63)
		&Data_In(48)&Data_In(49)&Data_In(50)&Data_In(51)&Data_In(52)&Data_In(53)&Data_In(54)&Data_In(55)
		&Data_In(40)&Data_In(41)&Data_In(42)&Data_In(43)&Data_In(44)&Data_In(45)&Data_In(46)&Data_In(47)
		&Data_In(32)&Data_In(33)&Data_In(34)&Data_In(35)&Data_In(36)&Data_In(37)&Data_In(38)&Data_In(39)
		&Data_In(24)&Data_In(25)&Data_In(26)&Data_In(27)&Data_In(28)&Data_In(29)&Data_In(30)&Data_In(31)
		&Data_In(16)&Data_In(17)&Data_In(18)&Data_In(19)&Data_In(20)&Data_In(21)&Data_In(22)&Data_In(23)
		&Data_In(8)&Data_In(9)&Data_In(10)&Data_In(11)&Data_In(12)&Data_In(13)&Data_In(14)&Data_In(15)
		&Data_In(0)&Data_In(1)&Data_In(2)&Data_In(3)&Data_In(4)&Data_In(5)&Data_In(6)&Data_In(7);
	
	FSM : process (Reset, Clk)
	begin
		if Reset = '1' then
			
			FSM_STATE <= STATE_RESET;
			Data_In_Copy <= (others => '0');
			ChannelMask_Count_Copy <= (others => '0');
			NumberOfSamples <= (others => '0');
			ReadEn_In_Copy <= '0';
			TriggerChannelsIn <= (others => '0');
			The10KHzCounter <= (others => '0');
			TimeStamp <= (others => '0');
			PacketsAreWrong <= '0';
			ErrorState <= (others => '0');
			
			NET_LENGTH_ERROR <= std_logic_vector(to_unsigned(18,16));
			NET_LENGTH_OK <= (others => '0');
			
			FIFO_PlusFullPacket <= '0';
			FIFO_wr_en <= '0';
			FIFO_din <= (others => '0');
			HadToFillUp <= '0';
			
			ThisIsALongHeader <= '0';
			DP_Input_IsHeader <= '0';
			DP_NewPacket <= '0';
			DP_Input_Valid <= '0';
			
		elsif rising_edge(Clk) then
			
			-- Defaults: 
			Data_In_Copy <= Data_In_Copy;
			ChannelMask_Count_Copy <= ChannelMask_Count_Copy;
			NumberOfSamples <= NumberOfSamples;
			ReadEn_In_Copy <= '0';
			
			TriggerChannelsIn <= TriggerChannelsIn;
			The10KHzCounter <= The10KHzCounter;
			TimeStamp <= TimeStamp;
			PacketsAreWrong <= PacketsAreWrong;
			ErrorState <= ErrorState;
			NET_LENGTH_ERROR <= NET_LENGTH_ERROR;
			NET_LENGTH_OK <= NET_LENGTH_OK;

			FIFO_PlusFullPacket <= '0';
			FIFO_wr_en <= '0';
			FIFO_din <= (others => '0');
			HadToFillUp <= HadToFillUp;
			
			ThisIsALongHeader <= ThisIsALongHeader;
			DP_Input_IsHeader <= '0'; 
			DP_NewPacket <= '0';
			DP_Input_Valid <= '0';
			
			case FSM_STATE is
				when STATE_RESET =>
					FSM_STATE <= STATE_IDLE;
				
				when STATE_IDLE =>
					-- Is there valid data and is the number of samples known? 
					if (Valid_In = '1') AND (ChannelMask_Ready = '1') then
						-- Yes => Let's start
						if Debug_DumpMode = '0' then
							FSM_STATE <= STATE_HEADER_A;
						else
							FSM_STATE <= STATE_DEBUG_DUMP_A; 
						end if;
						ReadEn_In_Copy <= '1';
					else
						-- No => Wait
						FSM_STATE <= STATE_IDLE;
					end if;
					ErrorState <= (others => '0');
					Data_In_Copy <= Data_In_BitReverse;
					ChannelMask_Count_Copy <= ChannelMask_Count;
					NumberOfSamples <= (others => '0');
					HadToFillUp <= '0';
					
				when STATE_HEADER_A =>
				
					if Data_In_Copy(0) /= '1' then
						FSM_STATE <= STATE_COOLDOWN;
					elsif (PacketHeader_0 = Data_In_Copy(4 downto 1)) AND (Implant_DeviceID = Data_In_Copy(12 downto 5)) 
						AND (Implant_ChannelMask(98 downto 0) = Data_In_Copy(111 downto 13) ) then 
					-- Header Long A
						FSM_STATE <= STATE_HEADER_B;
						ThisIsALongHeader <= '1';
					elsif (PacketHeader_1 = Data_In_Copy(4 downto 1)) AND (Implant_DeviceID = Data_In_Copy(12 downto 5)) 
						AND ('0' = Data_In_Copy(29)) AND ('0' = Data_In_Copy(30)) AND PacketsAreWrong = '0' then
					-- Header Short 
						FSM_STATE <= STATE_DATAMINING;
						ThisIsALongHeader <= '0';
					elsif PacketHeader_0 = Data_In_Copy(4 downto 1)	then 
					-- Something wrong with the first part of a long header 
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						PacketsAreWrong <= '1';
						ErrorState <= std_logic_vector(to_unsigned(600,16));
					elsif PacketHeader_1 = Data_In_Copy(4 downto 1)	then						
					-- Something wrong with the sort header 
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						ErrorState <= std_logic_vector(to_unsigned(601,16));
					elsif FIFO_Error_Overrun = '1' then
						-- If there was something wrong with the fifo then we have a problem
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						PacketsAreWrong <= '1';						
						ErrorState <= std_logic_vector(to_unsigned(603,16));
					elsif FIFO_Error_Underrun = '1' then
						-- If there was something wrong with the fifo then we have a problem
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						PacketsAreWrong <= '1';		
						ErrorState <= std_logic_vector(to_unsigned(604,16));						
					else
						-- Generate a defect package recieved network response !!!! 
						-- Unknown packet typ! 
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						ErrorState <= std_logic_vector(to_unsigned(602,16));
					end if;
					TriggerChannelsIn <= Data_In_Copy(143 downto 128);
					The10KHzCounter <= Data_In_Copy(127 downto 112);
					Timestamp <= Data_In_Copy(28 downto 13);
					
					NET_LENGTH_OK <= std_logic_vector(to_unsigned(22,16)+unsigned(ChannelMask_Count_Copy)+unsigned(ChannelMask_Count_Copy));
					
				when STATE_HEADER_B =>
				
					-- WAIT FOR VALID PACKAGE and then use it and request a new one! 
					if Valid_In = '1' then
						FSM_STATE <= STATE_HEADER_C;
						ReadEn_In_Copy <= '1';
						Data_In_Copy <= Data_In_BitReverse;
					else
						FSM_STATE <= STATE_HEADER_B;
					end if;
				
				when STATE_HEADER_C =>	
					-- Process the second half of the long header 
					if Data_In_Copy(0) = '1' then
						FSM_STATE <= STATE_HEADER_A;
					elsif (Implant_ChannelMask(127 downto 99) = Data_In_Copy(29 downto 1)) AND (Implant_Filter = Data_In_Copy(33 downto 30)) 
						AND (Implant_SampleRate = Data_In_Copy(57 downto 50)) AND (Implant_ADCResolution = Data_In_Copy (61 downto 58)) 
						AND (Data_In_Copy(62) = '0') AND (Data_In_Copy(63) = '0') then 
						-- Header Long B
						FSM_STATE <= STATE_DATAMINING;
						-- Absolution is given... 
						PacketsAreWrong <= '0';
					else
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						PacketsAreWrong <= '1';
						ErrorState <= std_logic_vector(to_unsigned(605,16));
					end if;
					Timestamp <= Data_In_Copy(49 downto 34);
				
				-- Wait until there is space in the FIFO... then begin!
				-- NOTE: Packet is closed with FIDO(16) => '1' at it's last element
				-- CRC is calculated later but add the 8 byte to the length!
				when STATE_DATAMINING => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DATAMINING;
					else
						FSM_STATE <= STATE_PREPARE_HEADER;
					end if;

				when STATE_PREPARE_HEADER => 
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & NET_HEADER;				
					FSM_STATE <= STATE_PREPARE_LENGTH;
					
				when STATE_PREPARE_LENGTH =>
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & NET_LENGTH_OK;
					FSM_STATE <= STATE_PREPARE_RESPONSEID;					

				when STATE_PREPARE_RESPONSEID => 
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & NET_RESPONSEID_OK;
					FSM_STATE <= STATE_PREPARE_TIMESTAMP_A;
					
				when STATE_PREPARE_TIMESTAMP_A =>
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & TimeStamp;
					FSM_STATE <= STATE_PREPARE_TIMESTAMP_B;
					
				when STATE_PREPARE_TIMESTAMP_B =>
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & The10KHzCounter;
					FSM_STATE <= STATE_PREPARE_TRIGGER;
					
				when STATE_PREPARE_TRIGGER =>
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & TriggerChannelsIn;				
					FSM_STATE <= STATE_PREPARE_DATA;
					
				when STATE_PREPARE_DATA =>
					-- Reset the data processor 
					DP_NewPacket <= '1';
					FSM_STATE <= STATE_PREPARE_DATA_FIRST_PRE;
				
				when STATE_PREPARE_DATA_FIRST_PRE => 
					FSM_STATE <= STATE_PREPARE_DATA_FIRST;
					DP_Input_IsHeader <= '1';
					DP_Input_Valid <= '1';

				when STATE_PREPARE_DATA_FIRST => 
					if DP_Input_RequestNext = '1' then
						FSM_STATE <= STATE_PREPARE_DATA_PROCESS;
					else
						FSM_STATE <= STATE_PREPARE_DATA_FIRST;
					end if;

				when STATE_PREPARE_DATA_PROCESS => 
					
					-- Stop at the maximum number of expected samples 
					-- Don't send more than that 
					if NumberOfSamples < unsigned(ChannelMask_Count_Copy) then 
						FIFO_wr_en <= DP_Output_Valid;
					else
						null;
					end if;
					
					-- Copy the result to the network fifo
					-- FIFO_din <= "0" & DP_Output_Data;
					FIFO_din <= "0" & DP_Output_Data(0) & DP_Output_Data(1) & DP_Output_Data(2) 
									& DP_Output_Data(3) & DP_Output_Data(4) & DP_Output_Data(5) & DP_Output_Data(6)
									& DP_Output_Data(7) & DP_Output_Data(8) & DP_Output_Data(9) & DP_Output_Data(10)
									& DP_Output_Data(11) & DP_Output_Data(12) & DP_Output_Data(13) 
									& DP_Output_Data(14) & DP_Output_Data(15);
					
					-- Count the emitted samples 
					if DP_Output_Valid = '1' then
						NumberOfSamples <= NumberOfSamples + 1;
					else
						null;
					end if;
					
					if (NumberOfSamples < unsigned(ChannelMask_Count_Copy)) AND (DP_Input_RequestNext = '1') then 
						FSM_STATE <= STATE_PREPARE_DATA_GETDATA;
					elsif NumberOfSamples < unsigned(ChannelMask_Count_Copy) then 
						FSM_STATE <= STATE_PREPARE_DATA_PROCESS;
					else
						FSM_STATE <= STATE_PREPARE_FILLUP;
					end if;
					
				when STATE_PREPARE_DATA_GETDATA =>
			
					-- Stop at the maximum number of expected samples 
					-- Don't send more than that 
					if NumberOfSamples < unsigned(ChannelMask_Count_Copy) then 
						FIFO_wr_en <= DP_Output_Valid;
					else
						null;
					end if;
					
					-- Copy the result to the network fifo
					--FIFO_din <= "0" & DP_Output_Data;
					FIFO_din <= "0" & DP_Output_Data(0) & DP_Output_Data(1) & DP_Output_Data(2) 
									& DP_Output_Data(3) & DP_Output_Data(4) & DP_Output_Data(5) & DP_Output_Data(6)
									& DP_Output_Data(7) & DP_Output_Data(8) & DP_Output_Data(9) & DP_Output_Data(10)
									& DP_Output_Data(11) & DP_Output_Data(12) & DP_Output_Data(13) 
									& DP_Output_Data(14) & DP_Output_Data(15);					
					-- Count the emitted samples 
					if DP_Output_Valid = '1' then
						NumberOfSamples <= NumberOfSamples + 1;
					else
						null;
					end if;
					
					if (Valid_In = '1') AND (Data_In_BitReverse(0) = '1') then
						-- This can be bad!!! 
						-- Packet could have ended to soon... 
						FSM_STATE <= STATE_PREPARE_FILLUP;
					elsif NumberOfSamples >= unsigned(ChannelMask_Count_Copy) then
						-- Done... 
						FSM_STATE <= STATE_PREPARE_FILLUP;
					elsif (Valid_In = '1') AND (ReadEn_In_Copy = '0') then
						-- Looks good... 
						ReadEn_In_Copy <= '1';
						Data_In_Copy <= Data_In_BitReverse;
						DP_Input_Valid <= '1';
						FSM_STATE <= STATE_PREPARE_DATA_PROCESS;
					else
						-- No Data here yet
						FSM_STATE <= STATE_PREPARE_DATA_GETDATA;
					end if;
					
				when STATE_PREPARE_FILLUP =>
				
					if NumberOfSamples < unsigned(ChannelMask_Count_Copy) then
						FIFO_wr_en <= '1';
						FIFO_din <= (others => '0');
						NumberOfSamples <= NumberOfSamples + 1;
						HadToFillUp <= '1';
					else
						FSM_STATE <= STATE_PREPARE_VALID;
					end if;
					
				when STATE_PREPARE_VALID =>
					FIFO_PlusFullPacket <= '1';
					FIFO_wr_en <= '1';
					
					-- sufficent samples 
					if HadToFillUp = '0' then
						FIFO_din <= "1" & "0000000000000001";				
					else
					-- too few samples
						FIFO_din <= "1" & "0000000000000010";				
					end if;
					
					FSM_STATE <= STATE_COOLDOWN;
					
----- ERROR HANDLING: 	
			
				when STATE_PACKAGE_DEFECT => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PACKAGE_DEFECT;
					else
						FSM_STATE <= STATE_PREPARE_HEADER_ERROR;
					end if;

				when STATE_PREPARE_HEADER_ERROR => 
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & NET_HEADER;				
					FSM_STATE <= STATE_PREPARE_LENGTH_ERROR;
					
				when STATE_PREPARE_LENGTH_ERROR =>
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & NET_LENGTH_ERROR;
					FSM_STATE <= STATE_PREPARE_RESPONSEID_ERROR;
					
				when STATE_PREPARE_RESPONSEID_ERROR =>
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & NET_RESPONSEID_ERROR;
					FSM_STATE <= STATE_PREPARE_ID_ERROR;
					
				when STATE_PREPARE_ID_ERROR =>
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & std_logic_vector(to_unsigned(254,16));
					FSM_STATE <= STATE_PREPARE_ERROR_ERROR;
					
				when STATE_PREPARE_ERROR_ERROR =>				
					FIFO_PlusFullPacket <= '1';
					FIFO_wr_en <= '1';
					FIFO_din <= "1" & ErrorState;				
					FSM_STATE <= STATE_COOLDOWN;
				
				when STATE_COOLDOWN =>
					FSM_STATE <= STATE_IDLE;

-- --------- DEBUG.... 

				when STATE_DEBUG_DUMP_A => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_A;
					else
						FIFO_din <= "0" & "0000000011111111";
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_B;
					end if;
					
				when STATE_DEBUG_DUMP_B => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_B;
					else
						FIFO_din <= "0" & Implant_ChannelMask(15 downto 0);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_C;
					end if;

				when STATE_DEBUG_DUMP_C => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_C;
					else
						FIFO_din <= "0" & Implant_ChannelMask(31 downto 16);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_D;
					end if;

				when STATE_DEBUG_DUMP_D => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_D;
					else
						FIFO_din <= "0" & Implant_ChannelMask(47 downto 32);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_E;
					end if;

				when STATE_DEBUG_DUMP_E => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_E;
					else
						FIFO_din <= "0" & Implant_ChannelMask(63 downto 48);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_F;
					end if;

				when STATE_DEBUG_DUMP_F => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_F;
					else
						FIFO_din <= "0" & Implant_ChannelMask(79 downto 64);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_G;
					end if;

				when STATE_DEBUG_DUMP_G => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_G;
					else
						FIFO_din <= "0" & Implant_ChannelMask(95 downto 80);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_H;
					end if;

				when STATE_DEBUG_DUMP_H => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_H;
					else
						FIFO_din <= "0" & Implant_ChannelMask(111 downto 96);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_I;
					end if;

				when STATE_DEBUG_DUMP_I => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_I;
					else
						FIFO_din <= "0" & Implant_ChannelMask(127 downto 112);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_J;
					end if;

				when STATE_DEBUG_DUMP_J => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_J;
					else
						FIFO_din <= "0" & "0000001011111111";
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_K;
					end if;					

				when STATE_DEBUG_DUMP_K => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_K;
					else
						FIFO_din <= "0" & "00000000" & "0000" & Implant_ADCResolution;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_L;
					end if;

				when STATE_DEBUG_DUMP_L => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_L;
					else
						FIFO_din <= "0" & "0000011011111111";
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_M;
					end if;					

				when STATE_DEBUG_DUMP_M => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_M;
					else
						FIFO_din <= "0" & "00000000" & "0000" & Implant_Filter;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_N;
					end if;

				when STATE_DEBUG_DUMP_N => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_N;
					else
						FIFO_din <= "0" & "0000111011111111";
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_O;
					end if;	

				when STATE_DEBUG_DUMP_O => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_O;
					else
						FIFO_din <= "0" & "00000000" & Implant_SampleRate;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_P;
					end if;

				when STATE_DEBUG_DUMP_P => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_P;
					else
						FIFO_din <= "0" & "0001111011111111";
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_Q;
					end if;	

				when STATE_DEBUG_DUMP_Q => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_Q;
					else
						FIFO_din <= "0" & "00000000" & Implant_DeviceID;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_R;
					end if;

				when STATE_DEBUG_DUMP_R => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_R;
					else
						FIFO_din <= "0" & "0011111011111111";
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_S;
					end if;

				when STATE_DEBUG_DUMP_S => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_S;
					else
						FIFO_din <= "0" & Data_In_Copy(15 downto 0);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_T;
					end if;

				when STATE_DEBUG_DUMP_T => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_T;
					else
						FIFO_din <= "0" & Data_In_Copy(31 downto 16);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_U;
					end if;

				when STATE_DEBUG_DUMP_U => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_U;
					else
						FIFO_din <= "0" & Data_In_Copy(47 downto 32);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_V;
					end if;

				when STATE_DEBUG_DUMP_V => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_V;
					else
						FIFO_din <= "0" & Data_In_Copy(63 downto 48);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_W;
					end if;

				when STATE_DEBUG_DUMP_W => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_W;
					else
						FIFO_din <= "0" & Data_In_Copy(79 downto 64);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_X;
					end if;

				when STATE_DEBUG_DUMP_X => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_X;
					else
						FIFO_din <= "0" & Data_In_Copy(95 downto 80);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_Y;
					end if;

				when STATE_DEBUG_DUMP_Y => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_Y;
					else
						FIFO_din <= "0" & Data_In_Copy(111 downto 96);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_Z;
					end if;

				when STATE_DEBUG_DUMP_Z => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_Z;
					else
						FIFO_din <= "0" & Data_In_Copy(127 downto 112);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_NEARLYDONE;
					end if;

				when STATE_DEBUG_DUMP_NEARLYDONE => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_NEARLYDONE;
					else
						FIFO_din <= "1" & Data_In_Copy(143 downto 128);
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_EXIT;
					end if;
				
					when STATE_DEBUG_DUMP_EXIT => 
						FIFO_PlusFullPacket <= '1';
						FSM_STATE <= STATE_IDLE;
						
				when others =>
					FSM_STATE <= STATE_RESET;
			end case;
		
		end if;
	
	end process FSM;

MyStreamDecomposer : entity work.StreamDecomposer
    Port MAP ( 

		Resolution => Implant_ADCResolution,
		
		-- This comes in... 
		Input_Data => DP_Input_Data,
		Input_IsHeader	=> DP_Input_IsHeader,
		Input_HeaderLong => ThisIsALongHeader,
		Input_Valid	=> DP_Input_Valid,
		Input_RequestNext => DP_Input_RequestNext,
		
		-- This comes out...
		Output_Data	=> DP_Output_Data,
		Output_Valid => DP_Output_Valid,
		
		-- New Packet
		NewPacket => DP_NewPacket,

		-- Usual Ports 
		Clk => Clk,
      Reset => Reset
	 );

end Behavioral;

