library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
 
ENTITY DummyCommandElement_UUT IS
END DummyCommandElement_UUT;
 
ARCHITECTURE behavior OF DummyCommandElement_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT DummyCommandElement
	 Generic (
		Module_Name : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
		Module_HasMemory : STD_LOGIC := '1';
		-- Needs to be at least 1 => 
		Module_LengthInWord : integer := 1;
		Module_UsesZarlink : STD_LOGIC := '1'
	 );	 
    PORT(
         In_Data : IN  std_logic_vector(15 downto 0);
         In_Valid : IN  std_logic;
         In_Wait : OUT  std_logic;
         Packet_Start : IN  std_logic;
         Packet_End : IN  std_logic;
         Packet_Valid : IN  std_logic;
         Packet_Unknown : IN  std_logic;
         Zarlink_Data_Out : OUT  std_logic_vector(119 downto 0);
         Zarlink_WriteEn : OUT  std_logic;
         Zarlink_Data_In : IN  std_logic_vector(111 downto 0);
         Zarlink_Valid_In : IN  std_logic;
         Zarlink_BlockMode : OUT  std_logic;
         Zarlink_Wait : IN  std_logic;
         Zarlink_RequestChannel : OUT  std_logic;
         Zarlink_GetChannel : IN  std_logic;
         Network_Data : OUT  std_logic_vector(15 downto 0);
         Network_WriteEn : OUT  std_logic;
         Network_Wait : IN  std_logic;
         Network_RequestChannel : OUT  std_logic;
         Network_GetChannel : IN  std_logic;
         CRC_Reset : OUT  std_logic;
         CRC_00 : IN  std_logic_vector(15 downto 0);
         CRC_01 : IN  std_logic_vector(15 downto 0);
         CRC_10 : IN  std_logic_vector(15 downto 0);
         CRC_11 : IN  std_logic_vector(15 downto 0);
         Reset : IN  std_logic;
         Clk : IN  std_logic
        );
    END COMPONENT;
    
	COMPONENT CRC_Checker 
	Port ( 
		Network_Data				: in STD_LOGIC_VECTOR(15 downto 0); 
		Network_Valid				: in STD_LOGIC; 

		CRC_Reset					: in STD_LOGIC;
		CRC_00						: out 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_01						: out 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_10						: out 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_11						: out 	STD_LOGIC_VECTOR(15 downto 0);			
		
		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	);	 
	END COMPONENT;

   --Inputs
   signal In_Data : std_logic_vector(15 downto 0) := (others => '0');
   signal In_Valid : std_logic := '0';
   signal Packet_Start : std_logic := '0';
   signal Packet_End : std_logic := '0';
   signal Packet_Valid : std_logic := '0';
   signal Packet_Unknown : std_logic := '0';
   signal Zarlink_Data_In : std_logic_vector(111 downto 0) := (others => '0');
   signal Zarlink_Valid_In : std_logic := '0';
   signal Zarlink_Wait : std_logic := '0';
   signal Zarlink_GetChannel : std_logic := '0';
   signal Network_Wait : std_logic := '0';
   signal Network_GetChannel : std_logic := '0';
   signal CRC_00 : std_logic_vector(15 downto 0) := (others => '0');
   signal CRC_01 : std_logic_vector(15 downto 0) := (others => '0');
   signal CRC_10 : std_logic_vector(15 downto 0) := (others => '0');
   signal CRC_11 : std_logic_vector(15 downto 0) := (others => '0');
   signal Reset : std_logic := '0';
   signal Clk : std_logic := '0';

 	--Outputs
   signal In_Wait : std_logic;
   signal Zarlink_Data_Out : std_logic_vector(119 downto 0);
   signal Zarlink_WriteEn : std_logic;
   signal Zarlink_BlockMode : std_logic;
   signal Zarlink_RequestChannel : std_logic;
   signal Network_Data : std_logic_vector(15 downto 0);
   signal Network_WriteEn : std_logic;
   signal Network_RequestChannel : std_logic;
   signal CRC_Reset : std_logic;

	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 
BEGIN
 	-- Instantiate the Unit Under Test (UUT)
   uut: DummyCommandElement 
	GENERIC MAP(
		Module_Name => std_logic_vector(to_unsigned(257,16)),
		Module_HasMemory => '0',
		Module_LengthInWord => 1,
		Module_UsesZarlink => '1'	
	)
	PORT MAP (
          In_Data => In_Data,
          In_Valid => In_Valid,
          In_Wait => In_Wait,
          Packet_Start => Packet_Start,
          Packet_End => Packet_End,
          Packet_Valid => Packet_Valid,
          Packet_Unknown => Packet_Unknown,
          Zarlink_Data_Out => Zarlink_Data_Out,
          Zarlink_WriteEn => Zarlink_WriteEn,
          Zarlink_Data_In => Zarlink_Data_In,
          Zarlink_Valid_In => Zarlink_Valid_In,
          Zarlink_BlockMode => Zarlink_BlockMode,
          Zarlink_Wait => Zarlink_Wait,
          Zarlink_RequestChannel => Zarlink_RequestChannel,
          Zarlink_GetChannel => Zarlink_GetChannel,
          Network_Data => Network_Data,
          Network_WriteEn => Network_WriteEn,
          Network_Wait => Network_Wait,
          Network_RequestChannel => Network_RequestChannel,
          Network_GetChannel => Network_GetChannel,
          CRC_Reset => CRC_Reset,
          CRC_00 => CRC_00,
          CRC_01 => CRC_01,
          CRC_10 => CRC_10,
          CRC_11 => CRC_11,
          Reset => Reset,
          Clk => Clk
        );
		  
	CRC : CRC_Checker 
	Port MAP( 
			 Network_Data => Network_Data,
			 Network_Valid => Network_WriteEn,
          CRC_Reset => CRC_Reset,
          CRC_00 => CRC_00,
          CRC_01 => CRC_01,
          CRC_10 => CRC_10,
          CRC_11 => CRC_11,
          Reset => Reset,
          Clk => Clk
	);	 

TheMasterClock : process
begin
	clockloop: for i in 0 to 25000 loop
		clk <= NOT clk;
		wait for 4ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

TheSecondClock : process(clk) 
begin
	if rising_edge(clk) then

		Zarlink_GetChannel <= Zarlink_RequestChannel;
		Network_GetChannel <= Network_RequestChannel;
		
		MasterCounter <= MasterCounter + 1;
		
		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
		else
			Reset <= '0';
		end if;
		
		-- Default:
		Packet_Start <= '0';
		In_Data <= (others => '0');
		In_Valid <= '0';
		Packet_End <= '0';
		Packet_Valid <= '0';

		
		if MasterCounter = to_unsigned(10,128) then
			Packet_Start <= '1';
			In_Data <= std_logic_vector(to_unsigned(257,16));
			In_Valid <= '1';
		elsif MasterCounter = to_unsigned(15,128) then
			Packet_End <= '1';
			Packet_Valid <= '1';
		elsif MasterCounter = to_unsigned(42,128) then
			Packet_Start <= '1';
			In_Data <= std_logic_vector(to_unsigned(257,16));
			In_Valid <= '1';
		elsif MasterCounter = to_unsigned(43,128) then
			Packet_End <= '1';
			Packet_Valid <= '0';
		else
			NULL;
		end if;
					

	end if;
end process TheSecondClock;


END;
