--   ZestET1 Example 2
--   File name: Example2.vhd
--   Version: 1.00
--   Date: 26/7/2009
--
--   ZestET1 Example 2 - TCP test code.
--   Copies data to and from the Ethernet interface
--
--   Copyright (C) 2009 Orange Tree Technologies Ltd. All rights reserved.
--   Orange Tree Technologies grants the purchaser of a ZestET1 the right to use and 
--   modify this logic core in any form such as VHDL source code or EDIF netlist in 
--   FPGA designs that target the ZestET1.
--   Orange Tree Technologies prohibits the use of this logic core in any form such 
--   as VHDL source code or EDIF netlist in FPGA designs that target any other
--   hardware unless the purchaser of the ZestET1 has purchased the appropriate 
--   licence from Orange Tree Technologies. Contact Orange Tree Technologies if you 
--   want to purchase such a licence.
--
--  *****************************************************************************************
--  **
--  **  Disclaimer: LIMITED WARRANTY AND DISCLAIMER. These designs are
--  **              provided to you "as is". Orange Tree Technologies and its licensors 
--  **              make and you receive no warranties or conditions, express, implied, 
--  **              statutory or otherwise, and Orange Tree Technologies specifically 
--  **              disclaims any implied warranties of merchantability, non-infringement,
--  **              or fitness for a particular purpose. Orange Tree Technologies does not
--  **              warrant that the functions contained in these designs will meet your 
--  **              requirements, or that the operation of these designs will be 
--  **              uninterrupted or error free, or that defects in the Designs will be 
--  **              corrected. Furthermore, Orange Tree Technologies does not warrant or 
--  **              make any representations regarding use or the results of the use of the 
--  **              designs in terms of correctness, accuracy, reliability, or otherwise.                                               
--  **
--  **              LIMITATION OF LIABILITY. In no event will Orange Tree Technologies 
--  **              or its licensors be liable for any loss of data, lost profits, cost or 
--  **              procurement of substitute goods or services, or for any special, 
--  **              incidental, consequential, or indirect damages arising from the use or 
--  **              operation of the designs or accompanying documentation, however caused 
--  **              and on any theory of liability. This limitation will apply even if 
--  **              Orange Tree Technologies has been advised of the possibility of such 
--  **              damage. This limitation shall apply notwithstanding the failure of the 
--  **              essential purpose of any limited remedies herein.
--  **
--  *****************************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity Example2 is
    port (
        -- Programmable Clock
        CLK : in std_logic;
        
        -- Flash Interface
        Flash_CSOn : out std_logic;
        Flash_CLK : out std_logic;
        Flash_MOSI : out std_logic;
        Flash_MISO : in std_logic;

        -- IO connector
			SPI_ZL_CS : out std_logic;
			SPI_SDI : out std_logic;
			SPI_SDO : in std_logic;
			SPI_CLK : out std_logic;

			TriggerOut_Address 		: out STD_LOGIC_VECTOR(4 downto 0);
			TriggerOut_Data			: out STD_LOGIC;
			TriggerOut_Clk				: out STD_LOGIC;
		
			TriggerIn_Address 		: out STD_LOGIC_VECTOR(4 downto 0);
			TriggerIn_Data 			: in STD_LOGIC;

			ToZero 		: out STD_LOGIC_VECTOR(2 downto 0);
--			VGA_CS 	: out std_logic_vector(31 downto 0);
--			VGA_DATA : out std_logic_vector(1 downto 0);
--			VGA_CLK 	: out std_logic_vector(1 downto 0);
--
--			ADC_DATAREADY : in std_logic_vector(1 downto 0);
--			ADC_MCLK : out std_logic_vector(1 downto 0);
--			ADC_SCLK : out std_logic_vector(1 downto 0);
--			ADC_SYNC : out std_logic_vector(1 downto 0);
--			ADC_NOT_CS : out std_logic_vector(1 downto 0);
--			ADC_SDO  : in std_logic_vector(1 downto 0);
			
        -- Ethernet Interface
        Eth_Clk : out std_logic;
        Eth_CSn : out std_logic;
        Eth_WEn : out std_logic;
        Eth_A : out std_logic_vector(4 downto 0);
        Eth_D : inout std_logic_vector(15 downto 0);
        Eth_BE : out std_logic_vector(1 downto 0);
        Eth_Intn : in std_logic--;

--        -- DDR SDRAM interface
--        DS_CLK_P : out std_logic;
--        DS_CLK_N : out std_logic;
--        DS_CKE : out std_logic;
--        DS_A : out std_logic_vector(12 downto 0);
--        DS_BA : out std_logic_vector(1 downto 0);
--        DS_CAS_N : out std_logic;
--        DS_RAS_N : out std_logic;
--        DS_WE_N : out std_logic;
--        DS_DQS : inout std_logic_vector(1 downto 0);
--        DS_DM : out std_logic_vector(1 downto 0);
--        DS_DQ : inout std_logic_vector(15 downto 0)
		);
end Example2;

architecture arch of Example2 is

    --------------------------------------------------------------------------
    -- Declare constants
    
    -- GigExpedite register addresses
    constant LOCAL_IP_ADDR_LOW          : std_logic_vector(4 downto 0) := "00000";
    constant LOCAL_IP_ADDR_HIGH         : std_logic_vector(4 downto 0) := "00001";
    constant LINK_STATUS                : std_logic_vector(4 downto 0) := "00010";
    constant CONNECTION_PAGE            : std_logic_vector(4 downto 0) := "00111";
    constant LOCAL_PORT                 : std_logic_vector(4 downto 0) := "01000";
    constant REMOTE_IP_ADDR_LOW         : std_logic_vector(4 downto 0) := "01001";
    constant REMOTE_IP_ADDR_HIGH        : std_logic_vector(4 downto 0) := "01010";
    constant REMOTE_PORT                : std_logic_vector(4 downto 0) := "01011";
    constant MTU_TTL                    : std_logic_vector(4 downto 0) := "01100";
    constant INTERRUPT_ENABLE_STATUS    : std_logic_vector(4 downto 0) := "01101";
    constant CONNECTION_STATE           : std_logic_vector(4 downto 0) := "01110";
    constant FRAME_LENGTH               : std_logic_vector(4 downto 0) := "01111";
    constant DATA_FIFO                  : std_logic_vector(4 downto 0) := "10000";
    
    -- Connection states (for CONNECTION_STATE reg)
    constant CLOSED       : std_logic_vector(15 downto 0) := X"0000";
    constant LISTEN       : std_logic_vector(15 downto 0) := X"0001";
    constant CONNECT      : std_logic_vector(15 downto 0) := X"0002";
    constant ESTABLISHED  : std_logic_vector(15 downto 0) := X"0003";
    constant CONN_TCP     : std_logic_vector(15 downto 0) := X"0010";
    constant CONN_ENABLE  : std_logic_vector(15 downto 0) := X"0020";
    
    -- Interrupt enable and status bits (for INTERRUPT_ENABLE_STATUS reg)
    constant IE_INCOMING          : std_logic_vector(15 downto 0) := X"0001";
    constant IE_OUTGOING_EMPTY    : std_logic_vector(15 downto 0) := X"0002";
    constant IE_OUTGOING_NOT_FULL : std_logic_vector(15 downto 0) := X"0004";
    constant IE_STATE             : std_logic_vector(15 downto 0) := X"0008";

    -- Length of burst from GigExpedite (192 bytes)
    constant BURST_WORDS          : std_logic_vector(15 downto 0) := X"0060";

    -- Test transfer length
    constant WRITE_LENGTH         : std_logic_vector(31 downto 0) := conv_std_logic_vector(500*1024*1024, 32);

    --------------------------------------------------------------------------
    -- Declare signals
    
    -- Clocks and reset
    signal RST : std_logic;
    signal RAMRST : std_logic;
    signal RAMClk : std_logic;
    signal RAMClk90 : std_logic;
    signal EthClk : std_logic;

    -- Ethernet interface state machine
    signal UserWE : std_logic;
    signal UserRE : std_logic;
    signal UserAddr : std_logic_vector(4 downto 0);
    signal UserBE : std_logic_vector(1 downto 0);
    signal UserWriteData : std_logic_vector(15 downto 0);
    signal UserReadData : std_logic_vector(15 downto 0);
    signal UserReadDataValid : std_logic;
    signal UserInterrupt : std_logic;
    signal Delay : std_logic_vector(23 downto 0);
    signal UserFPGASubState : std_logic_vector(15 downto 0);
    signal UserValidCount : std_logic_vector(15 downto 0);
    signal ConnectionState : std_logic_vector(5 downto 0);
    signal InterruptStatus : std_logic_vector(15 downto 0);
    signal FrameLength : std_logic_vector(15 downto 0);
    signal Clean : std_logic;
    signal LatencyDelay : std_logic_vector(3 downto 0);

    type STATE_TYPE is (
        USER_FPGA_DELAY_INIT,
        USER_FPGA_CLEAN,
        USER_FPGA_CLEAN_CHECK,
        USER_FPGA_INIT,
        USER_FPGA_IDLE,
        USER_FPGA_CHECK_STATE,
        USER_FPGA_CHECK_SPACE,
        USER_FPGA_WRITE_DATA,
        USER_FPGA_READ_LENGTH,
        USER_FPGA_READ_DATA
    );
    signal UserFPGAState : STATE_TYPE;

-- > Change 
	 COMPONENT FIFO_IO_Network
	 PORT (
		rst : IN STD_LOGIC;
		wr_clk : IN STD_LOGIC;
		rd_clk : IN STD_LOGIC;
		din : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		wr_en : IN STD_LOGIC;
		rd_en : IN STD_LOGIC;
		dout : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		full : OUT STD_LOGIC;
		almost_full : OUT STD_LOGIC;
		empty : OUT STD_LOGIC;
		almost_empty : OUT STD_LOGIC;
		valid : OUT STD_LOGIC;
		prog_full : OUT STD_LOGIC
	 );
	 END COMPONENT;

    signal FIFO_Command_Compiler_Pipe_din : STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '0');
    signal FIFO_Command_Compiler_Pipe_wr_en : STD_LOGIC := '0';
    signal FIFO_Command_Compiler_Pipe_almost_full : STD_LOGIC := '0';
	 signal FIFO_Compile_rd_en : STD_LOGIC := '0';
	 signal FIFO_Compile_dout : STD_LOGIC_VECTOR(29 DOWNTO 0) := (others => '0');
	 signal FIFO_Compile_valid : STD_LOGIC := '0';	
	
    signal FIFO_IN_din : STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '0');
    signal FIFO_IN_wr_en : STD_LOGIC := '0';
    signal FIFO_IN_rd_en : STD_LOGIC := '0';
    signal FIFO_IN_dout : STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '0');
    signal FIFO_IN_almost_full : STD_LOGIC := '0';
    signal FIFO_IN_empty : STD_LOGIC  := '0';
    signal FIFO_IN_almost_empty : STD_LOGIC := '0';
    signal FIFO_IN_valid : STD_LOGIC := '0';
	 signal FIFO_IN_prog_full : STD_LOGIC := '0';

    signal FIFO_OUT_din : STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '0');
    signal FIFO_OUT_wr_en : STD_LOGIC := '0';
    signal FIFO_OUT_rd_en : STD_LOGIC := '0';
    signal FIFO_OUT_dout : STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '0');
    signal FIFO_OUT_almost_full : STD_LOGIC := '0';
    signal FIFO_OUT_empty : STD_LOGIC  := '0';
    signal FIFO_OUT_almost_empty : STD_LOGIC := '0';
    signal FIFO_OUT_valid : STD_LOGIC := '0';
	 signal FIFO_OUT_prog_full : STD_LOGIC := '0';


	 signal Clk_Processing : STD_LOGIC := '0';



	signal ADC_NOT_DATAREADY : std_logic_vector(1 downto 0) := (others => '0');
	signal ADC_Clk_OUT : std_logic_vector(1 downto 0) := (others => '0');	
	signal Clk1024KHz : std_logic := '0';
	
-- NEEDS TO BE EXPORTED !!!!!!!!!!!!1
	signal ADC_DATA_IN : std_logic_vector(1 downto 0):= (others => '0');

	signal VGA_CS 	: std_logic_vector(31 downto 0) := (others => '0');
	signal VGA_DATA : std_logic_vector(1 downto 0) := (others => '0');
	signal VGA_CLK 	: std_logic_vector(1 downto 0) := (others => '0');

	signal ADC_DATAREADY : std_logic_vector(1 downto 0) := (others => '0');
	signal ADC_MCLK : std_logic_vector(1 downto 0) := (others => '0');
	signal ADC_SCLK : std_logic_vector(1 downto 0) := (others => '0');
	signal ADC_SYNC : std_logic_vector(1 downto 0) := (others => '0');

-- < Change 

begin

-- < Change 
	ADC_DATAREADY <= "00";
	ADC_NOT_DATAREADY <= ADC_DATAREADY;
	ADC_SCLK <= ADC_Clk_OUT;
	ADC_MCLK <= Clk1024KHz & Clk1024KHz;
	ToZero <= (others => '0');
	
--	ADC_NOT_CS <= "00";
--	ADC_DATA_IN <= ADC_SDO;
	
	MyClockFrom125MHzDownto95_8333MHz: entity work.ClockFrom125MHzDownto95_8333MHzUB 
	PORT MAP(
		CLKIN_IN => EthClk,
		RST_IN => RAMRST,
		CLKFX_OUT => Clk_Processing,
		CLKIN_IBUFG_OUT => open,
		CLK0_OUT => open,
		LOCKED_OUT => open 
	);

FIFO_IN : FIFO_IO_NETWORK
  PORT MAP (
    wr_clk => EthClk,
	 rd_clk => Clk_Processing,
    rst => RAMRST,
    din => FIFO_IN_din,
    wr_en => FIFO_IN_wr_en,
    rd_en => FIFO_IN_rd_en,
    dout => FIFO_IN_dout,
    full => open,
    almost_full => FIFO_IN_almost_full,
    empty => FIFO_IN_empty,
    almost_empty => FIFO_IN_almost_empty,
    valid => FIFO_IN_valid,
	 prog_full => FIFO_IN_prog_full
  );


FIFO_OUT: FIFO_IO_NETWORK
  PORT MAP (
    wr_clk => Clk_Processing,
    rd_clk => EthClk,
    rst => RAMRST,
    din => FIFO_OUT_din,
    wr_en => FIFO_OUT_wr_en,
    rd_en => FIFO_OUT_rd_en,
    dout => FIFO_OUT_dout,
    full => open,
    almost_full => FIFO_OUT_almost_full,
    empty => FIFO_OUT_empty,
    almost_empty => FIFO_OUT_almost_empty,
    valid => FIFO_OUT_valid,
	 prog_full => FIFO_OUT_prog_full
  );

  
 MyPacketFrontend : entity work.PacketFrontend 
    Port Map( 
		-- FIFO IN
		FIFO_IN_Data => FIFO_IN_dout,
		FIFO_IN_Valid => FIFO_IN_valid,
		FIFO_IN_rd_en => FIFO_IN_rd_en,

		-- Connection to the Zarlink 
		SPI_ZL_CS => SPI_ZL_CS,
		SPI_SDI => SPI_SDI,
		SPI_SDO => SPI_SDO,
		SPI_CLK => SPI_CLK,
		
		-- Connection to AMP
		VGA_CS => VGA_CS,
		VGA_DATA => VGA_DATA,
		VGA_CLK => VGA_CLK,
		
		-- Connection to the Network
		Network_Data => FIFO_OUT_din,
		Network_WriteEn => FIFO_OUT_wr_en,
		Network_Wait => FIFO_OUT_prog_full,
	
		-- Digital Trigger Channels 
		TriggerOut_Address => TriggerOut_Address,
		TriggerOut_Data => TriggerOut_Data,
		TriggerOut_Clk	=> TriggerOut_Clk,
		
		TriggerIn_Address => TriggerIn_Address,
		TriggerIn_Data => TriggerIn_Data,
		
		-- ADC Ports 
		ADC_DATA_IN => ADC_DATA_IN,
		ADC_NOT_DATAREADY => ADC_NOT_DATAREADY,
		ADC_Clk_OUT => ADC_Clk_OUT,
		ADC_SYNC => ADC_SYNC,
		Clk1024KHz => Clk1024KHz,
		Clk125MHz => EthClk,
		
		-- The usual ports 
		Reset => RAMRST,
		Clk => Clk_Processing
	);
			


-- > Change     
    -- Tie unused signals
    Flash_CSOn <= '1';
    Flash_CLK <= '1';
    Flash_MOSI <= '1';
    
    -- Reset controller
    ROCInst : ROC port map (O=>RST);
    
    --------------------------------------------------------------------------
    -- Instantiate clocks
    -- (Assumes default 125MHz reference clock)
    ClockInst : entity work.ZestET1_Clocks
        port map (
            RST => RST,
            RefClk => CLK,
            EthClk => EthClk,
            RAMRST => RAMRST,
            RAMClk => RAMClk,
            RAMClk90 => RAMClk90    
        );

    --------------------------------------------------------------------------
    -- Ethernet interface test code
    EthernetInst : entity work.ZestET1_Ethernet
        port map (
            User_CLK => EthClk,
            User_RST => RST,

            -- User interface
            User_WE => UserWE,
            User_RE => UserRE,
            User_Addr => UserAddr,
            User_Owner => X"00",
            User_WriteData => UserWriteData,
            User_BE => UserBE,
            User_ReadData => UserReadData,
            User_ReadDataValid => UserReadDataValid,
            --User_ValidOwner => ,
            User_Interrupt => UserInterrupt,
            
            -- Interface to GigExpedite
            Eth_Clk => Eth_Clk,
            Eth_CSn => Eth_CSn,
            Eth_WEn => Eth_WEn,
            Eth_A => Eth_A,
            Eth_D => Eth_D,
            Eth_BE => Eth_BE,
            Eth_Intn => Eth_Intn
        );

    -- State machine to read/write Ethernet
    -- Note: use RAMRST to keep the state machine in reset until 
    -- the DCMs are locked
    process (RAMRST, EthClk) begin
        if (RAMRST='1') then
            UserFPGAState <= USER_FPGA_DELAY_INIT;
            Delay <= (others=>'0');
            UserFPGASubState <= (others=>'0');
            UserValidCount <= (others=>'0');
            ConnectionState <= (others=>'0');
            InterruptStatus <= (others=>'0');
            FrameLength <= (others=>'0');
            Clean <= '1';
-- > Change 				
--            RampData <= X"0001";
--            WriteCount <= (others=>'0');
-- < Change 				
            LatencyDelay <= (others=>'0');
-- > Change 
				-- Do not read or write the fifos during the reset...
				FIFO_IN_wr_en <= '0';
				FIFO_OUT_rd_en <= '0';
				FIFO_IN_din <= (others => '0');
-- < Change				
				
        elsif (EthClk'event and EthClk='1') then
		  
-- > Change 
				-- Default is: do nothing with the fifos... 
				FIFO_IN_wr_en <= '0';
				FIFO_OUT_rd_en <= '0';
				FIFO_IN_din <= (others => '0');
-- < Change			  
            -- Counter of completed register reads
            if (UserReadDataValid='1') then
                UserValidCount <= UserValidCount + 1;
            end if;
            
            -- Interrupt status latency counter
            if (LatencyDelay/=X"0") then
                LatencyDelay <= LatencyDelay - 1;
            end if;
            
            case (UserFPGAState) is
                -- Wait here to ensure the GigExpedite has locked to
                -- our clock
                when USER_FPGA_DELAY_INIT =>
                    Delay <= Delay + 1;
                    if (Delay=X"FFFFFF") then
                        UserFPGAState <= USER_FPGA_CLEAN;
                    end if;
                
                -- Reset all connections in case the previous
                -- application closed unexpectedly and left some open
                -- Write 0 to each connection state then
                -- read the status for each connection in turn
                -- Loop round until all statuses are zero.
                when USER_FPGA_CLEAN =>
                    UserFPGASubState <= UserFPGASubState + 1;
                    if (UserFPGASubState=X"001f") then
                        UserFPGASubState <= (others=>'0');
                        UserFPGAState <= USER_FPGA_CLEAN_CHECK;
                        Clean <= '1';
                    end if;
                when USER_FPGA_CLEAN_CHECK =>
                    UserFPGASubState <= UserFPGASubState + 1;
                    if (UserReadDataValid='1' and UserReadData/=X"0000") then
                        Clean <= '0';
                    end if;
                    if (UserFPGASubState=X"001ff") then
                        if (Clean='1') then
                            UserFPGASubState <= (others=>'0');
                            UserValidCount <= (others=>'0');
                            UserFPGAState <= USER_FPGA_INIT;
                        else
                            Clean <= '1';
                            UserFPGASubState <= (others=>'0');
                        end if;
                    end if;
                
                -- Initialise register set
                -- Configures one connection as a TCP server waiting
                -- for a connection from a client (i.e. the host program)
                when USER_FPGA_INIT => 
                    UserFPGASubState <= UserFPGASubState + 1;
                    if (UserFPGASubState=X"0004") then
                        UserFPGAState <= USER_FPGA_IDLE;
                    end if;
                
                -- Wait for interrupt from GigExpedite device
                when USER_FPGA_IDLE => 
                    UserFPGASubState <= (others=>'0');
                    UserValidCount <= (others=>'0');
                    if (UserInterrupt='1') then
                        -- Interrupt has been received
                        UserFPGAState <= USER_FPGA_CHECK_STATE;
                    end if;

                -- Check if the connection state has changed
                when USER_FPGA_CHECK_STATE => 
                    UserFPGASubState <= UserFPGASubState + 1;
                    if (UserReadDataValid='1' and UserValidCount(0)='0') then
                        -- Store the interrupt status bits
                        InterruptStatus <= UserReadData;
                    end if;
                    if (UserReadDataValid='1' and UserValidCount(0)='1') then
                        -- Store the new connection state
                        ConnectionState <= UserReadData(5 downto 0);

                        -- Next, check if there is incoming data available
                        UserFPGASubState <= (others=>'0');
                        UserValidCount <= (others=>'0');
-- > Change
									-- If the fifo has not room for at least 512 elements, don't try to read
--                        if (LatencyDelay/=X"0" or InterruptStatus(0)='0') then
									if (FIFO_IN_prog_full = '1') OR (LatencyDelay/=X"0" or InterruptStatus(0)='0') then
-- < Change								
                            -- Loopback FIFO is nearly full or there is no data available
                            -- Next, check if there is outgoing data to send
                            UserFPGAState <= USER_FPGA_CHECK_SPACE;
                        else
                            -- Read frame length or next burst
                            if (FrameLength=X"0000") then
                                UserFPGAState <= USER_FPGA_READ_LENGTH;
                            else
                                UserFPGAState <= USER_FPGA_READ_DATA;
                            end if;
                        end if;
                    end if;

                -- Check if there is incoming data
                when USER_FPGA_READ_LENGTH =>
                    UserFPGASubState <= UserFPGASubState + 1;
                    if (UserReadDataValid='1' and UserValidCount=X"0000") then
                        -- Read frame length from GigExpedite
                        -- Round the number of bytes up to the total number
                        -- of 16 bit reads we will need to do
                        UserFPGASubState <= (others=>'0');
                        UserValidCount <= (others=>'0');
                        FrameLength <= ('0'&UserReadData(15 downto 1)) + UserReadData(0);
 
								if (UserReadData=conv_std_logic_vector(0, 16)) then
                            -- Length was zero - skip the read
                            UserFPGAState <= USER_FPGA_IDLE;
                        else
                            -- Got a valid frame length - read the data
                            UserFPGAState <= USER_FPGA_READ_DATA;
                        end if;
                    end if;
                
                -- Read data from GigExpedite device
                when USER_FPGA_READ_DATA => 

						UserFPGASubState <= UserFPGASubState + 1;

-- > Change 
						-- If there is valid data then copy it into the fifo_in
						FIFO_IN_din <= UserReadData;
						FIFO_IN_wr_en <= UserReadDataValid;							  

						-- If the FIFO_IN is nearly full, stop reading! And continue with writing. 
						if (FIFO_IN_almost_full = '1') then
                        LatencyDelay <= X"F";
                        FrameLength <= (others=>'0');
                        UserFPGASubState <= (others=>'0');
                        UserValidCount <= (others=>'0');
                        UserFPGAState <= USER_FPGA_CHECK_SPACE;
							
--                    if (UserValidCount>=FrameLength) then
                    elsif (UserValidCount>=FrameLength) then
-- < Change 								

                        -- End of frame reached
                        -- There is a latency delay on the interrupt
                        -- status at the end of a frame
                        LatencyDelay <= X"F";
                        FrameLength <= (others=>'0');
                        UserFPGASubState <= (others=>'0');
                        UserValidCount <= (others=>'0');
                        UserFPGAState <= USER_FPGA_CHECK_SPACE;
                    elsif (UserValidCount>=BURST_WORDS) then
                        -- End of burst reached
                        FrameLength <= FrameLength - BURST_WORDS;
                        UserFPGASubState <= (others=>'0');
                        UserValidCount <= (others=>'0');
                        UserFPGAState <= USER_FPGA_IDLE;
                    end if;
                
                -- Check if there is space in the outgoing GigExpedite buffer
                -- and we have data to send back to the host
                when USER_FPGA_CHECK_SPACE => 
                    UserFPGASubState <= UserFPGASubState + 1;
-- > Change 								

--                    if (WriteCount=WRITE_LENGTH or
--                        ConnectionState(3 downto 0)/=ESTABLISHED(3 downto 0) or
--                        InterruptStatus(2)='0') then
							-- If there is nothing to write, then don't try to write... 
                    if ((FIFO_OUT_empty = '1') or
                        (ConnectionState(3 downto 0)/=ESTABLISHED(3 downto 0)) or
                        (InterruptStatus(2)='0')) then
-- < Change 								

                        -- Already sent all our data or the 
                        -- connection from the host has not been made
                        -- or there is no space in the GigExpedite buffer
                        UserFPGASubState <= (others=>'0');
                        UserValidCount <= (others=>'0');
                        UserFPGAState <= USER_FPGA_IDLE;

                    else
                        -- OK to write data to GigExpedite FIFO
                        UserFPGASubState <= (others=>'0');
                        UserValidCount <= (others=>'0');
                        UserFPGAState <= USER_FPGA_WRITE_DATA;
-- > Change 
								-- FIFO_OUT is a First Word Fall Through FIFO
								-- Since we don't want to write the First Value
								-- twice. Start the request for a new value now. 
								FIFO_OUT_rd_en <= '1';
-- < Change 										
                    end if;
                
                -- Write 1kbyte data to outgoing FIFO
                when USER_FPGA_WRITE_DATA =>
					 
					 UserFPGASubState <= UserFPGASubState + 1;
-- > Change 
								
--                    RampData(7 downto 0) <= RampData(7 downto 0) + 2;
--                    RampData(15 downto 8) <= RampData(15 downto 8) + 2;
--                    WriteCount <= WriteCount + 2;
										

--                    if (UserFPGASubState=conv_std_logic_vector(511, 16)) then
							-- If there is nothing left to write or end of the 512 word block is reached
							-- then go back to idle state 
                    if (FIFO_OUT_empty = '1') OR (UserFPGASubState=conv_std_logic_vector(511, 16)) then
-- < Change 

                        UserFPGASubState <= (others=>'0');
                        UserValidCount <= (others=>'0');
                        UserFPGAState <= USER_FPGA_IDLE;
                    end if;

-- > Change 						  
								-- If the end of the 512 word block is reached then don't continue reading, 
								-- otherwise one word is lost. Theoretically the same could happen if the
								-- the fifo is empty... 
								if (FIFO_OUT_empty = '1') OR (UserFPGASubState=conv_std_logic_vector(511, 16)) then
									FIFO_OUT_rd_en <= '0';
								else
									FIFO_OUT_rd_en <= '1';
								end if;
-- < Change 
						  
                when others => null;
            end case;
        end if;
    end process;
    
    -- Map states to Ethernet register accesses
    process (UserFPGAState, UserFPGASubState, UserReadDataValid,
             UserValidCount, UserReadData, ConnectionState,
             FrameLength) begin
        case UserFPGAState is
            when USER_FPGA_CLEAN =>
                -- Reset all connections
                UserRE <= '0';
                UserWE <= '1';
                UserBE <= "11";
                if (UserFPGASubState(0)='0') then 
                    UserAddr <= CONNECTION_PAGE;
                    UserWriteData <= X"000" & UserFPGASubState(4 downto 1);
                else
                    UserAddr <= CONNECTION_STATE;
                    UserWriteData <= (others=>'0');
                end if;
            
            when USER_FPGA_CLEAN_CHECK =>
                -- Check all connections have been reset
                if (UserFPGASubState(4 downto 0)="10000") then
                    UserRE <= '1';
                else
                    UserRE <= '0';
                end if;
                if (UserFPGASubState(4 downto 0)="00000") then
                    UserWE <= '1';
                    UserAddr <= CONNECTION_PAGE;
                else
                    UserWE <= '0';
                    UserAddr <= CONNECTION_STATE;
                end if;
                UserBE <= "11";
                UserWriteData <= X"000" & UserFPGASubState(8 downto 5);
            
            when USER_FPGA_INIT => 
                -- Set up registers to make one connection listen on
                -- port 0x5002 for TCP connections
                UserRE <= '0';
                UserWE <= '1';
                UserBE <= "11";
                case UserFPGASubState(4 downto 0) is
                    when "00000" =>
                        UserAddr <= CONNECTION_PAGE;
                        UserWriteData <= (others=>'0');
                    when "00001" =>
                        UserAddr <= LOCAL_PORT;
                        UserWriteData <= X"5002";
                    when "00010" => 
                        UserAddr <= MTU_TTL;
                        UserWriteData <= X"8080";
                    when "00011" =>
                        UserAddr <= INTERRUPT_ENABLE_STATUS;
                        UserWriteData <= (IE_OUTGOING_NOT_FULL or IE_INCOMING or IE_STATE);
                    when others =>
                        UserAddr <= CONNECTION_STATE;
                        UserWriteData <= (CONN_TCP or CONN_ENABLE or LISTEN);
                end case;
                
            when USER_FPGA_CHECK_STATE => 
                -- Read connection state then update accordingly
                -- The only important state change is to ESTABLISHED which
                -- we must acknowledge by changing our state to ESTABLISHED.
                -- All other state changes result in us returning to LISTEN
                -- to wait for another connection from a new client.
                if (UserFPGASubState(3 downto 1)="000") then 
                    UserRE <= '1';
                else
                    UserRE <= '0';
                end if;
                if (UserReadDataValid='1' and UserValidCount(0)='1' and
                    UserReadData(5 downto 0)/=ConnectionState) then
                    UserWE <= '1';
                else
                    UserWE <= '0';
                end if;
                UserBE <= "11";
                if (UserFPGASubState(3 downto 0)=X"0") then
                    UserAddr <= INTERRUPT_ENABLE_STATUS;
                else
                    UserAddr <= CONNECTION_STATE;
                end if;
                if (UserReadData(3 downto 0)=ESTABLISHED(3 downto 0)) then
                    UserWriteData <= (CONN_TCP or CONN_ENABLE or ESTABLISHED);
                else
                    UserWriteData <= (CONN_TCP or CONN_ENABLE or LISTEN);
                end if;
            
            when USER_FPGA_READ_LENGTH => 
                -- Read from interrupt status and then frame length register
                if (UserFPGASubState(3 downto 0)=X"0") then
                    UserRE <= '1';
                else
                    UserRE <= '0';
                end if;
                UserWE <= '0';
                UserBE <= "11";
                UserAddr <= FRAME_LENGTH;
                UserWriteData <= (others=>'0');
                
            when USER_FPGA_READ_DATA => 
                -- Read from connection FIFO
                if (UserFPGASubState<FrameLength and
                    UserFPGASubState<BURST_WORDS) then
                    UserRE <= '1';
                else
                    UserRE <= '0';
                end if;
                UserWE <= '0';
                UserBE <= "11";
                UserAddr <= DATA_FIFO;
                UserWriteData <= (others=>'0');

            when USER_FPGA_CHECK_SPACE =>
                -- No access necessary
                UserRE <= '0';
                UserWE <= '0';
                UserBE <= "11";
                UserAddr <= (others=>'0');
                UserWriteData <= (others=>'0');
                
            when USER_FPGA_WRITE_DATA => 
                -- Write test data to connection FIFO

                UserRE <= '0';
-- > Change 
                -- UserWE <= '1';
					 -- If the fifo_out output is valid, then write it away
					 UserWE <= FIFO_OUT_valid;
-- < Change                 
					 
                UserBE <= "11";
                UserAddr <= DATA_FIFO;
-- > Change 
					-- Assign the data from the fifo_out
					 --UserWriteData <= RampData;
					 UserWriteData <= FIFO_OUT_dout;
-- < Change                 
            when others =>
                -- Don't do anything
                UserRE <= '0';
                UserWE <= '0';
                UserAddr <= (others=>'0');
                UserBE <= "11";
                UserWriteData <= (others=>'0');
            
        end case;
    end process;

    --------------------------------------------------------------------------
    -- SDRAM Buffer
    -- Place holder - not used in the example but ties the pins correctly
    
--    -- Instantiate SDRAM component
--    SDRAMInst : entity work.ZestET1_SDRAM
--        port map (
--            User_CLK => RAMClk,
--            User_CLK90 => RAMClk90,
--            User_RST => RAMRST,
--
--            User_A => (others=>'0'),
--            User_RE => '0',
--            User_WE => '0',
--            User_Owner => X"00",
--            User_BE => (others=>'0'),
--            User_DW => (others=>'0'),
--            --User_DR => ,
--            --User_DR_Valid => ,
--            --User_ValidOwner => ,
--            --User_Busy => ,
--            --User_InitDone => ,
--
--            DS_CLK_P => DS_CLK_P,
--            DS_CLK_N => DS_CLK_N,
--            DS_A => DS_A,
--            DS_BA => DS_BA,
--            DS_DQ => DS_DQ,
--            DS_DQS => DS_DQS,
--            DS_DM => DS_DM,
--            DS_CAS_N => DS_CAS_N,
--            DS_RAS_N => DS_RAS_N,
--            DS_CKE => DS_CKE,
--            DS_WE_N => DS_WE_N
--        );
    
end arch;

