library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

 
ENTITY ExternalADCArbiter_UUT IS
END ExternalADCArbiter_UUT;
 
ARCHITECTURE behavior OF ExternalADCArbiter_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT ExternalADCArbiter
    PORT(
         DATA_IN : IN  std_logic_vector(1 downto 0);
         NOT_DATAREADY : IN  std_logic_vector(1 downto 0);
         Clk_OUT : OUT  std_logic_vector(1 downto 0);
         ADC_SYNC : OUT  std_logic_vector(1 downto 0);
         ADC_ChannelMask : IN  std_logic_vector(31 downto 0);
         ChannelMask_Count : IN  std_logic_vector(15 downto 0);
         ChannelMask_Ready : IN  std_logic;
         FIFO_PlusFullPacket : OUT  std_logic;
         FIFO_wr_en : OUT  std_logic;
         FIFO_din : OUT  std_logic_vector(16 downto 0);
         FIFO_FULL : IN  std_logic;
         FIFO_Error_Overrun : IN  std_logic;
         FIFO_Error_Underrun : IN  std_logic;
         Timestamp : IN  std_logic_vector(15 downto 0);
         TriggerChannelsIn : IN  std_logic_vector(15 downto 0);
         Enable : IN  std_logic;
         Reset : IN  std_logic;
         Clk : IN  std_logic
        );
    END COMPONENT;
    

   --Inputs
   signal DATA_IN : std_logic_vector(1 downto 0) := "01";
   signal NOT_DATAREADY : std_logic_vector(1 downto 0) := (others => '0');
   signal ADC_ChannelMask : std_logic_vector(31 downto 0) := "00000000000000100000000000000000";
   signal ChannelMask_Count : std_logic_vector(15 downto 0) := std_logic_vector(to_unsigned(1,16));
   signal ChannelMask_Ready : std_logic := '1';
   signal FIFO_FULL : std_logic := '0';
   signal FIFO_Error_Overrun : std_logic := '0';
   signal FIFO_Error_Underrun : std_logic := '0';
   signal Timestamp : std_logic_vector(15 downto 0) := (others => '0');
   signal TriggerChannelsIn : std_logic_vector(15 downto 0) := (others => '0');
   signal Enable : std_logic := '1';
   signal Reset : std_logic := '0';
   signal Clk : std_logic := '0';

 	--Outputs
   signal Clk_OUT : std_logic_vector(1 downto 0);
   signal ADC_SYNC : std_logic_vector(1 downto 0);
   signal FIFO_PlusFullPacket : std_logic;
   signal FIFO_wr_en : std_logic;
   signal FIFO_din : std_logic_vector(16 downto 0);

  	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 
	signal SlowClock : std_logic := '0';
	signal CounterX : unsigned(7 downto 0) := (others => '0');
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: ExternalADCArbiter PORT MAP (
          DATA_IN => DATA_IN,
          NOT_DATAREADY => NOT_DATAREADY,
          Clk_OUT => Clk_OUT,
          ADC_SYNC => ADC_SYNC,
          ADC_ChannelMask => ADC_ChannelMask,
          ChannelMask_Count => ChannelMask_Count,
          ChannelMask_Ready => ChannelMask_Ready,
          FIFO_PlusFullPacket => FIFO_PlusFullPacket,
          FIFO_wr_en => FIFO_wr_en,
          FIFO_din => FIFO_din,
          FIFO_FULL => FIFO_FULL,
          FIFO_Error_Overrun => FIFO_Error_Overrun,
          FIFO_Error_Underrun => FIFO_Error_Underrun,
          Timestamp => Timestamp,
          TriggerChannelsIn => TriggerChannelsIn,
          Enable => Enable,
          Reset => Reset,
          Clk => Clk
        );


     -- Clock process definitions
TheMasterClock : process
begin
	clockloop: for i in 0 to 50000 loop
		clk <= NOT clk;
		wait for 4ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

TheSlowClock : process(SlowClock) 
begin
	if rising_edge(SlowClock) then
		DATA_IN <= not DATA_IN;
	end if;
end process TheSlowClock;

TheSecondClock : process(clk) 
begin
	if rising_edge(clk) then
		
		

		MasterCounter <= MasterCounter + 1;
		
		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
			CounterX <= (others => '0');
			SlowClock <= '0';
		else
			Reset <= '0';
			
			if CounterX < to_unsigned(5,8) then
				CounterX <= CounterX + 1;
				SlowClock <= SlowClock;
			else
				CounterX <= (others => '0');			
				SlowClock <= not SlowClock;
			end if;
		end if;
		
		NOT_DATAREADY <= "00";
		
		if MasterCounter = to_unsigned(10,128) then
			null;
		elsif MasterCounter = to_unsigned(11,128) then
			NOT_DATAREADY <= "10";
		elsif MasterCounter = to_unsigned(12,128) then
			NOT_DATAREADY <= "11";
		elsif MasterCounter = to_unsigned(13,128) then
			NOT_DATAREADY <= "11";
		elsif MasterCounter = to_unsigned(14,128) then
			NOT_DATAREADY <= "11";
		elsif MasterCounter = to_unsigned(10800,128) then
			NOT_DATAREADY <= "10";
		elsif MasterCounter = to_unsigned(10801,128) then
			NOT_DATAREADY <= "11";
		elsif MasterCounter = to_unsigned(10802,128) then
			NOT_DATAREADY <= "11";
		elsif MasterCounter = to_unsigned(10803,128) then
			NOT_DATAREADY <= "11";
		else
			null;
		end if;

	end if;
end process TheSecondClock;

 

END;
