library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
 
ENTITY InputPacketInterpreter_UUT IS
END InputPacketInterpreter_UUT;
 
ARCHITECTURE behavior OF InputPacketInterpreter_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT InputPacketInterpreter
    PORT(
         FIFO_IN_Data : IN  std_logic_vector(15 downto 0);
         FIFO_IN_Valid : IN  std_logic;
         FIFO_IN_rd_en : OUT  std_logic;
         FIFO_OUT_Data : OUT  std_logic_vector(15 downto 0);
         FIFO_OUT_Valid : OUT  std_logic;
         Packet_Start : OUT  std_logic;
         Packet_End : OUT  std_logic;
         Packet_Valid : OUT  std_logic;
			Packet_Unknown	: OUT STD_LOGIC;
         WaitForListener : IN  std_logic_vector(0 downto 0);
         Reset : IN  std_logic;
         Clk : IN  std_logic
        );
    END COMPONENT;

COMPONENT FIFO_16Bit_1000BlockCheck
  PORT (
    rst : IN STD_LOGIC;
    wr_clk : IN STD_LOGIC;
    rd_clk : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
    wr_en : IN STD_LOGIC;
    rd_en : IN STD_LOGIC;
    dout : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    full : OUT STD_LOGIC;
    almost_full : OUT STD_LOGIC;
    empty : OUT STD_LOGIC;
    almost_empty : OUT STD_LOGIC;
    valid : OUT STD_LOGIC;
    prog_full : OUT STD_LOGIC
  );
END COMPONENT;

   signal din : STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '0');
	signal wr_en : STD_LOGIC := '0';
	
   --Inputs
   signal FIFO_IN_Data : std_logic_vector(15 downto 0) := (others => '0');
   signal FIFO_IN_Valid : std_logic := '0';
   signal WaitForListener : std_logic_vector(0 downto 0) := (others => '0');
   signal Reset : std_logic := '0';
   signal Clk : std_logic := '0';

 	--Outputs
   signal FIFO_IN_rd_en : std_logic;
   signal FIFO_OUT_Data : std_logic_vector(15 downto 0);
   signal FIFO_OUT_Valid : std_logic;
   signal Packet_Start : std_logic;
   signal Packet_End : std_logic;
   signal Packet_Valid : std_logic;
	signal Packet_Unknown : STD_LOGIC;

   -- Clock period definitions
	signal Counter : unsigned(10 downto 0) := (others=>'0'); 
	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 
 
	signal PacketType : integer := 1;
	
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: InputPacketInterpreter PORT MAP (
          FIFO_IN_Data => FIFO_IN_Data,
          FIFO_IN_Valid => FIFO_IN_Valid,
          FIFO_IN_rd_en => FIFO_IN_rd_en,
          FIFO_OUT_Data => FIFO_OUT_Data,
          FIFO_OUT_Valid => FIFO_OUT_Valid,
          Packet_Start => Packet_Start,
          Packet_End => Packet_End,
          Packet_Valid => Packet_Valid,
			 Packet_Unknown => Packet_Unknown,
          WaitForListener => WaitForListener,
          Reset => Reset,
          Clk => Clk
        );
		  
   -- Clock process definitions
TheMasterClock : process
begin
	clockloop: for i in 0 to 25000 loop
		clk <= NOT clk;
		wait for 4ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

TheSecondClock : process(clk) 
begin
	if rising_edge(clk) then
		
		WaitForListener <= (others => '0');
		wr_en <= '0';
		din <= (others => '0');

		MasterCounter <= MasterCounter + 1;
		
		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
		else
			Reset <= '0';
		end if;

		case PacketType is

			-- Defect Packet 0101
			when 0 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(100,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;

			-- Correct Packet 0101
			when 1 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(257,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;

			-- Correct Packet 0102
			when 2 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(18,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(258,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(0,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(0,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						din <= std_logic_vector(to_unsigned(28,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						din <= std_logic_vector(to_unsigned(0,16));
						wr_en <= '1';
					else
						NULL;
					end if;
					
			when others =>
				NULL;
		end case;

	end if;
end process TheSecondClock;

Test_Fifo : FIFO_16Bit_1000BlockCheck
  PORT MAP (
    rst => Reset,
    wr_clk => Clk,
    rd_clk => Clk,
    din => din,
    wr_en => wr_en,
    rd_en => FIFO_IN_rd_en,
    dout => FIFO_IN_Data,
    full => open,
    almost_full => open,
    empty => open,
    almost_empty => open,
    valid => FIFO_IN_Valid,
    prog_full => open
  );

END;
