library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

ENTITY MyPacketFrintend_UUT IS
END MyPacketFrintend_UUT;
 
ARCHITECTURE behavior OF MyPacketFrintend_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
	component PacketFrontend
    Port ( 
		-- FIFO IN
		FIFO_IN_Data 				: IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		FIFO_IN_Valid 				: IN STD_LOGIC;
		FIFO_IN_rd_en 				: OUT STD_LOGIC;			
		
		
		-- Connection to the Zarlink 
		SPI_ZL_CS 					: out STD_LOGIC;
		SPI_SDI 						: out STD_LOGIC;
		SPI_SDO 						: in 	STD_LOGIC;
		SPI_CLK 						: out STD_LOGIC;		

		-- Programm the AMP 
		VGA_CS 						: OUT std_logic_vector(31 downto 0);
		VGA_DATA 					: OUT std_logic_vector(1 downto 0);
		VGA_CLK 						: OUT std_logic_vector(1 downto 0);	
		
		-- Connection to the Network
		Network_Data				: out STD_LOGIC_VECTOR(15 downto 0); 
		Network_WriteEn			: out STD_LOGIC; 
		Network_Wait				: in  STD_LOGIC;

		-- Digital Trigger Channels 


		TriggerOut_Address 		: out STD_LOGIC_VECTOR(4 downto 0);
		TriggerOut_Data			: out STD_LOGIC;
		TriggerOut_Clk				: out STD_LOGIC;
		
		TriggerIn_Address 		: out STD_LOGIC_VECTOR(4 downto 0);
		TriggerIn_Data 			: in STD_LOGIC;
		

		-- ADC Ports 
		ADC_DATA_IN						: in std_logic_vector(1 downto 0);
		ADC_NOT_DATAREADY				: in std_logic_vector(1 downto 0);
		ADC_Clk_OUT						: out std_logic_vector(1 downto 0);
		ADC_SYNC 						: out std_logic_vector(1 downto 0);
		Clk1024KHz						: out std_logic;
		
		Clk125MHz						: in STD_LOGIC;
		
		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	);
	end COMPONENT;
    
COMPONENT FIFO_16Bit_1000BlockCheck
  PORT (
    rst : IN STD_LOGIC;
    wr_clk : IN STD_LOGIC;
    rd_clk : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
    wr_en : IN STD_LOGIC;
    rd_en : IN STD_LOGIC;
    dout : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    full : OUT STD_LOGIC;
    almost_full : OUT STD_LOGIC;
    empty : OUT STD_LOGIC;
    almost_empty : OUT STD_LOGIC;
    valid : OUT STD_LOGIC;
    prog_full : OUT STD_LOGIC
  );
  END COMPONENT;
  
  	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 
 

   --Inputs
   signal FIFO_IN_Data : std_logic_vector(15 downto 0) := (others => '0');
   signal FIFO_IN_Valid : std_logic := '0';
   signal SPI_SDO : std_logic := '0';
   signal Network_Wait : std_logic := '0';
   signal Clk8MHz : std_logic := '0';
   signal Reset : std_logic := '0';
   signal Clk : std_logic := '0';
	
	-- Programm the AMP 
	signal VGA_CS 	: std_logic_vector(31 downto 0);
	signal VGA_DATA : std_logic_vector(1 downto 0);
	signal VGA_CLK : std_logic_vector(1 downto 0);
		
 	--Outputs
   signal FIFO_IN_rd_en : std_logic;
   signal SPI_ZL_CS : std_logic;
   signal SPI_SDI : std_logic;
   signal SPI_CLK : std_logic;
   signal Network_Data : std_logic_vector(15 downto 0);
   signal Network_WriteEn : std_logic;

   signal din : STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '0');
	signal wr_en : STD_LOGIC := '0';
	
	signal PacketType : integer := 12; 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: PacketFrontend PORT MAP (
          FIFO_IN_Data => FIFO_IN_Data,
          FIFO_IN_Valid => FIFO_IN_Valid,
          FIFO_IN_rd_en => FIFO_IN_rd_en,
			 
          SPI_ZL_CS => SPI_ZL_CS,
          SPI_SDI => SPI_SDI,
          SPI_SDO => SPI_SDO,
          SPI_CLK => SPI_CLK,
			 
			 VGA_CS => VGA_CS,
			 VGA_DATA => VGA_DATA,
			 VGA_CLK => VGA_CLK,
			 
          Network_Data => Network_Data,
          Network_WriteEn => Network_WriteEn,
          Network_Wait => Network_Wait,

			TriggerOut_Address => open,
			TriggerOut_Data => open,
			TriggerOut_Clk => open,
			TriggerIn_Address => open,
			TriggerIn_Data => '0',
			
			ADC_DATA_IN => "00",
			ADC_NOT_DATAREADY => "00",
			ADC_Clk_OUT => open,
			ADC_SYNC => open,
			Clk1024KHz => open,

			Clk125MHz => Clk,

          Reset => Reset,
          Clk => Clk
        );


     -- Clock process definitions
TheMasterClock : process
begin
	clockloop: for i in 0 to 500000 loop
		clk <= NOT clk;
		wait for 4ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

     -- Clock process definitions
--The8MHzClock : process
--begin
--	clockloop: for i in 0 to 1600 loop
--		Clk8MHz <= NOT Clk8MHz;
--		wait for 63ns;
--	end loop clockloop;
--	wait;
--end process The8MHzClock; 

TheSecondClock : process(clk) 
begin
	if rising_edge(clk) then
		
		wr_en <= '0';
		din <= (others => '0');

		MasterCounter <= MasterCounter + 1;
		
		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
		else
			Reset <= '0';
		end if;

		case PacketType is

			-- Defect Packet 0101
			when 0 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(257,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						din <= std_logic_vector(to_unsigned(9,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(20,128) then
						din <= std_logic_vector(to_unsigned(257,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(21,128) then
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(22,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(23,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(24,128) then
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';						
					else
						NULL;
					end if;

			-- Correct Packet 0103 (Close RF Connection)
			when 1 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(259,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					else
						NULL;
					end if;

			-- Correct Packet 0273 (Set Sample Rate)
			when 2 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(627,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(97,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0272 (Set ASIC AD Resolution)
			when 3 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(626,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(97,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(7,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0266 (Set ASIC Filters)
			when 4 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(614,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(97,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(15,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(9,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(0,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0263 (Set Channelmask of ASIC)
			when 5 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(30,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(611,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						din <= std_logic_vector(to_unsigned(3,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						din <= std_logic_vector(to_unsigned(4,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(20,128) then
						din <= std_logic_vector(to_unsigned(7,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(21,128) then
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(22,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(62,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(23,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(24,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(10,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(25,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(4,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0404 (Block Write Zarlink)
			when 6 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(28,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(1028,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						din <= std_logic_vector(to_unsigned(3,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						din <= std_logic_vector(to_unsigned(4,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(20,128) then
						din <= std_logic_vector(to_unsigned(7,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(21,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(58,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(22,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(13,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(23,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(24,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(3,16));
						wr_en <= '1';
					else
						NULL;
					end if;					


			-- Correct Packet 0402 (Write register Page 0)
			when 7 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(1026,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(77,8)) & "0" & std_logic_vector(to_unsigned(33,7));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(9,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0403 (Write Register Page 1 )
			when 8 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(1027,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(77,8)) & "0" & std_logic_vector(to_unsigned(33,7));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(9,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0500 (Set trigger channels)
			when 9 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(1280,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(77,8)) & "0" & std_logic_vector(to_unsigned(33,7));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(10,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 027A (Set channel mask analog)
			when 10 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(18,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(634,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(77,8)) & "0" & std_logic_vector(to_unsigned(33,7));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(77,8)) & "0" & std_logic_vector(to_unsigned(33,7));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(15,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(10,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(3,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0400 (Read Register Page0 )
			when 11 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(1024,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(77,8)) & "0" & std_logic_vector(to_unsigned(33,7));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(17,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(9,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0401 (Read Register Page 1)
			when 12 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(1025,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(77,8)) & "0" & std_logic_vector(to_unsigned(33,7));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(10,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0405 (Block read )
			when 13 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(1029,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(11,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(7,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet Unknown
			when 14 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(8929,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(11,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(7,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0x0102 Open RF (dec258) (L2)
			when 15 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(18,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(258,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(40,8)) & std_logic_vector(to_unsigned(30,8));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(0,8)) & std_logic_vector(to_unsigned(50,8));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(21,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(7,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(10,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0x0501 Set Amp on Basisstation (L2)
			when 16 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(18,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(1281,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(4,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(200,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(24,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(9,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0x0301 Start Recoding L0
			when 17 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(769,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Packet 0x0301 Start & Stop 0x0302 the Recoding L0
			when 18 => 
					-- Start
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(769,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					-- Stop
					elsif MasterCounter = to_unsigned(15011,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15012,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15013,128) then
						din <= std_logic_vector(to_unsigned(770,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15016,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15017,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(4,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15018,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15019,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
						
					else
						NULL;
					end if;					

			-- Correct Packet 0x00FF Debug Constants
			when 19 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(22,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(255,16));
						wr_en <= '1';
					
					-- ID 
					elsif MasterCounter = to_unsigned(14,128) then
						--din <= std_logic_vector(to_unsigned(1,16));
						--din <= std_logic_vector(to_unsigned(2,16));
						din <= std_logic_vector(to_unsigned(3,16));
						--din <= std_logic_vector(to_unsigned(4,16));
						--din <= std_logic_vector(to_unsigned(5,16));
						--din <= std_logic_vector(to_unsigned(6,16));
						--din <= std_logic_vector(to_unsigned(7,16));
						--din <= std_logic_vector(to_unsigned(8,16));
						--din <= std_logic_vector(to_unsigned(9,16));
						--din <= std_logic_vector(to_unsigned(10,16));
						wr_en <= '1';
					-- Value LOW
					elsif MasterCounter = to_unsigned(15,128) then
					-- 1 to 7
						din <= "0101010101010100";
					-- 8 to 10 
						-- din <= "0101010101010111";
						wr_en <= '1';
					-- VALUE UP
					elsif MasterCounter = to_unsigned(16,128) then
						din <= "0101010101010101";
						wr_en <= '1';
						
					elsif MasterCounter = to_unsigned(17,128) then
						din <= "0101010101010101";
						wr_en <= '1';
						
					elsif MasterCounter = to_unsigned(18,128) then
						-- 00
						--din <= std_logic_vector(to_unsigned(18,16));
						--din <= std_logic_vector(to_unsigned(18,16));
						din <= std_logic_vector(to_unsigned(17,16));
						--din <= std_logic_vector(to_unsigned(18,16));
						--din <= std_logic_vector(to_unsigned(17,16));
						--din <= std_logic_vector(to_unsigned(17,16));
						--din <= std_logic_vector(to_unsigned(17,16));
						--din <= std_logic_vector(to_unsigned(17,16));
						--din <= std_logic_vector(to_unsigned(16,16));
						--din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						-- 01
						-- din <= std_logic_vector(to_unsigned(22,16));
						--din <= std_logic_vector(to_unsigned(21,16));
						din <= std_logic_vector(to_unsigned(29,16));
						--din <= std_logic_vector(to_unsigned(22,16));
						--din <= std_logic_vector(to_unsigned(23,16));
						--din <= std_logic_vector(to_unsigned(22,16));
						--din <= std_logic_vector(to_unsigned(22,16));
						--din <= std_logic_vector(to_unsigned(21,16));
						--din <= std_logic_vector(to_unsigned(22,16));
						--din <= std_logic_vector(to_unsigned(21,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(20,128) then
						-- 10
						-- din <= std_logic_vector(to_unsigned(4,16));
						--din <= std_logic_vector(to_unsigned(5,16));
						din <= std_logic_vector(to_unsigned(5,16));
						--din <= std_logic_vector(to_unsigned(4,16));
						--din <= std_logic_vector(to_unsigned(4,16));
						--din <= std_logic_vector(to_unsigned(5,16));
						--din <= std_logic_vector(to_unsigned(4,16));
						--din <= std_logic_vector(to_unsigned(5,16));
						--din <= std_logic_vector(to_unsigned(5,16));
						--din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(21,128) then
						-- 11
						--din <= std_logic_vector(to_unsigned(4,16));
						--din <= std_logic_vector(to_unsigned(4,16));
						din <= std_logic_vector(to_unsigned(5,16));
						--din <= std_logic_vector(to_unsigned(4,16));
						--din <= std_logic_vector(to_unsigned(4,16));
						--din <= std_logic_vector(to_unsigned(4,16));
						--din <= std_logic_vector(to_unsigned(5,16));
						--din <= std_logic_vector(to_unsigned(5,16));
						--din <= std_logic_vector(to_unsigned(5,16));
						-- din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					else
						NULL;
					end if;					

			-- Correct Version Information 0x0101 L0
			when 20 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(257,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					else
						NULL;
					end if;					


			-- Correct Packet Test generator => on and redirect packets to Zarlink => on 
			-- Plus setting the other relevant parameters 
			when 21 => 
						if MasterCounter = to_unsigned(10,128) then
						null;
			-- SET CHANNEL MASK FOR ASIC
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(30,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(611,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						din <= std_logic_vector(to_unsigned(3,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						din <= std_logic_vector(to_unsigned(4,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(20,128) then
						din <= std_logic_vector(to_unsigned(7,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(21,128) then
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(22,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(62,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(23,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(24,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(10,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(25,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(4,16));
						wr_en <= '1';
			-- SET ASIC FILTER
					elsif MasterCounter = to_unsigned(26,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(27,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(28,128) then
						din <= std_logic_vector(to_unsigned(614,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(29,128) then
						din <= std_logic_vector(to_unsigned(97,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(30,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(15,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(31,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(9,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(32,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(33,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(0,16));
						wr_en <= '1';
			-- SET AD Resolution
					elsif MasterCounter = to_unsigned(34,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(35,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(36,128) then
						din <= std_logic_vector(to_unsigned(626,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(37,128) then
						din <= std_logic_vector(to_unsigned(15,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(38,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(17,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(39,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(40,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(41,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(3,16));
						wr_en <= '1';
			-- SET SAMPLE RATE 
					elsif MasterCounter = to_unsigned(42,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(43,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(44,128) then
						din <= std_logic_vector(to_unsigned(627,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(45,128) then
						din <= std_logic_vector(to_unsigned(97,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(46,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(47,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(48,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(49,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
				--	SET TEST PACKETS REDIRECT TO ZARLINK
					elsif MasterCounter = to_unsigned(50,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(51,128) then
						din <= std_logic_vector(to_unsigned(20,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(52,128) then
						din <= std_logic_vector(to_unsigned(255,16));
						wr_en <= '1';
					
					-- ID 
					elsif MasterCounter = to_unsigned(53,128) then
						din <= std_logic_vector(to_unsigned(9,16));
						wr_en <= '1';
					-- Value LOW
					elsif MasterCounter = to_unsigned(54,128) then
						-- Only the last bit is important 
						din <= "0101010101010111";
						wr_en <= '1';
					-- VALUE UP
					elsif MasterCounter = to_unsigned(55,128) then
						din <= "0101010101010101";
						wr_en <= '1';
						
					elsif MasterCounter = to_unsigned(56,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(57,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(22,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(58,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(59,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
				-- ENABLE TEST PACKET GENERATOR
					elsif MasterCounter = to_unsigned(60,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(61,128) then
						din <= std_logic_vector(to_unsigned(20,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(62,128) then
						din <= std_logic_vector(to_unsigned(255,16));
						wr_en <= '1';
					-- ID 
					elsif MasterCounter = to_unsigned(63,128) then
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					-- Value LOW
					elsif MasterCounter = to_unsigned(64,128) then
					-- Only the last bit is important 
						din <= "0101010101010111";
						wr_en <= '1';
					-- VALUE UP
					elsif MasterCounter = to_unsigned(65,128) then
						din <= "0101010101010101";
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(66,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(17,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(67,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(21,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(68,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(69,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
			-- ENABLE RECORDING
--					elsif MasterCounter = to_unsigned(70,128) then
--						din <= std_logic_vector(to_unsigned(21930,16));
--						wr_en <= '1';
--					elsif MasterCounter = to_unsigned(71,128) then
--						din <= std_logic_vector(to_unsigned(14,16));
--						wr_en <= '1';
--					elsif MasterCounter = to_unsigned(72,128) then
--						din <= std_logic_vector(to_unsigned(769,16));
--						wr_en <= '1';
--					elsif MasterCounter = to_unsigned(73,128) then
--						-- 00
--						din <= std_logic_vector(to_unsigned(12,16));
--						wr_en <= '1';
--					elsif MasterCounter = to_unsigned(74,128) then
--						-- 01
--						din <= std_logic_vector(to_unsigned(5,16));
--						wr_en <= '1';
--					elsif MasterCounter = to_unsigned(75,128) then
--						-- 10
--						din <= std_logic_vector(to_unsigned(5,16));
--						wr_en <= '1';
--					elsif MasterCounter = to_unsigned(76,128) then
--						-- 11
--						din <= std_logic_vector(to_unsigned(2,16));
--						wr_en <= '1';
						
					else
						NULL;
					end if;					

			-- Correct Packet Test generator => on and redirect packets to Zarlink => off 
			-- Plus setting the other relevant parameters 
			when 22 => 
						if MasterCounter = to_unsigned(10,128) then
						null;
			-- SET CHANNEL MASK FOR ASIC
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(30,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(611,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						din <= std_logic_vector(to_unsigned(3,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						din <= std_logic_vector(to_unsigned(4,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(20,128) then
						din <= std_logic_vector(to_unsigned(7,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(21,128) then
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(22,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(62,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(23,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(12,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(24,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(10,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(25,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(4,16));
						wr_en <= '1';
			-- SET ASIC FILTER
					elsif MasterCounter = to_unsigned(26,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(27,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(28,128) then
						din <= std_logic_vector(to_unsigned(614,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(29,128) then
						din <= std_logic_vector(to_unsigned(97,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(30,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(15,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(31,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(9,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(32,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(33,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(0,16));
						wr_en <= '1';
			-- SET AD Resolution
					elsif MasterCounter = to_unsigned(34,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(35,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(36,128) then
						din <= std_logic_vector(to_unsigned(626,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(37,128) then
						din <= std_logic_vector(to_unsigned(15,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(38,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(17,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(39,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(40,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(41,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(3,16));
						wr_en <= '1';
			-- SET SAMPLE RATE 
					elsif MasterCounter = to_unsigned(42,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(43,128) then
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(44,128) then
						din <= std_logic_vector(to_unsigned(627,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(45,128) then
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(46,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(17,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(47,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(48,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(49,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
				-- ENABLE TEST PACKET GENERATOR
					elsif MasterCounter = to_unsigned(60,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(61,128) then
						din <= std_logic_vector(to_unsigned(22,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(62,128) then
						din <= std_logic_vector(to_unsigned(255,16));
						wr_en <= '1';
					-- ID 
					elsif MasterCounter = to_unsigned(63,128) then
						din <= std_logic_vector(to_unsigned(8,16));
						wr_en <= '1';
					-- Value LOW
					elsif MasterCounter = to_unsigned(64,128) then
					-- Only the last bit is important 
						din <= "0101010101010111";
						wr_en <= '1';
					-- VALUE UP
					elsif MasterCounter = to_unsigned(65,128) then
						din <= "0101010101010101";
						wr_en <= '1';

					elsif MasterCounter = to_unsigned(66,128) then
						din <= "0101010101010101";
						wr_en <= '1';

					elsif MasterCounter = to_unsigned(67,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(16,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(68,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(29,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(69,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(70,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';

				-- LOWER THE TIME BETWEEN THE CREATION OF TWO TESTPACKAGES
					elsif MasterCounter = to_unsigned(80,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(81,128) then
						din <= std_logic_vector(to_unsigned(22,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(82,128) then
						din <= std_logic_vector(to_unsigned(255,16));
						wr_en <= '1';
					-- ID 
					elsif MasterCounter = to_unsigned(83,128) then
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					-- Value LOW
					elsif MasterCounter = to_unsigned(84,128) then
					-- Only the last bit is important 
						din <= std_logic_vector(to_unsigned(5,16));
						wr_en <= '1';
					-- VALUE UP
					elsif MasterCounter = to_unsigned(85,128) then
						din <= (others =>'0');
						wr_en <= '1';

					elsif MasterCounter = to_unsigned(86,128) then
						din <= (others =>'0');
						wr_en <= '1';

					elsif MasterCounter = to_unsigned(87,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(37,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(88,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(89,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(9,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(90,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(4,16));
						wr_en <= '1';

					else
						NULL;
					end if;					


			-- Correct Packet 0263 (Set Channelmask of ASIC)
			-- + Channel Info 
			when 23 => 
					if MasterCounter = to_unsigned(10,128) then
						null;
					elsif MasterCounter = to_unsigned(11,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(12,128) then
						din <= std_logic_vector(to_unsigned(30,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(13,128) then
						din <= std_logic_vector(to_unsigned(611,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(14,128) then
						din <= std_logic_vector(to_unsigned(1,8)) & std_logic_vector(to_unsigned(1,8));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(15,128) then
						din <= std_logic_vector(to_unsigned(1,8)) & std_logic_vector(to_unsigned(1,8));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(16,128) then
						din <= std_logic_vector(to_unsigned(1,8)) & std_logic_vector(to_unsigned(1,8));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(17,128) then
						din <= std_logic_vector(to_unsigned(1,8)) & std_logic_vector(to_unsigned(1,8));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(18,128) then
						din <= std_logic_vector(to_unsigned(1,8)) & std_logic_vector(to_unsigned(1,8));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(19,128) then
						din <= std_logic_vector(to_unsigned(1,8)) & std_logic_vector(to_unsigned(1,8));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(20,128) then
						din <= std_logic_vector(to_unsigned(1,8)) & std_logic_vector(to_unsigned(1,8));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(21,128) then
						din <= std_logic_vector(to_unsigned(1,8)) & std_logic_vector(to_unsigned(1,8));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(22,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(57,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(23,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(22,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(24,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(7,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(25,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(2,16));
						wr_en <= '1';
					-- Get Channel info 
					elsif MasterCounter = to_unsigned(60,128) then
						din <= std_logic_vector(to_unsigned(21930,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(61,128) then
						din <= std_logic_vector(to_unsigned(14,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(62,128) then
						din <= std_logic_vector(to_unsigned(1282,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(63,128) then
						-- 00
						din <= std_logic_vector(to_unsigned(11,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(64,128) then
						-- 01
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(65,128) then
						-- 10
						din <= std_logic_vector(to_unsigned(6,16));
						wr_en <= '1';
					elsif MasterCounter = to_unsigned(66,128) then
						-- 11
						din <= std_logic_vector(to_unsigned(1,16));
						wr_en <= '1';

					else
						NULL;
					end if;					


			when others =>
				NULL;
		end case;

	end if;
end process TheSecondClock;

Test_Fifo : FIFO_16Bit_1000BlockCheck
  PORT MAP (
    rst => Reset,
    wr_clk => Clk,
    rd_clk => Clk,
    din => din,
    wr_en => wr_en,
    rd_en => FIFO_IN_rd_en,
    dout => FIFO_IN_Data,
    full => open,
    almost_full => open,
    empty => open,
    almost_empty => open,
    valid => FIFO_IN_Valid,
    prog_full => open
  );
  
  
END;
