----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:55:15 12/30/2011 
-- Design Name: 
-- Module Name:    NetworkDataProcessInterface - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity NetworkDataProcessInterface is
		Port
		(
			-- Input Data 
			FIFO_PlusFullPacket		: in  STD_LOGIC;
			FIFO_wr_en 					: in  STD_LOGIC;
			FIFO_din 					: in  STD_LOGIC_VECTOR(16 DOWNTO 0);
			FIFO_FULL					: out STD_LOGIC;
			
			-- Information about error states 
			FIFO_Error_Overrun  		: out STD_LOGIC;
			FIFO_Error_Underrun 		: out STD_LOGIC;
			
			-- Connection to the Network
			Network_Data				: out STD_LOGIC_VECTOR(15 downto 0); -- Tri-State 
			Network_WriteEn			: out STD_LOGIC; -- Tri-State
			Network_Wait				: in  STD_LOGIC;
			Network_RequestChannel  : out STD_LOGIC; 
			Network_GetChannel		: in  STD_LOGIC;
			CRC_Reset					: out STD_LOGIC;
			CRC_00						: in 	STD_LOGIC_VECTOR(15 downto 0);
			CRC_01						: in 	STD_LOGIC_VECTOR(15 downto 0);
			CRC_10						: in 	STD_LOGIC_VECTOR(15 downto 0);
			CRC_11						: in 	STD_LOGIC_VECTOR(15 downto 0);

			-- The usual ports 
			Reset 						: in  STD_LOGIC;			
			Clk 							: in  STD_LOGIC
		);
end NetworkDataProcessInterface;

architecture Behavioral of NetworkDataProcessInterface is
	signal CompletePacketInFIFO : STD_LOGIC := '0';
	signal FIFO_MinusFullPacket : STD_LOGIC := '0';
	signal FIFO_rd_en : STD_LOGIC := '0';
	signal FIFO_dout : STD_LOGIC_VECTOR(16 DOWNTO 0) := (others => '0');
	signal FIFO_empty : STD_LOGIC := '0';
	signal FIFO_valid : STD_LOGIC := '0';

	-- Network Outputs
	signal Network_Data_Internal : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal Network_WriteEn_Internal : STD_LOGIC := '0';
	signal Network_GetChannel_Internal : STD_LOGIC := '0';
	signal CRC_Reset_Internal : STD_LOGIC := '0';

	signal CRC_01_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_10_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_11_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
COMPONENT NetworkOutgoingPaketProcessingFIFO
  PORT (
    clk : IN STD_LOGIC;
    rst : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR(16 DOWNTO 0);
    wr_en : IN STD_LOGIC;
    rd_en : IN STD_LOGIC;
    dout : OUT STD_LOGIC_VECTOR(16 DOWNTO 0);
    full : OUT STD_LOGIC;
    almost_full : OUT STD_LOGIC;
    empty : OUT STD_LOGIC;
    valid : OUT STD_LOGIC;
	 prog_full : OUT STD_LOGIC
  );
END COMPONENT;

	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		-- Prepare the network for transmission
		STATE_NETWORK_PREPARE,
		-- Communicate with the network (DATA)
		STATE_NETWORK_COMMUNICATE_DATA,
		-- Communicate with the network - WAIT A
		STATE_NETWORK_COMMUNICATE_WAIT_A,
		-- Communicate with the network - WAIT B
		STATE_NETWORK_COMMUNICATE_WAIT_B,		
		-- Communicate with the network - WAIT C
		STATE_NETWORK_COMMUNICATE_WAIT_C,		
		-- Communicate with the network (CRC 00)
		STATE_NETWORK_COMMUNICATE_CRC00,
		-- Communicate with the network (CRC 01)
		STATE_NETWORK_COMMUNICATE_CRC01,
		-- Communicate with the network (CRC 10)
		STATE_NETWORK_COMMUNICATE_CRC10,
		-- Communicate with the network (CRC 11)
		STATE_NETWORK_COMMUNICATE_CRC11
	);
	signal FSM_STATE : STATE_TYPE;
	
begin

-- Network Tri-State Output
Network_Data <= Network_Data_Internal when Network_GetChannel_Internal = '1' else (others => 'Z');
Network_WriteEn <= Network_WriteEn_Internal when Network_GetChannel_Internal = '1' else 'Z';
CRC_Reset <= CRC_Reset_Internal when Network_GetChannel_Internal = '1' else 'Z';

	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			-- Default:
				Network_Data_Internal <= (others => '0');
				Network_WriteEn_Internal <= '0';
				Network_RequestChannel <= '0';
				CRC_Reset_Internal <= '0';			
				FSM_STATE <= STATE_RESET;
				CRC_01_Copy <= (others => '0');
				CRC_10_Copy <= (others => '0');
				CRC_11_Copy <= (others => '0');
				Network_GetChannel_Internal <= '0';
				FIFO_rd_en <= '0';
				FIFO_MinusFullPacket <= '0';
				
		elsif rising_edge(Clk) then
				Network_GetChannel_Internal <= Network_GetChannel;
			-- Default:
				Network_Data_Internal <= (others => '0');
				Network_WriteEn_Internal <= '0';
				Network_RequestChannel <= '0';
				CRC_Reset_Internal <= '0';
				CRC_01_Copy <= (others => '0');
				CRC_10_Copy <= (others => '0');
				CRC_11_Copy <= (others => '0');
				FIFO_rd_en <= '0'; 
				FIFO_MinusFullPacket <= '0';
				
				case FSM_STATE is
					when STATE_RESET => 
						FSM_STATE <= STATE_IDLE;
					when STATE_IDLE =>
						if CompletePacketInFIFO = '1' then
							FSM_STATE <= STATE_NETWORK_PREPARE;
							FIFO_MinusFullPacket <= '1';
						else
							FSM_STATE <= STATE_IDLE;
						end if;
						
					--	We need the Network
					-- So... Lets get it. 
					when STATE_NETWORK_PREPARE => 
						Network_RequestChannel <= '1';
						if Network_GetChannel_Internal = '1' then
							-- We have the channel to the network.
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_DATA;
							CRC_Reset_Internal <= '1';
						else
							-- Continue waiting
							FSM_STATE <= STATE_NETWORK_PREPARE;
						end if;	

					-- Communicate with the network (DATA)
					when STATE_NETWORK_COMMUNICATE_DATA =>
						
						Network_RequestChannel <= '1';					
						
						-- Request next data element from FIFO (if required)
						if (Network_Wait = '0') AND (FIFO_valid = '1') AND (FIFO_dout(16) /= '1') then
							FIFO_rd_en <= '1';
						else
							NULL;
						end if;
						
						-- Get rid of the data 
						if (Network_Wait = '0') AND (FIFO_valid = '1') AND (FIFO_rd_en = '1') then
							Network_Data_Internal <= FIFO_dout(15 downto 0);
							Network_WriteEn_Internal <= '1';
							
							if FIFO_dout(16) = '1' then
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_A;
							else
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_DATA;
							end if;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_DATA;
						end if;
						
					-- Wait for the CRC - Values I.
					when STATE_NETWORK_COMMUNICATE_WAIT_A =>
						Network_RequestChannel <= '1';						
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_B;

					-- Wait for the CRC - Values II.
					when STATE_NETWORK_COMMUNICATE_WAIT_B =>
						Network_RequestChannel <= '1';	
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_C;						
					
					-- Wait for the CRC - Values III.
					when STATE_NETWORK_COMMUNICATE_WAIT_C =>
						Network_RequestChannel <= '1';						
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
						
					-- Communicate with the network (CRC 00)
					when STATE_NETWORK_COMMUNICATE_CRC00 => 
						Network_RequestChannel <= '1';						
						CRC_01_Copy <= CRC_01;
						CRC_10_Copy <= CRC_10;
						CRC_11_Copy <= CRC_11;
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_00;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
						end if;
						
					-- Communicate with the network (CRC 01)
					when STATE_NETWORK_COMMUNICATE_CRC01 =>
						Network_RequestChannel <= '1';						
						CRC_01_Copy <= CRC_01_Copy;
						CRC_10_Copy <= CRC_10_Copy;
						CRC_11_Copy <= CRC_11_Copy;		
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_01_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
						end if;
						
					-- Communicate with the network (CRC 10)
					when STATE_NETWORK_COMMUNICATE_CRC10 => 
						Network_RequestChannel <= '1';						
						CRC_10_Copy <= CRC_10_Copy;
						CRC_11_Copy <= CRC_11_Copy;	
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_10_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
						end if;

						-- Get rid of the last used data sample if it is the end marker 
						if (FIFO_valid = '1') AND (FIFO_dout(16) = '1') then
							FIFO_rd_en <= '1';
						else
							NULL;
						end if;

						
					-- Communicate with the network (CRC 11)
					when STATE_NETWORK_COMMUNICATE_CRC11 => 
						Network_RequestChannel <= '1';						

						CRC_11_Copy <= CRC_11_Copy;		
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_11_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_IDLE;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
						end if;						
						
						
					when others =>
						FSM_STATE <= STATE_RESET;		
				end case;
				
		end if;				
	end process FSM;
	
MyNetworkPacketCounter : entity work.NetworkPacketCounter 
    Port Map( 
				CompletePacketInFIFO => CompletePacketInFIFO,
			
				Plus => FIFO_PlusFullPacket,
				Minus => FIFO_MinusFullPacket,
				
				Error_Overrun => FIFO_Error_Overrun,
				Error_Underrun => FIFO_Error_Underrun,
				Reset => Reset,
				Clk => Clk
			);

MyNetworkOutgoingPaketProcessingFIFO : NetworkOutgoingPaketProcessingFIFO
  PORT MAP (
    clk => Clk,
    rst => Reset,
    din => FIFO_din,
    wr_en => FIFO_wr_en,
    rd_en => FIFO_rd_en,
    dout => FIFO_dout,
    full => open,
    almost_full => FIFO_FULL,
    empty => open,
    valid => FIFO_valid,
	 prog_full => FIFO_empty
  );

end Behavioral;

