library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
 
ENTITY Output_PacketArbiter_UUT IS
END Output_PacketArbiter_UUT;
 
ARCHITECTURE behavior OF Output_PacketArbiter_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT OutputPacketArbiter
    PORT(
         RequestChannel : IN  std_logic_vector(1 downto 0);
         GrantChannel : OUT  std_logic_vector(1 downto 0);
         Reset : IN  std_logic;
         Clk : IN  std_logic
        );
    END COMPONENT;
    

   --Inputs
   signal Out_AlmostFull : std_logic := '0';
   signal In_Data : std_logic_vector(31 downto 0) := (others => '0');
   signal In_Valid : std_logic_vector(1 downto 0) := (others => '0');
   signal In_RequestChannel : std_logic_vector(1 downto 0) := (others => '0');
   signal Reset : std_logic := '0';
   signal Clk : std_logic := '0';

 	--Outputs
   signal Out_Data : std_logic_vector(15 downto 0);
   signal Out_WriteEn : std_logic;
   signal In_GrantChannel : std_logic_vector(1 downto 0);
   signal In_AlmostFull : std_logic;

   -- Clock period definitions
 	signal Counter : unsigned(10 downto 0) := (others=>'0'); 
	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 
	
BEGIN

	In_Data(15 downto 0) <= std_logic_vector(MasterCounter(15 downto 0));
	In_Data(31 downto 16) <= NOT std_logic_vector(MasterCounter(15 downto 0));
	In_Valid <= std_logic_vector(MasterCounter(1 downto 0));
 
	-- Instantiate the Unit Under Test (UUT)
   uut: OutputPacketArbiter PORT MAP (
          RequestChannel => In_RequestChannel,
          GrantChannel => In_GrantChannel,
          Reset => Reset,
          Clk => Clk
        );

TheMasterClock : process
begin
	clockloop: for i in 0 to 25000 loop
		clk <= NOT clk;
		wait for 4ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

TheSecondClock : process(clk) 
begin
	if rising_edge(clk) then
		

		MasterCounter <= MasterCounter + 1;
		
		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
		else
			Reset <= '0';
		end if;
		In_RequestChannel <= "00";
		
		if MasterCounter = to_unsigned(10,128) then
			In_RequestChannel <= "10";
		elsif MasterCounter = to_unsigned(11,128) then
			In_RequestChannel <= "10";
		elsif MasterCounter = to_unsigned(12,128) then
			In_RequestChannel <= "10";
		elsif MasterCounter = to_unsigned(13,128) then
			In_RequestChannel <= "10";
		elsif MasterCounter = to_unsigned(14,128) then
			In_RequestChannel <= "10";
		elsif MasterCounter = to_unsigned(15,128) then
			In_RequestChannel <= "11";
		elsif MasterCounter = to_unsigned(16,128) then
			In_RequestChannel <= "11";
		elsif MasterCounter = to_unsigned(16,128) then
			In_RequestChannel <= "11";
		elsif MasterCounter = to_unsigned(17,128) then
			In_RequestChannel <= "11";
		elsif MasterCounter = to_unsigned(18,128) then
			In_RequestChannel <= "01";
		elsif MasterCounter = to_unsigned(19,128) then
			In_RequestChannel <= "01";
		elsif MasterCounter = to_unsigned(20,128) then
			In_RequestChannel <= "01";
		elsif MasterCounter = to_unsigned(21,128) then
			In_RequestChannel <= "01";
		elsif MasterCounter = to_unsigned(22,128) then
			In_RequestChannel <= "01";
		elsif MasterCounter = to_unsigned(23,128) then
			In_RequestChannel <= "01";
		elsif MasterCounter = to_unsigned(24,128) then
			In_RequestChannel <= "11";
		elsif MasterCounter = to_unsigned(25,128) then
			In_RequestChannel <= "11";
		elsif MasterCounter = to_unsigned(26,128) then
			In_RequestChannel <= "11";
		elsif MasterCounter = to_unsigned(27,128) then
			In_RequestChannel <= "11";
		elsif MasterCounter = to_unsigned(28,128) then
			In_RequestChannel <= "11";
		elsif MasterCounter = to_unsigned(29,128) then
			In_RequestChannel <= "11";
		else
			NULL;
		end if;
					

	end if;
end process TheSecondClock;

END;
