----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:59:37 12/29/2011 
-- Design Name: 
-- Module Name:    TestClock - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity TestClock is
    Port ( 
		-- Event
		EventPort		: out STD_LOGIC;
		
		-- The Sample Rate 
		Implant_SampleRate : in std_logic_vector(7 downto 0);
		
		-- 95.8333MHz => 10.000 Hz Clock
		The10thOf1MS : in std_logic_vector (29 downto 0);
		-- Usual Ports 
		Reset 			: in  STD_LOGIC;
		Clk 				: in  STD_LOGIC
	 );
end TestClock;

architecture Behavioral of TestClock is

	-- The 95+ MHZ Counter
	signal Counter : unsigned (29 downto 0) := (others => '0');
	
	-- The 10kHz Counter
	signal SlowCounter : unsigned (8 downto 0) := (others => '0');
	
begin

	FastClock : process(Reset,Clk)
	begin
		if (Reset = '1') then
			
			-- Default:
			Counter <= (others => '0');
			SlowCounter <= (others => '0');
			
			EventPort <= '0';
			
		elsif rising_edge(Clk) then
			
			-- Default:
			EventPort <= '0';
			SlowCounter <= SlowCounter;
			
			-- Waited for 1/10 ms ?
			if Counter = unsigned(The10thOf1MS) then
				-- Yes
				-- Then Restart Counter
				Counter <= (others => '0');
				
				-- And Count the 10ms Counter one up
				-- Or do we have an event? 
				if SlowCounter >= unsigned(Implant_SampleRate) then 
					-- Yes, we have an event! 
				
					SlowCounter <= to_unsigned(1,8);
					EventPort <= '1';
				
				else
					-- Wait... 
					SlowCounter <= SlowCounter + 1;
				end if;
			
			else
				-- No
				Counter <= Counter + 1;
			end if;
			
		end if;
		
	end process FastClock;

end Behavioral;

