library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity TestPacketEmitter is
	 Generic (
		Module_Name : STD_LOGIC_VECTOR (15 downto 0) := (others => '0')
	 );
    Port ( 

		-- Connection to the FIFO
	 	Data_In						: in STD_LOGIC_VECTOR(111 downto 0);
		Valid_In						: in STD_LOGIC;
		ReadEn_In					: out STD_LOGIC;
	 
		-- Connection to the Zarlink 
		Zarlink_Data_Out			: out STD_LOGIC_VECTOR(119 downto 0); -- Tri-State
		Zarlink_WriteEn			: out STD_LOGIC; -- Tri-State
		Zarlink_Data_In			: in 	STD_LOGIC_VECTOR(111 downto 0);
		Zarlink_Valid_In			: in 	STD_LOGIC;
		Zarlink_BlockMode			: out	STD_LOGIC; -- Tri-State 
		Zarlink_Wait				: in  STD_LOGIC;
		Zarlink_RequestChannel  : out STD_LOGIC;
		Zarlink_GetChannel		: in  STD_LOGIC;
		Zarlink_Name				: out STD_LOGIC_VECTOR (15 downto 0); -- Tri-State

		Zarlink_EnableBlockBufferCheck : out STD_LOGIC;
		Zarlink_MaxBufferFill   : in STD_LOGIC_VECTOR (7 downto 0);
		Zarlink_LinkQuality_Report : in STD_LOGIC;
		Zarlink_Testemitter_Retry : in STD_LOGIC_VECTOR (31 downto 0); 
		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	 );

end TestPacketEmitter;

architecture Behavioral of TestPacketEmitter is
-- Handle the Tri-States 	
	-- Zarlink Outputs
	signal Zarlink_Data_Out_Internal	: STD_LOGIC_VECTOR(119 downto 0) := (others => '0');
	signal Zarlink_WriteEn_Internal : STD_LOGIC := '0';
	signal Zarlink_BlockMode_Internal : STD_LOGIC := '0';
	
	signal ZDOR	: STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
	signal Z_RD : STD_LOGIC := '0';
	signal Z_Address : STD_LOGIC_VECTOR(6 downto 0) := (others => '0');
	signal Z_BM : STD_LOGIC := '0';
	signal Z_WE : STD_LOGIC := '0';
	signal Z_Done : STD_LOGIC := '0';

	signal Input_Memory : STD_LOGIC_VECTOR (111 downto 0) := (others => '0');
	signal RetryCounter : unsigned(31 downto 0):= (others => '0');
	signal LastKnownStatusOfTheBuffer : unsigned(7 downto 0) := (others => '0');
	
	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		-- Request the Zalink Channel
		STATE_ZARLINK_PREPARE,

		-- Debug stuff
		STATE_DEBUG_START,
		STATE_DEBUG_ECC,
		STATE_DEBUG_CRC,
		STATE_DEBUG_CNT1,
		STATE_DEBUG_CNT2,
		STATE_DEBUG_STOP,
		
		STATE_CHECKBUFFER,
		STATE_CHECKBUFFER_WAIT,
		STATE_CHECKBUFFER_RETRY,
		-- Comunicate with the Zarlink
		STATE_ZARLINK_COMMUNICATE
	);
	signal FSM_STATE : STATE_TYPE;

	--signal Data_In_BitReverse	: STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
begin
-- Resort the Zarlink_Data_Out_Internal in Zarlink-Fashion
Zarlink_Data_Out_Internal <= 
		 ZDOR(104)&ZDOR(105)&ZDOR(106)&ZDOR(107)&ZDOR(108)&ZDOR(109)&ZDOR(110)&ZDOR(111)
		&ZDOR(96)&ZDOR(97)&ZDOR(98)&ZDOR(99)&ZDOR(100)&ZDOR(101)&ZDOR(102)&ZDOR(103)
		&ZDOR(88)&ZDOR(89)&ZDOR(90)&ZDOR(91)&ZDOR(92)&ZDOR(93)&ZDOR(94)&ZDOR(95)
		&ZDOR(80)&ZDOR(81)&ZDOR(82)&ZDOR(83)&ZDOR(84)&ZDOR(85)&ZDOR(86)&ZDOR(87)
		&ZDOR(72)&ZDOR(73)&ZDOR(74)&ZDOR(75)&ZDOR(76)&ZDOR(77)&ZDOR(78)&ZDOR(79)
		&ZDOR(64)&ZDOR(65)&ZDOR(66)&ZDOR(67)&ZDOR(68)&ZDOR(69)&ZDOR(70)&ZDOR(71)
		&ZDOR(56)&ZDOR(57)&ZDOR(58)&ZDOR(59)&ZDOR(60)&ZDOR(61)&ZDOR(62)&ZDOR(63)
		&ZDOR(48)&ZDOR(49)&ZDOR(50)&ZDOR(51)&ZDOR(52)&ZDOR(53)&ZDOR(54)&ZDOR(55)
		&ZDOR(40)&ZDOR(41)&ZDOR(42)&ZDOR(43)&ZDOR(44)&ZDOR(45)&ZDOR(46)&ZDOR(47)
		&ZDOR(32)&ZDOR(33)&ZDOR(34)&ZDOR(35)&ZDOR(36)&ZDOR(37)&ZDOR(38)&ZDOR(39)
		&ZDOR(24)&ZDOR(25)&ZDOR(26)&ZDOR(27)&ZDOR(28)&ZDOR(29)&ZDOR(30)&ZDOR(31)
		&ZDOR(16)&ZDOR(17)&ZDOR(18)&ZDOR(19)&ZDOR(20)&ZDOR(21)&ZDOR(22)&ZDOR(23)
		&ZDOR(8)&ZDOR(9)&ZDOR(10)&ZDOR(11)&ZDOR(12)&ZDOR(13)&ZDOR(14)&ZDOR(15)
		&ZDOR(0)&ZDOR(1)&ZDOR(2)&ZDOR(3)&ZDOR(4)&ZDOR(5)&ZDOR(6)&ZDOR(7)
		&Z_Address(0)&Z_Address(1)&Z_Address(2)&Z_Address(3)&Z_Address(4)&Z_Address(5)&Z_Address(6)&Z_RD;
Zarlink_BlockMode_Internal	<= Z_BM;
Zarlink_WriteEn_Internal <= Z_WE;

-- Tri - State Buffers: 
-- Zarlink Tri-State Output
Zarlink_Data_Out <= Zarlink_Data_Out_Internal when Zarlink_GetChannel = '1' else (others => 'Z');
Zarlink_WriteEn <= Zarlink_WriteEn_Internal when Zarlink_GetChannel = '1' else 'Z';
Zarlink_BlockMode <= Zarlink_BlockMode_Internal when Zarlink_GetChannel = '1' else 'Z';
Zarlink_Name <= Module_Name when Zarlink_GetChannel = '1' else (others => 'Z');
--
--	Data_In_BitReverse <= 
--		Data_In(104)&Data_In(105)&Data_In(106)&Data_In(107)&Data_In(108)&Data_In(109)&Data_In(110)&Data_In(111)
--		&Data_In(96)&Data_In(97)&Data_In(98)&Data_In(99)&Data_In(100)&Data_In(101)&Data_In(102)&Data_In(103)
--		&Data_In(88)&Data_In(89)&Data_In(90)&Data_In(91)&Data_In(92)&Data_In(93)&Data_In(94)&Data_In(95)
--		&Data_In(80)&Data_In(81)&Data_In(82)&Data_In(83)&Data_In(84)&Data_In(85)&Data_In(86)&Data_In(87)
--		&Data_In(72)&Data_In(73)&Data_In(74)&Data_In(75)&Data_In(76)&Data_In(77)&Data_In(78)&Data_In(79)
--		&Data_In(64)&Data_In(65)&Data_In(66)&Data_In(67)&Data_In(68)&Data_In(69)&Data_In(70)&Data_In(71)
--		&Data_In(56)&Data_In(57)&Data_In(58)&Data_In(59)&Data_In(60)&Data_In(61)&Data_In(62)&Data_In(63)
--		&Data_In(48)&Data_In(49)&Data_In(50)&Data_In(51)&Data_In(52)&Data_In(53)&Data_In(54)&Data_In(55)
--		&Data_In(40)&Data_In(41)&Data_In(42)&Data_In(43)&Data_In(44)&Data_In(45)&Data_In(46)&Data_In(47)
--		&Data_In(32)&Data_In(33)&Data_In(34)&Data_In(35)&Data_In(36)&Data_In(37)&Data_In(38)&Data_In(39)
--		&Data_In(24)&Data_In(25)&Data_In(26)&Data_In(27)&Data_In(28)&Data_In(29)&Data_In(30)&Data_In(31)
--		&Data_In(16)&Data_In(17)&Data_In(18)&Data_In(19)&Data_In(20)&Data_In(21)&Data_In(22)&Data_In(23)
--		&Data_In(8)&Data_In(9)&Data_In(10)&Data_In(11)&Data_In(12)&Data_In(13)&Data_In(14)&Data_In(15)
--		&Data_In(0)&Data_In(1)&Data_In(2)&Data_In(3)&Data_In(4)&Data_In(5)&Data_In(6)&Data_In(7);

	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			-- Default:
				ZDOR <= (others => '0');
				Z_RD <= '0';
				Z_Address <= (others => '0');						
				Z_BM <= '0';
				Z_WE <= '0';
				Z_Done <= '0';
				Zarlink_RequestChannel <= '0';
				
				ReadEn_In <= '0';
				Input_Memory <= (others => '0');	
				RetryCounter <= (others => '0');				
				Zarlink_EnableBlockBufferCheck <= '1';
				LastKnownStatusOfTheBuffer <= to_unsigned(64,8);
				
		elsif rising_edge(Clk) then

			-- Default:
				ZDOR <= (others => '0');
				Z_RD <= '0';
				Z_Address <= (others => '0');						
				Z_BM <= '0';
				Z_WE <= '0';
				Z_Done <= Z_Done;
				Zarlink_RequestChannel <= '0';	
				
				ReadEn_In <= '0';	
				Input_Memory <= Input_Memory;
				RetryCounter <= RetryCounter;
				Zarlink_EnableBlockBufferCheck <= '1';
				
				LastKnownStatusOfTheBuffer <= LastKnownStatusOfTheBuffer;
				
				case FSM_STATE is
				
					when STATE_RESET => 
						FSM_STATE <= STATE_IDLE;
					
					when STATE_IDLE => 
						if Valid_In = '1' then
							FSM_STATE <= STATE_ZARLINK_PREPARE;
							ReadEn_In <= '1';
						else
							FSM_STATE <= STATE_IDLE;
						end if;
						--Input_Memory <= Data_In_BitReverse;
						Input_Memory <= Data_In;
						
					--	We need the Zarlink
					-- So... Lets get it. 
					when STATE_ZARLINK_PREPARE =>
						Zarlink_RequestChannel <= '1';
						if Zarlink_GetChannel = '1' then
							-- We have the channel to the zarlink. 
							if LastKnownStatusOfTheBuffer >= unsigned(Zarlink_MaxBufferFill) then
								if Zarlink_LinkQuality_Report = '1' then
									FSM_STATE <= STATE_DEBUG_START;
								else
									FSM_STATE <= STATE_CHECKBUFFER;
								end if;
							else
								FSM_STATE <= STATE_ZARLINK_COMMUNICATE;
								LastKnownStatusOfTheBuffer <= LastKnownStatusOfTheBuffer + 1; 
							end if;
							
						else
							-- Continue waiting
							FSM_STATE <= STATE_ZARLINK_PREPARE;
						end if;
						Z_Done <= '0';

				-- Debug stuff
				
					when STATE_DEBUG_START => 
					
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DEBUG_ECC;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_START;
							Z_Address <= std_logic_vector(to_unsigned(6,7));

							ZDOR(7 downto 0) <= "00000111";
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_START;						
						end if;
						
					when STATE_DEBUG_ECC => 
					
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DEBUG_CRC;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_ECC;
							Z_Address <= std_logic_vector(to_unsigned(15,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_ECC;						
						end if;
					
					when STATE_DEBUG_CRC =>

						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DEBUG_CNT1;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_CRC;
							Z_Address <= std_logic_vector(to_unsigned(16,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_CRC;						
						end if;
					
					when STATE_DEBUG_CNT1 =>

						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DEBUG_CNT2;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_CNT1;
							Z_Address <= std_logic_vector(to_unsigned(17,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_CNT1;						
						end if;					
						
					when STATE_DEBUG_CNT2 => 
					
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DEBUG_STOP;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_CNT2;
							Z_Address <= std_logic_vector(to_unsigned(18,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_CNT2;						
						end if;		
						
					when STATE_DEBUG_STOP => 
					
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_CHECKBUFFER;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_STOP;
							Z_Address <= std_logic_vector(to_unsigned(6,7));
							ZDOR(7 downto 0) <= "00000000";
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_STOP;						
						end if;


					when STATE_CHECKBUFFER =>
						-- Is there something in the buffer? 
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_CHECKBUFFER_WAIT;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_CHECKBUFFER;
							Z_Address <= std_logic_vector(to_unsigned(3,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_CHECKBUFFER;						
						end if;

					when STATE_CHECKBUFFER_WAIT =>
						Zarlink_RequestChannel <= '1';					
						-- In the case the Zarlink delivered no new data, continue to wait 
						if Zarlink_Valid_In = '0' then
							FSM_STATE <= STATE_CHECKBUFFER_WAIT;
						-- Buffer is emtpy and the polling ended. Which means that the buffer is clear. 
						-- Go to idle and wait until polling is re-activated 
						elsif unsigned(Zarlink_Data_In (7 downto 0)) >= unsigned(Zarlink_MaxBufferFill) then
							FSM_STATE <= STATE_CHECKBUFFER_RETRY;	
							-- There is something in the buffer 
							LastKnownStatusOfTheBuffer <= unsigned(Zarlink_Data_In (7 downto 0));
						else 
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE;
							Zarlink_EnableBlockBufferCheck <= '0';
							LastKnownStatusOfTheBuffer <= unsigned(Zarlink_Data_In (7 downto 0));
						end if;
						Z_Done <= '0';
						RetryCounter <= (others => '0');
						
					when STATE_CHECKBUFFER_RETRY =>
						Zarlink_RequestChannel <= '0';

						RetryCounter <= RetryCounter + 1;
						if RetryCounter > unsigned(Zarlink_Testemitter_Retry) then
							FSM_STATE <= STATE_ZARLINK_PREPARE;
						else
							FSM_STATE <= STATE_CHECKBUFFER_RETRY;
						end if;
						
					-- We have the link to the Zarlink
					-- Lets comunicate...
					when STATE_ZARLINK_COMMUNICATE =>
						
						Zarlink_EnableBlockBufferCheck <= '0';
						
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_IDLE;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE;
							Z_Address <= std_logic_vector(to_unsigned(45,7));
							ZDOR (111 downto 0) <= Input_Memory(111 downto 0);
							Z_RD <= '0';
							Z_BM <= '1';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE;						
						end if;
					when others => 
						FSM_STATE <= STATE_RESET;
				end case;

		end if;
	end process FSM;
	
end Behavioral;

