library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity TestPacketGenerator is
    Port ( 

		ClockEvent					: in STD_LOGIC;
		
		Implant_ChannelMask 		: in std_logic_vector(127 downto 0);
		Implant_ADCResolution 	: in std_logic_vector(3 downto 0);
		Implant_SampleRate		: in std_logic_vector(7 downto 0);
		Implant_Filter 			: in std_logic_vector(3 downto 0);
		Implant_DeviceID 			: in STD_LOGIC_VECTOR(7 downto 0);
		
		Data_Out						: out STD_LOGIC_VECTOR(143 downto 0);
		Valid_Out					: out STD_LOGIC;

		TriggerChannelsIn 		: in STD_LOGIC_VECTOR (31 downto 0);
		The10KHzCounter 			: in STD_LOGIC_VECTOR (15 downto 0);
		
		Clk 							: in  STD_LOGIC;
		Reset 						: in  STD_LOGIC
	);
end TestPacketGenerator;

architecture Behavioral of TestPacketGenerator is
	
	-- First 4 bit
	constant PacketHeader_0 : std_logic_vector(3 downto 0) := "0000";
	
	constant PacketHeader_1 : std_logic_vector(3 downto 0) := "0001";
	signal Timestamp			: unsigned(15 downto 0) := (others => '0');

	signal Output_Buffer		: std_logic_vector(111 downto 0) := (others => '0');
	signal Used_Bits			: unsigned (6 downto 0) := (others => '0');
	
	signal HeaderTyp1			: std_logic := '0';
	
	signal Implant_ChannelMask_Copy : std_logic_vector(127 downto 0) := (others => '0');
	signal Data_ChannelCounter	: unsigned (7 downto 0) := (others => '0');
	
	signal TestCounter : unsigned(7 downto 0) := (others => '0');
	signal TestData : std_logic_vector(15 downto 0) := (others => '0');
	
	signal LengthOfData : unsigned (4 downto 0) := (others => '0');
	signal BitCounter   : unsigned (4 downto 0) := (others => '0');
	signal Output_Copy	: std_logic_vector(15 downto 0) := (others => '0');
	
	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		-- Start Packet (Header) 
		STATE_FIRST,
		STATE_SECOND,
		-- Packet (Data)
		STATE_DATA_A,
		STATE_DATA_BITMOVER,
		STATE_FINISH
	);
	signal FSM_STATE : STATE_TYPE;
	signal Data_Out_Buffer : STD_LOGIC_VECTOR(143 downto 0) := (others => '0');		
begin

-- ALIAS 
-- TestData <= std_logic_vector(Data_ChannelCounter) & std_logic_vector(TestCounter);
TestData <= TestCounter(0) & TestCounter(1) & TestCounter(2) & TestCounter(3) & TestCounter(4)
				& TestCounter(5) & TestCounter(6) & TestCounter(7) & Data_ChannelCounter(0)
				& Data_ChannelCounter(1) & Data_ChannelCounter(2) & Data_ChannelCounter(3)
				& Data_ChannelCounter(4) & Data_ChannelCounter(5) & Data_ChannelCounter(6)
				& Data_ChannelCounter(7);

Data_Out <= Data_Out_Buffer(143 downto 112)
		&Data_Out_Buffer(104)&Data_Out_Buffer(105)&Data_Out_Buffer(106)&Data_Out_Buffer(107)&Data_Out_Buffer(108)&Data_Out_Buffer(109)&Data_Out_Buffer(110)&Data_Out_Buffer(111)
		&Data_Out_Buffer(96)&Data_Out_Buffer(97)&Data_Out_Buffer(98)&Data_Out_Buffer(99)&Data_Out_Buffer(100)&Data_Out_Buffer(101)&Data_Out_Buffer(102)&Data_Out_Buffer(103)
		&Data_Out_Buffer(88)&Data_Out_Buffer(89)&Data_Out_Buffer(90)&Data_Out_Buffer(91)&Data_Out_Buffer(92)&Data_Out_Buffer(93)&Data_Out_Buffer(94)&Data_Out_Buffer(95)
		&Data_Out_Buffer(80)&Data_Out_Buffer(81)&Data_Out_Buffer(82)&Data_Out_Buffer(83)&Data_Out_Buffer(84)&Data_Out_Buffer(85)&Data_Out_Buffer(86)&Data_Out_Buffer(87)
		&Data_Out_Buffer(72)&Data_Out_Buffer(73)&Data_Out_Buffer(74)&Data_Out_Buffer(75)&Data_Out_Buffer(76)&Data_Out_Buffer(77)&Data_Out_Buffer(78)&Data_Out_Buffer(79)
		&Data_Out_Buffer(64)&Data_Out_Buffer(65)&Data_Out_Buffer(66)&Data_Out_Buffer(67)&Data_Out_Buffer(68)&Data_Out_Buffer(69)&Data_Out_Buffer(70)&Data_Out_Buffer(71)
		&Data_Out_Buffer(56)&Data_Out_Buffer(57)&Data_Out_Buffer(58)&Data_Out_Buffer(59)&Data_Out_Buffer(60)&Data_Out_Buffer(61)&Data_Out_Buffer(62)&Data_Out_Buffer(63)
		&Data_Out_Buffer(48)&Data_Out_Buffer(49)&Data_Out_Buffer(50)&Data_Out_Buffer(51)&Data_Out_Buffer(52)&Data_Out_Buffer(53)&Data_Out_Buffer(54)&Data_Out_Buffer(55)
		&Data_Out_Buffer(40)&Data_Out_Buffer(41)&Data_Out_Buffer(42)&Data_Out_Buffer(43)&Data_Out_Buffer(44)&Data_Out_Buffer(45)&Data_Out_Buffer(46)&Data_Out_Buffer(47)
		&Data_Out_Buffer(32)&Data_Out_Buffer(33)&Data_Out_Buffer(34)&Data_Out_Buffer(35)&Data_Out_Buffer(36)&Data_Out_Buffer(37)&Data_Out_Buffer(38)&Data_Out_Buffer(39)
		&Data_Out_Buffer(24)&Data_Out_Buffer(25)&Data_Out_Buffer(26)&Data_Out_Buffer(27)&Data_Out_Buffer(28)&Data_Out_Buffer(29)&Data_Out_Buffer(30)&Data_Out_Buffer(31)
		&Data_Out_Buffer(16)&Data_Out_Buffer(17)&Data_Out_Buffer(18)&Data_Out_Buffer(19)&Data_Out_Buffer(20)&Data_Out_Buffer(21)&Data_Out_Buffer(22)&Data_Out_Buffer(23)
		&Data_Out_Buffer(8)&Data_Out_Buffer(9)&Data_Out_Buffer(10)&Data_Out_Buffer(11)&Data_Out_Buffer(12)&Data_Out_Buffer(13)&Data_Out_Buffer(14)&Data_Out_Buffer(15)
		&Data_Out_Buffer(0)&Data_Out_Buffer(1)&Data_Out_Buffer(2)&Data_Out_Buffer(3)&Data_Out_Buffer(4)&Data_Out_Buffer(5)&Data_Out_Buffer(6)&Data_Out_Buffer(7);

-- Packet-Header
-- Device ID
-- Channel Mask (HeaderTyp1 == 0)
-- RHA Filter (HeaderTyp1 == 0)
-- Timestamp 
-- Sample Rate (HeaderTyp1 == 0)
-- ADC Resolution (HeaderTyp1 == 0)
-- '0'
-- --==--- DATA --==--

	FSM : process(Reset,Clk)
	begin
	
		if (Reset = '1') then
			-- Default:
			Timestamp <= (others => '0');
			FSM_STATE <= STATE_RESET;
			Data_Out_Buffer <= (others => '0');
			Valid_Out <= '0';
			Output_Buffer <= (others => '0');
			Used_Bits <= (others => '0');
			Implant_ChannelMask_Copy <= (others => '0');
			Data_ChannelCounter <= (others => '0');
			TestCounter <= (others => '0');
			LengthOfData <= (others => '0');
			BitCounter <= (others => '0');
			Output_Copy <= (others => '0');
			HeaderTyp1 <= '0';
			
		elsif rising_edge(Clk) then
			-- Default:
			Timestamp <= Timestamp;
			Data_Out_Buffer <= (others => '0');
			Valid_Out <= '0';
			Output_Buffer <= Output_Buffer;
			Used_Bits <= Used_Bits;
			Implant_ChannelMask_Copy <= Implant_ChannelMask_Copy;
			Data_ChannelCounter <= Data_ChannelCounter;
			TestCounter <= TestCounter;
			LengthOfData <= LengthOfData;
			BitCounter <= BitCounter;
			Output_Copy <= Output_Copy;
			HeaderTyp1 <= HeaderTyp1;
			
			case FSM_STATE is
				
				-- After reset...
				when STATE_RESET =>
					FSM_STATE <= STATE_IDLE;
					
				when STATE_IDLE =>
					-- Do we need go generate a new packet? 
					if ClockEvent = '1' then
						-- Yes 
						FSM_STATE <= STATE_FIRST;
						-- Increase the Timestamp
						Timestamp <= Timestamp + unsigned("00000000" & Implant_SampleRate);
						-- Increase the testcounter for signalling the next packet
						TestCounter <= TestCounter + 1;
					else
						-- No: Wait
						FSM_STATE <= STATE_IDLE;
					end if;
					
					-- Reset the Channel Counter
					Data_ChannelCounter <= (others => '0');
					-- How long is one sample?
					LengthOfData <= unsigned("0" & Implant_ADCResolution) + 1;

				when STATE_FIRST => 
					-- If it is a long packet header, we write the first part of the header 
					if HeaderTyp1 = '0' then
						Data_Out_Buffer <= TriggerChannelsIn(15 downto 0) & The10KHzCounter & Implant_ChannelMask(98 downto 0) & Implant_DeviceID & PacketHeader_0 & "1";
						Valid_Out <= '1';
					else
						-- If it is a short packet header then we do nothing
						NULL;
					end if;
					-- We copy the channel mask which we will need for a bit shifting procedure 
					Implant_ChannelMask_Copy <= Implant_ChannelMask;
					FSM_STATE <= STATE_SECOND;
				
				when STATE_SECOND =>
					-- This is the rest for the long packet header 
					-- There is still space for 48 bits 
					if HeaderTyp1 = '0' then
						Output_Buffer <= "00" & Implant_ADCResolution & Implant_SampleRate & std_logic_vector(Timestamp) & 
							Implant_Filter & Implant_ChannelMask(127 downto 99) & "0" & std_logic_vector(to_unsigned(0,48));
						Used_Bits <= to_unsigned(64,7);
					else
					-- This is the short packet header 
					-- There is still space for 81 bits 
						Output_Buffer <= "00" & std_logic_vector(Timestamp) & Implant_DeviceID & PacketHeader_1 & "1" & 
							std_logic_vector(to_unsigned(0,81));
						Used_Bits <= to_unsigned(31,7);
					end if;
					FSM_STATE <= STATE_DATA_A;
				
				when STATE_DATA_A => 
					
					-- Is there still a sample left? 
					if (Data_ChannelCounter = 128) and (Used_Bits = to_unsigned(1,7)) then 
						-- NO... and there is NO data in the block
						FSM_STATE <= STATE_IDLE;
						HeaderTyp1 <= '1';
					elsif Data_ChannelCounter = 128 then 	
						-- NO... But there is still space in the block
						-- Fill the rest with zeros and send it on it's way 
						FSM_STATE <= STATE_FINISH;
						HeaderTyp1 <= '1';
					elsif Implant_ChannelMask_Copy(0) = '1' then
						-- Yes... and it is active  
						FSM_STATE <= STATE_DATA_BITMOVER;
						BitCounter <= (others => '0');
						Output_Copy <= TestData;
					else		
						-- Yes.. but is not active 
						FSM_STATE <= STATE_DATA_A;
					end if;
					
					-- Next channel...
					Implant_ChannelMask_Copy <= "0" & Implant_ChannelMask_Copy(127 downto 1);
					Data_ChannelCounter <= Data_ChannelCounter + 1;
				
				when STATE_DATA_BitMover => 
					
					-- The block is full 
					if Used_Bits = to_unsigned(112,7) then
						-- Export the result
						Data_Out_Buffer <= TriggerChannelsIn(15 downto 0) & The10KHzCounter & Output_Buffer;
						Valid_Out <= '1';
						
						-- Reset the export buffer (don't forget the 0 for marking the start of the block)
						Used_Bits <= to_unsigned(1,7);
						Output_Buffer <= (others => '0');
					
					-- This sample is done 	
					elsif BitCounter = LengthOfData then 
						FSM_STATE <= STATE_DATA_A;
					
					-- this sample is not finished 
					else
						FSM_STATE <= STATE_DATA_BitMover;
						BitCounter <= BitCounter + 1;
						Output_Buffer <= Output_Copy(0) & Output_Buffer(111 downto 1);
						Used_Bits <= Used_Bits + 1;
						Output_Copy <= "0" & Output_Copy(15 downto 1); 
					end if;
					
				when STATE_FINISH =>
					-- The last block needs to be filled
					if Used_Bits = to_unsigned(112,7) then
						-- Done!
						Data_Out_Buffer <= TriggerChannelsIn(15 downto 0) & The10KHzCounter & Output_Buffer;
						Valid_Out <= '1';
						FSM_STATE <= STATE_IDLE;
					else
						-- Still bits to fill up with 0
						Output_Buffer <= "0" & Output_Buffer(111 downto 1);
						Used_Bits <= Used_Bits + 1;
						FSM_STATE <= STATE_FINISH;
					end if;
				
				when others => 
					FSM_STATE <= STATE_RESET;
			end case;
			
		end if;
	
	end process FSM;


end architecture Behavioral;

