library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
use STD.textio.all;

ENTITY TestPacketGenerator_UUT IS
END TestPacketGenerator_UUT;
 
ARCHITECTURE behavior OF TestPacketGenerator_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT TestPacketGenerator
    PORT(
         ClockEvent : IN  std_logic;
         Implant_ChannelMask : IN  std_logic_vector(127 downto 0);
         Implant_ADCResolution : IN  std_logic_vector(3 downto 0);
         Implant_SampleRate : IN  std_logic_vector(7 downto 0);
         Implant_Filter : IN  std_logic_vector(3 downto 0);
         Implant_DeviceID : IN  std_logic_vector(7 downto 0);
         Data_Out : OUT  std_logic_vector(111 downto 0);
         Valid_Out : OUT  std_logic;
         Clk : IN  std_logic;
         Reset : IN  std_logic
        );
    END COMPONENT;
    

   --Inputs
   signal ClockEvent : std_logic := '0';
   signal Implant_ChannelMask : std_logic_vector(127 downto 0) := 
		"00000001" & "00000001" & "00000001" & "00000001" &
		"00000001" & "00000001" & "00000001" & "00000001" &
		"00000001" & "00000001" & "00000001" & "00000001" &
		"00000001" & "00000001" & "00000001" & "00000001" ;
   signal Implant_ADCResolution : std_logic_vector(3 downto 0) := "0111";
   signal Implant_SampleRate : std_logic_vector(7 downto 0) := std_logic_vector(to_unsigned(50,8));
   signal Implant_Filter : std_logic_vector(3 downto 0) := "0000";
   signal Implant_DeviceID : std_logic_vector(7 downto 0) := std_logic_vector(to_unsigned(53,8));
   signal Clk : std_logic := '0';
   signal Reset : std_logic := '0';

 	--Outputs
   signal Data_Out : std_logic_vector(111 downto 0);
   signal Valid_Out : std_logic;

	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 

	file file_id_out  : text;
	
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: TestPacketGenerator PORT MAP (
          ClockEvent => ClockEvent,
          Implant_ChannelMask => Implant_ChannelMask,
          Implant_ADCResolution => Implant_ADCResolution,
          Implant_SampleRate => Implant_SampleRate,
          Implant_Filter => Implant_Filter,
          Implant_DeviceID => Implant_DeviceID,
          Data_Out => Data_Out,
          Valid_Out => Valid_Out,
          Clk => Clk,
          Reset => Reset
        );

TheMasterClock : process
	variable f_status     : file_open_status;
begin
	file_open(f_status,file_id_out,"test_data.txt",write_mode);
	clockloop: for i in 0 to 25000 loop
		clk <= NOT clk;
		wait for 4ns;
	end loop clockloop;
	wait;
	file_close(file_id_out); 
end process TheMasterClock; 

TheSecondClock : process(clk) 
                                                                                          
	variable MyLine       : LINE;  
	
begin
	
	if rising_edge(clk) then
		

		MasterCounter <= MasterCounter + 1;
		
		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
		else
			Reset <= '0';
		end if;
		
		if MasterCounter = to_unsigned(15,128) then
			ClockEvent <= '1';
		elsif MasterCounter = to_unsigned(400,128) then
			ClockEvent <= '1';			
		elsif MasterCounter = to_unsigned(800,128) then
			ClockEvent <= '1';			
		else
			ClockEvent <= '0';
		end if;
					
	
		if Valid_Out = '1' then
			write(MyLine,to_bitvector(Data_Out));
			writeline(file_id_out,MyLine);
		end if;
		
	end if;
end process TheSecondClock;

END;
