----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:03:36 01/27/2012 
-- Design Name: 
-- Module Name:    TheAmpConTroll - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity TheAmpConTroll is
    Port ( 
			VGA_CS 					: out std_logic_vector(31 downto 0);	 
			VGA_DATA 				: out std_logic_vector(1 downto 0);
			VGA_CLK 					: out std_logic_vector(1 downto 0);			
			VGA_SelectionValue 	: in  std_logic_vector (7 downto 0);

			VGA_Values 				: in  STD_LOGIC_VECTOR (7 downto 0);
			VGA_Valid 				: in  std_logic; 
			VGA_Ready 				: out std_logic; 
			
			Reset 					: in std_logic;								-- Reset port (reset = 1)
			Clk 						: in std_logic								
		);
end TheAmpConTroll;

architecture Behavioral of TheAmpConTroll is

	signal VGA_SDI_Buffer : std_logic := '0'; 
	signal VGA_Clk_Buffer : std_logic := '0'; 
	signal VGA_CS_Buffer : std_logic := '0'; 

	signal Counter : unsigned (3 downto 0) := (others => '0');
	constant Counter_Max : unsigned (3 downto 0) := to_unsigned(10,4);
	
	signal Clk_Slow : std_logic := '0';
	
begin
   
	ClockProcess : process(Reset,Clk)
	begin
		if Reset = '1' then
			Counter <= (others => '0');
			Clk_Slow <= '0';
		elsif rising_edge(Clk) then
			-- Default:
			Counter <= Counter + 1;
			Clk_Slow <= Clk_Slow;
			
			if Counter >= Counter_Max then
				Clk_Slow <= not Clk_Slow;
				Counter <= (others => '0');
			else
				null;
			end if;
			
		end if;
	end process ClockProcess;
	
	VGA_DATA <= VGA_SDI_Buffer & VGA_SDI_Buffer;
	VGA_CLK <= VGA_Clk_Buffer & VGA_Clk_Buffer;
	
	MyAD5162_SPI: entity work.AD5162_SPI 
	PORT MAP(
		Values => VGA_Values,
		Valid => VGA_Valid,
		Ready => VGA_Ready,
		POTI_SDI => VGA_SDI_Buffer,
		POTI_CLK => VGA_Clk_Buffer,
		POTI_NOT_CS => VGA_CS_Buffer,
		Reset => Reset,
		Clk => Clk_Slow
	);
	
	MyVGA_CS_MUX: entity work.VGA_CS_MUX 
	PORT MAP(
		VGA_CS_Selected => VGA_CS_Buffer,
		VGA_CS => VGA_CS,
		VGA_SelectionValue => VGA_SelectionValue,
		Reset => Reset,
		Clk => Clk
	);

end Behavioral;

