library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;


entity TiggerIn_MuxControl is
    Port ( 

		TriggerIn_Address 		: out STD_LOGIC_VECTOR(4 downto 0);
		TriggerIn_Data 			: in STD_LOGIC;
	 
		Wait_TriggerIn 			: in STD_LOGIC_VECTOR (16 downto 0);
		TriggerChannelsIn 		: out STD_LOGIC_VECTOR (31 downto 0);
		Clk 							: in  STD_LOGIC;
		Reset 						: in  STD_LOGIC
	);
end TiggerIn_MuxControl;

architecture Behavioral of TiggerIn_MuxControl is

	signal Mux_Counter : unsigned (4 downto 0) := (others => '0');
	signal TriggerChannelsIn_Copy : STD_LOGIC_VECTOR (31 downto 0) := (others => '0');
	signal TriggerChannelsIn_Buffer : STD_LOGIC_VECTOR (31 downto 0) := (others => '0');

	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		STATE_WAIT,
		STATE_COLLECT
	);
	signal FSM_STATE : STATE_TYPE;
	signal Wait_Counter : unsigned (16 downto 0) := (others => '0');

begin

	TriggerIn_Address <= std_logic_vector(Mux_Counter);
	TriggerChannelsIn <= TriggerChannelsIn_Buffer;
	
	MuxFSM : process (Reset, Clk)
	begin
		
		if Reset = '1' then
			
			Mux_Counter <= (others => '0');	
			TriggerChannelsIn_Copy <= (others => '0');	
			TriggerChannelsIn_Buffer <= (others => '0');	
			Wait_Counter <= (others => '0');	
			
			FSM_STATE <= STATE_RESET;
			
		elsif rising_edge(Clk) then
			TriggerChannelsIn_Copy <= TriggerChannelsIn_Copy;
			TriggerChannelsIn_Buffer <= TriggerChannelsIn_Buffer;

			Mux_Counter <= Mux_Counter;
			Wait_Counter <= (others => '0');	
			
			case FSM_STATE is
				
				when STATE_RESET =>
					FSM_STATE <= STATE_WAIT;
				
				when STATE_WAIT => 
					Wait_Counter <= Wait_Counter + 1;
					
					if Wait_Counter < unsigned(Wait_TriggerIn) then
						FSM_STATE <= STATE_WAIT;
					else
						FSM_STATE <= STATE_COLLECT;
					end if;
				
				when STATE_COLLECT => 
					if std_logic_vector(Mux_Counter) = "11111" then
						TriggerChannelsIn_Buffer <= TriggerIn_Data & TriggerChannelsIn_Copy(31 downto 1);
						TriggerChannelsIn_Copy <= (others => '0');	
						Mux_Counter <= (others => '0');
					else
						TriggerChannelsIn_Copy <= TriggerIn_Data & TriggerChannelsIn_Copy(31 downto 1); 
						Mux_Counter <= Mux_Counter + 1;
					end if;
					FSM_STATE <= STATE_WAIT;
					
				when others => 
					FSM_STATE <= STATE_RESET;	
			end case;
			
		end if;
	
	end process MuxFSM;
	
end architecture Behavioral;

