library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;


entity TiggerOut_MuxControl is
    Port ( 
	 
				TriggerOut_Address 		: out STD_LOGIC_VECTOR(4 downto 0);
				TriggerOut_Data			: out STD_LOGIC;
				TriggerOut_Clk				: out STD_LOGIC;
				
				TriggerChannelsOut 		: in STD_LOGIC_VECTOR (31 downto 0);

				HalfCycle					: in STD_LOGIC_VECTOR (16 downto 0);
				
				Clk 							: in  STD_LOGIC;
				Reset 						: in  STD_LOGIC
  );
end TiggerOut_MuxControl;

architecture Behavioral of TiggerOut_MuxControl is

	signal Mux_Counter : unsigned (4 downto 0) := (others => '0');
	
	signal TriggerChannelsOut_Copy : STD_LOGIC_VECTOR (31 downto 0) := (others => '0');
	signal TriggerChannelsOut_Temp : STD_LOGIC_VECTOR (31 downto 0) := (others => '0');
	
	signal Input_Changed : std_logic := '0';
	
	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		STATE_PREPARE,
		STATE_LOW,
		STATE_HIGH,
		STATE_CHANGE
	);
	signal FSM_STATE : STATE_TYPE;
	signal HalfCycle_Counter : unsigned (16 downto 0) := (others => '0');

	signal TriggerOut_Data_Buffer : STD_LOGIC := '0';
	signal TriggerOut_Clk_Buffer : STD_LOGIC := '0';
	
begin

	TriggerOut_Address <= std_logic_vector(Mux_Counter);
	TriggerOut_Data <= TriggerOut_Data_Buffer;
	TriggerOut_Clk <= TriggerOut_Clk_Buffer;
	
	MuxFSM : process (Reset, Clk)
	begin
		
		if Reset = '1' then
			
			Mux_Counter <= (others => '0');
			HalfCycle_Counter <= (others => '0');
			TriggerChannelsOut_Copy <= (others => '1');
			TriggerChannelsOut_Temp <= (others => '1');
			TriggerOut_Clk_Buffer <= '0';
			TriggerOut_Data_Buffer <= '0';
			Input_Changed <= '1';
			
			FSM_STATE <= STATE_RESET;
			
		elsif rising_edge(Clk) then
		
			Mux_Counter <= Mux_Counter;
			TriggerChannelsOut_Copy <= TriggerChannelsOut_Copy;
			Input_Changed <= '0';
			HalfCycle_Counter <= (others => '0');
			TriggerOut_Clk_Buffer <= '0';
			TriggerOut_Data_Buffer <= TriggerOut_Data_Buffer;
			
			case FSM_STATE is
				
				when STATE_RESET =>
					FSM_STATE <= STATE_IDLE;
				
				when STATE_IDLE =>
					-- Something new? 
					if TriggerChannelsOut_Copy /= TriggerChannelsOut then
						-- Yes
						Input_Changed <= '1';
						TriggerChannelsOut_Copy <= TriggerChannelsOut;
					else
						-- No
						null;
					end if;
					
					if Input_Changed = '1' then
						FSM_STATE <= STATE_PREPARE;
					else
						FSM_STATE <= STATE_IDLE;
					end if;
				
				when STATE_PREPARE => 
					TriggerChannelsOut_Temp <= "0" & TriggerChannelsOut_Copy(31 downto 1);
					TriggerOut_Data_Buffer <= NOT TriggerChannelsOut_Copy(0);
					Mux_Counter <= (others => '0');
					FSM_STATE <= STATE_LOW;
					
				when STATE_LOW =>	
					HalfCycle_Counter <= HalfCycle_Counter + 1;
					TriggerOut_Clk_Buffer <= '0';
					
					if HalfCycle_Counter < unsigned(HalfCycle) then
						FSM_STATE <= STATE_LOW;
					else
						FSM_STATE <= STATE_HIGH;
						HalfCycle_Counter <= (others => '0');
					end if;
					
				when STATE_HIGH => 
					HalfCycle_Counter <= HalfCycle_Counter + 1;
					TriggerOut_Clk_Buffer <= '1';

					if HalfCycle_Counter < unsigned(HalfCycle) then
						FSM_STATE <= STATE_HIGH;
					else
						FSM_STATE <= STATE_Change;
						HalfCycle_Counter <= (others => '0');
					end if;
				
				when STATE_Change =>
					TriggerChannelsOut_Temp <= "0" & TriggerChannelsOut_Temp(31 downto 1);
					TriggerOut_Data_Buffer <= NOT TriggerChannelsOut_Temp(0);				
					if std_logic_vector(Mux_Counter) = "11111" then
						FSM_STATE <= STATE_IDLE;
						Mux_Counter <= (others => '0');
					else
						FSM_STATE <= STATE_LOW;
						Mux_Counter <= Mux_Counter + 1;
					end if;
					
				when others => 
					FSM_STATE <= STATE_RESET;	
			end case;
			
		end if;
	
	end process MuxFSM;


end architecture Behavioral;

