----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    18:17:41 10/09/2011 
-- Design Name: 
-- Module Name:    Zarlink_30Bit_Compiler - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity Zarlink_30Bit_Compiler is
    Port ( 
		-- FIFO IN
		FIFO_IN_din 				: IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		FIFO_IN_valid 				: IN STD_LOGIC;
		FIFO_IN_almost_full 		: OUT STD_LOGIC;
		
		-- Pause 
		Pause 						: IN STD_LOGIC;
		
		-- Command Inject Path 
		Command_Inject_din		: IN STD_LOGIC_VECTOR(29 DOWNTO 0);
		Command_Inject_valid		: IN STD_LOGIC;
		Command_Inject_almost_full : OUT STD_LOGIC;
		
		-- Slow 30 Bit Command FIFO 
		FIFO_rd_en 					: IN STD_LOGIC;
		FIFO_dout 					: OUT STD_LOGIC_VECTOR(29 DOWNTO 0);
		FIFO_valid 					: OUT STD_LOGIC;
		
		Reset 						: in  STD_LOGIC;	
		ClkSlow						: in  STD_LOGIC;	 
		Clk 							: in  STD_LOGIC	 
	 );
end Zarlink_30Bit_Compiler;

architecture Behavioral of Zarlink_30Bit_Compiler is

COMPONENT FIFO_30Bit_1000BlockCheck
  PORT (
    rst : IN STD_LOGIC;
    wr_clk : IN STD_LOGIC;
    rd_clk : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR(29 DOWNTO 0);
    wr_en : IN STD_LOGIC;
    rd_en : IN STD_LOGIC;
    dout : OUT STD_LOGIC_VECTOR(29 DOWNTO 0);
    full : OUT STD_LOGIC;
    almost_full : OUT STD_LOGIC;
    empty : OUT STD_LOGIC;
    almost_empty : OUT STD_LOGIC;
    valid : OUT STD_LOGIC;
    prog_full : OUT STD_LOGIC
  );
END COMPONENT;

    signal FIFO_din : STD_LOGIC_VECTOR(29 DOWNTO 0) := (others => '0');
    signal FIFO_wr_en : STD_LOGIC := '0';
    --signal FIFO_rd_en : STD_LOGIC := '0';
    --signal FIFO_dout : STD_LOGIC_VECTOR(29 DOWNTO 0) := (others => '0');
    signal FIFO_full : STD_LOGIC := '0';
    signal FIFO_almost_full : STD_LOGIC := '0';
    signal FIFO_empty : STD_LOGIC := '0';
    signal FIFO_almost_empty : STD_LOGIC := '0';
--    signal FIFO_valid : STD_LOGIC := '0';
    signal FIFO_prog_full : STD_LOGIC := '0';

	 signal ExpectSecondPage : std_logic := '0';
	 signal FirstPage : std_logic_vector(14 downto 0) := (others => '0');
	 
begin

Zarlink_30bit_FIFO : FIFO_30Bit_1000BlockCheck
  PORT MAP (
    rst => Reset,
    wr_clk => Clk,
    rd_clk => ClkSlow,
    din => FIFO_din,
    wr_en => FIFO_wr_en,
    rd_en => FIFO_rd_en,
    dout => FIFO_dout,
    full => FIFO_full,
    almost_full => FIFO_almost_full,
    empty => FIFO_empty,
    almost_empty => FIFO_almost_empty,
    valid => FIFO_valid,
    prog_full => FIFO_prog_full
  );
  
	FIFO_IN_almost_full <= FIFO_prog_full OR Pause;
	Command_Inject_almost_full <= FIFO_prog_full OR Pause;

	Zarlink30BitCompilerProcess : process (Clk,Reset)
		
		variable TransmitType : std_logic_vector (6 downto 0) := (others => '0');
		variable PageType : std_logic := '0';

	begin
		if Reset = '1' then
			FIFO_din <= (others => '0');
			FIFO_wr_en <= '0';
			ExpectSecondPage <= '0';
			FirstPage <= (others => '0');
			
		elsif rising_edge(Clk) then
			
			-- Default:
			FIFO_din <= (others => '0');
			FIFO_wr_en <= '0';
			ExpectSecondPage <= ExpectSecondPage;			
			FirstPage <= FirstPage;
			
			if Pause = '1' then
				FIFO_din <= Command_Inject_din;
				FIFO_wr_en <= Command_Inject_valid;			
			else
				-- Extract the putative type of the transmission
				TransmitType := FIFO_IN_din(6 downto 0);
				-- Extract the putative page-type of the transmission 
				PageType := FIFO_IN_din(0);			
				
				if FIFO_IN_valid = '1' then
						case TransmitType is

							-- Two pages...
							when "0000000" | "0000010" | "0010110" | "0011000" =>
								ExpectSecondPage <= '1';
								FirstPage <= FIFO_IN_din(15 downto 1);
								
							-- One page
							when "0000100" | "0000110" | "0011010" =>
								FIFO_din <= "000000000000000" & FIFO_IN_din(15 downto 1);
								FIFO_wr_en <= '1';
								ExpectSecondPage <= '0';
								FirstPage <= (others => '0');
								
							when others =>
								if (ExpectSecondPage = '1') AND (PageType = '1') then 
									FIFO_din <= FIFO_IN_din(15 downto 1) & FirstPage;
									FIFO_wr_en <= '1';
								else
									null;
								end if;
								FirstPage <= (others => '0');
						end case;
				else
					null;
				end if;
				
			end if;
		
		end if;
	
	end process Zarlink30BitCompilerProcess;
	
end Behavioral;

