----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:50:40 10/09/2011 
-- Design Name: 
-- Module Name:    Zarlink_Command_Filter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity Zarlink_Command_Filter is
    Port ( 
	 -- FIFO IN
	 FIFO_IN_dout 					: IN STD_LOGIC_VECTOR(15 DOWNTO 0);
	 FIFO_IN_valid 				: IN STD_LOGIC;
	 FIFO_IN_rd_en 				: OUT STD_LOGIC;
	 
	 -- FIFO OUT
	 FIFO_OUT_din 					: OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
	 FIFO_OUT_wr_en 				: OUT STD_LOGIC;
	 FIFO_OUT_almost_full 		: IN STD_LOGIC;
	 
	 -- Irrelevant 
	 Irrelevant_OUT_din 			 : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
	 Irrelevant_OUT_wr_en 		 : OUT STD_LOGIC;
	 Irrelevant_OUT_almost_full : IN STD_LOGIC;
	
	 OnGoingBlock 					: out STD_LOGIC;
	 
	 Reset 							: in  STD_LOGIC;	
	 Clk 								: in  STD_LOGIC
	 
	 );
end Zarlink_Command_Filter;

architecture Behavioral of Zarlink_Command_Filter is

	signal AllowPage1ToPass : std_logic := '0';
	signal FIFO_IN_rd_en_Internal : std_logic := '0';

begin
	
	FIFO_IN_rd_en <= FIFO_IN_rd_en_Internal;
	
	ZarlinkCommandFirewall : process (Clk,Reset)
		
		variable TransmitType : std_logic_vector (6 downto 0) := (others => '0');
		variable PageType : std_logic := '0';
		
	begin
		
		if Reset = '1' then
			
			AllowPage1ToPass <= '0';
			FIFO_IN_rd_en_Internal <= '0';
			FIFO_OUT_wr_en <= '0';
			FIFO_OUT_din <= (others => '0');
			Irrelevant_OUT_wr_en <= '0';
			Irrelevant_OUT_din <= (others => '0');
			
			OnGoingBlock <= '0';
			
		elsif rising_edge(Clk) then
		
			-- Extract the putative type of the transmission
			TransmitType := FIFO_IN_dout(6 downto 0);
			-- Extract the putative page-type of the transmission 
			PageType := FIFO_IN_dout(0);
			
			-- Default:
			AllowPage1ToPass <= AllowPage1ToPass;
			FIFO_IN_rd_en_Internal <= '0';
			FIFO_OUT_wr_en <= '0';
			FIFO_OUT_din <= (others => '0');
			Irrelevant_OUT_wr_en <= '0';
			Irrelevant_OUT_din <= (others => '0');			
			OnGoingBlock <= '0';
			
			if (FIFO_IN_valid = '1') then
					
					if (FIFO_OUT_almost_full = '1') OR (Irrelevant_OUT_almost_full = '1') then
						FIFO_IN_rd_en_Internal <= '0';
					else
						FIFO_IN_rd_en_Internal <= '1';
					end if;
					
					if (FIFO_IN_rd_en_Internal = '1') AND (FIFO_IN_valid = '1') then
					
					
						case TransmitType is
							-- Two pages...
							when "0000000" | "0000010" | "0010110" | "0011000" =>
								FIFO_OUT_din <= FIFO_IN_dout;
								FIFO_OUT_wr_en <= '1';
								AllowPage1ToPass <= '1';
								OnGoingBlock <= '1';
								
							-- One page
							when "0000100" | "0000110" | "0011010" =>
								FIFO_OUT_din <= FIFO_IN_dout;
								FIFO_OUT_wr_en <= '1';
								AllowPage1ToPass <= '0';
								
								if TransmitType = "0000100" then
									OnGoingBlock <= '1';
								else
									null;
								end if;
							-- Irrelevant or second page 
							when others => 
								-- Second page 
								if (AllowPage1ToPass = '1') AND (PageType = '1') then
									FIFO_OUT_din <= FIFO_IN_dout;
									FIFO_OUT_wr_en <= '1';
									AllowPage1ToPass <= '0';
								-- Irrelevant 
								else
									Irrelevant_OUT_din <= FIFO_IN_dout;
									Irrelevant_OUT_wr_en <= '1';
									AllowPage1ToPass <= '0';
								end if;
							end case;
							
					else
						null;
					end if;
			else 
					null;
			end if;
			
			
		end if;
	
	end process ZarlinkCommandFirewall;


end Behavioral;

