library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
 
ENTITY Zarlink_SPI_Module_UUT IS
END Zarlink_SPI_Module_UUT;
 
ARCHITECTURE behavior OF Zarlink_SPI_Module_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT Zarlink_SPI_Module
    PORT(
         Zarlink_Data_Out : IN  std_logic_vector(119 downto 0);
         Zarlink_WriteEn : IN  std_logic;
         Zarlink_Data_In : OUT  std_logic_vector(111 downto 0);
         Zarlink_Valid_In : OUT  std_logic;
         Zarlink_BlockMode : IN  std_logic;
         Zarlink_Wait : OUT  std_logic;
         SPI_ZL_CS : OUT  std_logic;
         SPI_SDI : OUT  std_logic;
         SPI_SDO : IN  std_logic;
         SPI_CLK : OUT  std_logic;
         Reset : IN  std_logic;
         Clk : IN  std_logic
        );
    END COMPONENT;

   --Inputs
   signal Zarlink_Data_Out : std_logic_vector(119 downto 0) := (others => '0');
   signal Zarlink_WriteEn : std_logic := '0';
   signal Zarlink_BlockMode : std_logic := '0';
   signal SPI_SDO : std_logic := '0';
   signal Reset : std_logic := '0';
   signal Clk8MHz : std_logic := '0';
   signal Clk : std_logic := '0';

 	--Outputs
   signal Zarlink_Data_In : std_logic_vector(111 downto 0);
   signal Zarlink_Valid_In : std_logic;
   signal Zarlink_Wait : std_logic;
   signal SPI_ZL_CS : std_logic;
   signal SPI_SDI : std_logic;
   signal SPI_CLK : std_logic;

  	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 
	signal SecondCounter : unsigned(127 downto 0) := (others=>'0'); 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: Zarlink_SPI_Module PORT MAP (
          Zarlink_Data_Out => Zarlink_Data_Out,
          Zarlink_WriteEn => Zarlink_WriteEn,
          Zarlink_Data_In => Zarlink_Data_In,
          Zarlink_Valid_In => Zarlink_Valid_In,
          Zarlink_BlockMode => Zarlink_BlockMode,
          Zarlink_Wait => Zarlink_Wait,
          SPI_ZL_CS => SPI_ZL_CS,
          SPI_SDI => SPI_SDI,
          SPI_SDO => SPI_SDO,
          SPI_CLK => SPI_CLK,
          Reset => Reset,
          Clk => Clk
        );

     -- Clock process definitions
TheMasterClock : process
begin
	clockloop: for i in 0 to 25000 loop
		clk <= NOT clk;
		wait for 4ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

TheSecondClock : process
begin
	clockloop: for i in 0 to 1562 loop
		Clk8MHz <= NOT Clk8MHz;
		wait for 63ns;
	end loop clockloop;
	wait;
end process TheSecondClock; 



MasterProzess : process(clk) 
begin
	if rising_edge(clk) then
		
		MasterCounter <= MasterCounter + 1;
		
		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
		else
			Reset <= '0';
		end if;
		
		if MasterCounter = to_unsigned(15,128) then
			Zarlink_Data_Out <= std_logic_vector(to_unsigned(98720,120));
			Zarlink_WriteEn <= '1';
			Zarlink_BlockMode <= '0';
		else
			Zarlink_Data_Out <= (others => '0');
			Zarlink_WriteEn <= '0';
			Zarlink_BlockMode <= '0';
		end if;
		
	end if;
end process MasterProzess;

SecondProzess : process(Clk8MHz) 
begin
	if rising_edge(Clk8MHz) then
		SPI_SDO <= SecondCounter(3);
		SecondCounter <= SecondCounter + 1;

	end if;
end process SecondProzess;
 
END;
