-- ZestET1 Ethernet interface physical layer
--   File name: ZestET1_Ethernet.vhd
--   Version: 1.00
--   Date: 23/6/2009
--
--   Copyright (C) 2009 Orange Tree Technologies Ltd. All rights reserved.
--   Orange Tree Technologies grants the purchaser of a ZestET1 the right to use and
--   modify this logic core in any form including but not limited to VHDL source code or
--   EDIF netlist in FPGA designs that target the ZestET1.
--   Orange Tree Technologies prohibits the use of this logic core or any modification of
--   it in any form including but not limited to VHDL source code or EDIF netlist in
--   FPGA or ASIC designs that target any other hardware unless the purchaser of the
--   ZestET1 has purchased the appropriate licence from Orange Tree Technologies.
--   Contact Orange Tree Technologies if you want to purchase such a licence.
--
--  *****************************************************************************************
--  **
--  **  Disclaimer: LIMITED WARRANTY AND DISCLAIMER. These designs are
--  **              provided to you "as is". Orange Tree Technologies and its licensors 
--  **              make and you receive no warranties or conditions, express, implied, 
--  **              statutory or otherwise, and Orange Tree Technologies specifically 
--  **              disclaims any implied warranties of merchantability, non-infringement,
--  **              or fitness for a particular purpose. Orange Tree Technologies does not
--  **              warrant that the functions contained in these designs will meet your 
--  **              requirements, or that the operation of these designs will be 
--  **              uninterrupted or error free, or that defects in the Designs will be 
--  **              corrected. Furthermore, Orange Tree Technologies does not warrant or 
--  **              make any representations regarding use or the results of the use of the 
--  **              designs in terms of correctness, accuracy, reliability, or otherwise.                                               
--  **
--  **              LIMITATION OF LIABILITY. In no event will Orange Tree Technologies 
--  **              or its licensors be liable for any loss of data, lost profits, cost or 
--  **              procurement of substitute goods or services, or for any special, 
--  **              incidental, consequential, or indirect damages arising from the use or 
--  **              operation of the designs or accompanying documentation, however caused 
--  **              and on any theory of liability. This limitation will apply even if 
--  **              Orange Tree Technologies has been advised of the possibility of such 
--  **              damage. This limitation shall apply notwithstanding the failure of the 
--  **              essential purpose of any limited remedies herein.
--  **
--  *****************************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity ZestET1_Ethernet is
    port (
        User_CLK : in std_logic;
        User_RST : in std_logic;

        -- User interface
        User_WE : in std_logic;
        User_RE : in std_logic;
        User_Addr : in std_logic_vector(4 downto 0);
        User_WriteData : in std_logic_vector(15 downto 0);
        User_Owner : in std_logic_vector(7 downto 0);
        User_BE : in std_logic_vector(1 downto 0);
        User_ReadData : out std_logic_vector(15 downto 0);
        User_ReadDataValid : out std_logic;
        User_ValidOwner : out std_logic_vector(7 downto 0);
        User_Interrupt : out std_logic;
        
        -- Interface to GigExpedite
        Eth_Clk : out std_logic;
        Eth_CSn : out std_logic;
        Eth_WEn : out std_logic;
        Eth_A : out std_logic_vector(4 downto 0);
        Eth_D : inout std_logic_vector(15 downto 0);
        Eth_BE : out std_logic_vector(1 downto 0);
        Eth_Intn : in std_logic
    );
end ZestET1_Ethernet;

architecture arch of ZestET1_Ethernet is

    -- Declare signals
    signal UserDataDelay3 : std_logic_vector(15 downto 0);
    signal UserDataDelay2 : std_logic_vector(15 downto 0);
    signal UserDataDelay1 : std_logic_vector(15 downto 0);
    signal UserWEDelay3 : std_logic;
    signal UserWEDelay2 : std_logic;
    signal UserWEDelay1 : std_logic;

    signal RegUser_Addr : std_logic_vector(4 downto 0);
    signal RegUser_CS : std_logic;
    signal RegUser_WE : std_logic;
    signal RegUser_RE : std_logic;
    signal RegUser_BE : std_logic_vector(1 downto 0);
    
    signal Eth_DVal : std_logic_vector(15 downto 0);
    signal Eth_TSVal : std_logic_vector(15 downto 0);
    signal Eth_DTSVal : std_logic_vector(15 downto 0);

    signal Eth_CSnVal : std_logic;
    signal Eth_WEnVal : std_logic;
    signal Eth_AVal : std_logic_vector(4 downto 0);
    signal Eth_BEVal : std_logic_vector(1 downto 0);

    signal RegEth_D : std_logic_vector(15 downto 0);
    signal ReadDelay : std_logic_vector(3 downto 0);
    type OWNER_TYPE is array(0 to 3) of std_logic_vector(7 downto 0);
    signal Owner : OWNER_TYPE;

begin

    -- Simple connections
    Eth_Clk <= User_CLK;
    process (User_CLK)
    begin
        if (User_CLK'event and User_CLK='1') then
            User_Interrupt <= not Eth_Intn;
        end if;
    end process;

    -- Data outputs
    -- Signals are passed through DDR FFs to ensure consistent
    -- and predictable timing to pads
    process (User_CLK)
    begin
        if (User_CLK'event and User_CLK='1') then
            UserDataDelay3 <= UserDataDelay2;
            UserDataDelay2 <= UserDataDelay1;
            UserDataDelay1 <= User_WriteData;
            UserWEDelay3 <= UserWEDelay2;
            UserWEDelay2 <= UserWEDelay1;
            UserWEDelay1 <= not User_WE;
        end if;
    end process;

    DataOutCopy:
        for g in 0 to 15 generate
        begin
            DataOutCopyInst : ODDR2
                generic map (
                    DDR_ALIGNMENT => "NONE",
                    INIT => '0',
                    SRTYPE => "SYNC"
                )
                port map (
                    Q => Eth_DVal(g),
                    C0 => User_CLK,
                    C1 => not User_CLK,
                    CE => '1',
                    D0 => UserDataDelay3(g),
                    D1 => UserDataDelay3(g),
                    R => '0',
                    S => '0'
                );
        end generate;
			
    DataTSCopy:
        for g in 0 to 15 generate
        begin
            DataTSCopyInst : ODDR2
                generic map (
                    DDR_ALIGNMENT => "NONE",
                    INIT => '0',
                    SRTYPE => "SYNC"
                )
                port map (
                    Q => Eth_TSVal(g),
                    C0 => User_CLK,
                    C1 => not User_CLK,
                    CE => '1',
                    D0 => UserWEDelay3,
                    D1 => UserWEDelay3,
                    R => '0',
                    S => '0'
                );
        end generate;
	
    DataTS:
        for g in 0 to 15 generate
        begin
            Eth_DTSVal(g) <= Eth_DVal(g) when Eth_TSVal(g)='0' else 'Z';
        end generate;
    Eth_D <= Eth_DTSVal;
    
    -- Address and control outputs
    process (User_CLK)
    begin
        if (User_CLK'event and User_CLK='1') then
            RegUser_CS <= not (User_WE or User_RE);
            RegUser_BE <= not User_BE;
            RegUser_WE <= not User_WE;
            RegUser_RE <= not User_RE;
            RegUser_Addr <= User_Addr;
        end if;
    end process;
    
    CSnInst : ODDR2
        generic map (
            DDR_ALIGNMENT => "NONE",
            INIT => '0',
            SRTYPE => "SYNC"
        )
        port map (
            Q => Eth_CSnVal,
            C0 => User_CLK,
            C1 => not User_CLK,
            CE => '1',
            D0 => RegUser_CS,
            D1 => RegUser_CS,
            R => '0',
            S => '0'
        );

    WEnInst : ODDR2
        generic map (
            DDR_ALIGNMENT => "NONE",
            INIT => '0',
            SRTYPE => "SYNC"
        )
        port map (
            Q => Eth_WEnVal,
            C0 => User_CLK,
            C1 => not User_CLK,
            CE => '1',
            D0 => RegUser_WE,
            D1 => RegUser_WE,
            R => '0',
            S => '0'
        );

    AddrCopy:
        for g in 0 to 4 generate
        begin
            AddrCopyInst : ODDR2
                generic map (
                    DDR_ALIGNMENT => "NONE",
                    INIT => '0',
                    SRTYPE => "SYNC"
                )
                port map (
                    Q => Eth_AVal(g),
                    C0 => User_CLK,
                    C1 => not User_CLK,
                    CE => '1',
                    D0 => RegUser_Addr(g),
                    D1 => RegUser_Addr(g),
                    R => '0',
                    S => '0'
                );
        end generate;

    BECopy:
        for g in 0 to 1 generate
        begin
            BECopyInst : ODDR2
                generic map (
                    DDR_ALIGNMENT => "NONE",
                    INIT => '0',
                    SRTYPE => "SYNC"
                )
                port map (
                    Q => Eth_BEVal(g),
                    C0 => User_CLK,
                    C1 => not User_CLK,
                    CE => '1',
                    D0 => RegUser_BE(g),
                    D1 => RegUser_BE(g),
                    R => '0',
                    S => '0'
                );
        end generate;

    Eth_CSn <= Eth_CSnVal;
    Eth_WEn <= Eth_WEnVal;
    Eth_A <= Eth_AVal;
    Eth_BE <= Eth_BEVal;
    
    -- Read path
    process (User_CLK)
    begin
        if (User_CLK'event and User_CLK='0') then
            RegEth_D <= Eth_D;
        end if;
    end process;
    process (User_CLK)
    begin
        if (User_CLK'event and User_CLK='1') then
            User_ReadData <= RegEth_D;
        end if;
    end process;
    process (User_CLK)
    begin
        if (User_CLK'event and User_CLK='1') then
            ReadDelay <= ReadDelay(2 downto 0) & not RegUser_RE;
            Owner(3) <= Owner(2);
            Owner(2) <= Owner(1);
            Owner(1) <= Owner(0);
            Owner(0) <= User_Owner;
        end if;
    end process;
    User_ReadDataValid <= ReadDelay(3);
    User_ValidOwner <= Owner(3);

end arch;
