-- ZestET1 DDR SDRAM Physical layer
--   File name: ZestET1_SDRAMPhy.vhd
--   Version: 1.00
--   Date: 23/6/2009
--
--   DDR SDRAM controller for user FPGA on ZestET1 boards
--   Physical layer.  Directly instantiates components on time critical sections
--   of the controller logic.
--    
--   Copyright (C) 2009 Orange Tree Technologies Ltd. All rights reserved.
--   Orange Tree Technologies grants the purchaser of a ZestET1 the right to use and
--   modify this logic core in any form including but not limited to VHDL source code or
--   EDIF netlist in FPGA designs that target the ZestET1.
--   Orange Tree Technologies prohibits the use of this logic core or any modification of
--   it in any form including but not limited to VHDL source code or EDIF netlist in
--   FPGA or ASIC designs that target any other hardware unless the purchaser of the
--   ZestET1 has purchased the appropriate licence from Orange Tree Technologies.
--   Contact Orange Tree Technologies if you want to purchase such a licence.
--
--  *****************************************************************************************
--  **
--  **  Disclaimer: LIMITED WARRANTY AND DISCLAIMER. These designs are
--  **              provided to you "as is". Orange Tree Technologies and its licensors 
--  **              make and you receive no warranties or conditions, express, implied, 
--  **              statutory or otherwise, and Orange Tree Technologies specifically 
--  **              disclaims any implied warranties of merchantability, non-infringement,
--  **              or fitness for a particular purpose. Orange Tree Technologies does not
--  **              warrant that the functions contained in these designs will meet your 
--  **              requirements, or that the operation of these designs will be 
--  **              uninterrupted or error free, or that defects in the Designs will be 
--  **              corrected. Furthermore, Orange Tree Technologies does not warrant or 
--  **              make any representations regarding use or the results of the use of the 
--  **              designs in terms of correctness, accuracy, reliability, or otherwise.                                               
--  **
--  **              LIMITATION OF LIABILITY. In no event will Orange Tree Technologies 
--  **              or its licensors be liable for any loss of data, lost profits, cost or 
--  **              procurement of substitute goods or services, or for any special, 
--  **              incidental, consequential, or indirect damages arising from the use or 
--  **              operation of the designs or accompanying documentation, however caused 
--  **              and on any theory of liability. This limitation will apply even if 
--  **              Orange Tree Technologies has been advised of the possibility of such 
--  **              damage. This limitation shall apply notwithstanding the failure of the 
--  **              essential purpose of any limited remedies herein.
--  **
--  *****************************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity ZestET1_SDRAMPhy is
    port (
        CLK : in std_logic;
        CLK90 : in std_logic;
        RST : in std_logic;
        ReadVal : in std_logic;
        DataOut : in std_logic_vector(31 downto 0);
        DataTri_P : in std_logic;
        DataTri_N : in std_logic;
        DQSTri_P : in std_logic;
        DQSTri_N : in std_logic;
        DM_P : in std_logic_vector(1 downto 0);
        DM_N : in std_logic_vector(1 downto 0);
        DS_CLK_P : out std_logic;
        DS_CLK_N : out std_logic;
        DataIn : out std_logic_vector(31 downto 0);
        DS_DM : out std_logic_vector(1 downto 0);
        DataValid : out std_logic;
        DS_DQ : inout std_logic_vector(15 downto 0);
        DS_DQS : inout std_logic_vector(1 downto 0)
    );
end ZestET1_SDRAMPhy;

architecture arch of ZestET1_SDRAMPhy is

    --------------------------------------------------------------------------
    -- Declare constants
    -- FOR XC3S1400-5
    constant DQ_DELAY : string := "2";
    constant DQS_DELAY : string := "1";

    --------------------------------------------------------------------------
    -- Declare signals
    signal DQSBuf1 : std_logic;
    signal DQSBuf0 : std_logic;
    attribute syn_keep: boolean;
    attribute buffer_type : string;
    attribute buffer_type of DQSBuf0 : signal is "none";
    attribute buffer_type of DQSBuf1 : signal is "none";
    attribute buffer_type of DS_DQS : signal is "none";
    attribute syn_keep of DQSBuf0: signal is true;
    attribute syn_keep of DQSBuf1: signal is true;

    signal DDRIn15 : std_logic;
    signal DDRIn14 : std_logic;
    signal DDRIn13 : std_logic;
    signal DDRIn12 : std_logic;
    signal DDRIn11 : std_logic;
    signal DDRIn10 : std_logic;
    signal DDRIn9 : std_logic;
    signal DDRIn8 : std_logic;
    signal DDRIn7 : std_logic;
    signal DDRIn6 : std_logic;
    signal DDRIn5 : std_logic;
    signal DDRIn4 : std_logic;
    signal DDRIn3 : std_logic;
    signal DDRIn2 : std_logic;
    signal DDRIn1 : std_logic;
    signal DDRIn0 : std_logic;
    attribute syn_keep of DDRIn15: signal is true;
    attribute syn_keep of DDRIn14: signal is true;
    attribute syn_keep of DDRIn13: signal is true;
    attribute syn_keep of DDRIn12: signal is true;
    attribute syn_keep of DDRIn11: signal is true;
    attribute syn_keep of DDRIn10: signal is true;
    attribute syn_keep of DDRIn9: signal is true;
    attribute syn_keep of DDRIn8: signal is true;
    attribute syn_keep of DDRIn7: signal is true;
    attribute syn_keep of DDRIn6: signal is true;
    attribute syn_keep of DDRIn5: signal is true;
    attribute syn_keep of DDRIn4: signal is true;
    attribute syn_keep of DDRIn3: signal is true;
    attribute syn_keep of DDRIn2: signal is true;
    attribute syn_keep of DDRIn1: signal is true;
    attribute syn_keep of DDRIn0: signal is true;
    
    signal DQSOut : std_logic_vector(1 downto 0);
    signal DQSTri : std_logic_vector(1 downto 0);
    signal DDRMask : std_logic_vector(1 downto 0);
    signal DDROut : std_logic_vector(15 downto 0);
    signal DDRTri : std_logic_vector(15 downto 0);
    signal nTristate_p : std_logic_vector(15 downto 0);
    signal nTristate_n : std_logic_vector(15 downto 0);

    signal DataInDQS : std_logic_vector(31 downto 0);
    attribute syn_keep of DataInDQS: signal is true;
    signal DataInAlignDQS : std_logic_vector(15 downto 0);
    attribute syn_keep of DataInAlignDQS: signal is true;
    signal ReadHistory : std_logic_vector(7 downto 0);
    signal ReclockDQ : std_logic_vector(31 downto 0);
    signal DQSDelayed0 : std_logic_vector(1 downto 0);
    attribute syn_keep of DQSDelayed0: signal is true;
    signal DQSDelayed1 : std_logic_vector(1 downto 0);
    attribute syn_keep of DQSDelayed1: signal is true;

begin
    
    -- Instantiate DDR FFs for clock output to RAM
    CLK_PInst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DS_CLK_P, C0 => not CLK, C1 => CLK, CE => '1',
                  D0 => '1', D1 => '0', R => '0', S => '0');
    CLK_NInst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DS_CLK_N, C0 => not CLK, C1 => CLK, CE => '1',
                  D0 => '0', D1 => '1', R => '0', S => '0');

    -- Instantiate FFs for DQS signals
    DQSOut1Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DQSOut(1), C0 => not CLK, C1 => CLK, CE => '1',
                  D0 => '1', D1 => '0', R => '0', S => '0');

    DQSOut0Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DQSOut(0), C0 => not CLK, C1 => CLK, CE => '1',
                  D0 => '1', D1 => '0', R => '0', S => '0');

    DQSTri1Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DQSTri(1), C0 => not CLK, C1 => CLK, CE => '1',
                  D0 => DQSTri_P, D1 => DQSTri_N, R => '0', S => '0');

    DQSTri0Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DQSTri(0), C0 => not CLK, C1 => CLK, CE => '1',
                  D0 => DQSTri_P, D1 => DQSTri_N, R => '0', S => '0');

    -- Instantiate FFs for DM signals
    DM1Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDRMask(0), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DM_P(1), D1 => DM_N(1), R => '0', S => '0');
                 
    DM0Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDRMask(1), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DM_P(0), D1 => DM_N(0), R => '0', S => '0');
                 
    -- Instantiate FFs for DQ signals
    DQ15Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(15), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(31), D1 => DataOut(15), R => '0', S => '0');

    DQ14Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(14), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(30), D1 => DataOut(14), R => '0', S => '0');

    DQ13Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(13), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(29), D1 => DataOut(13), R => '0', S => '0');

    DQ12Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(12), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(28), D1 => DataOut(12), R => '0', S => '0');

    DQ11Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(11), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(27), D1 => DataOut(11), R => '0', S => '0');

    DQ10Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(10), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(26), D1 => DataOut(10), R => '0', S => '0');

    DQ9Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(9), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(25), D1 => DataOut(9), R => '0', S => '0');

    DQ8Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(8), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(24), D1 => DataOut(8), R => '0', S => '0');

    DQ7Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(7), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(23), D1 => DataOut(7), R => '0', S => '0');

    DQ6Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(6), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(22), D1 => DataOut(6), R => '0', S => '0');

    DQ5Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(5), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(21), D1 => DataOut(5), R => '0', S => '0');

    DQ4Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(4), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(20), D1 => DataOut(4), R => '0', S => '0');

    DQ3Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(3), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(19), D1 => DataOut(3), R => '0', S => '0');

    DQ2Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(2), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(18), D1 => DataOut(2), R => '0', S => '0');

    DQ1Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(1), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(17), D1 => DataOut(1), R => '0', S => '0');

    DQ0Inst : ODDR2
        generic map (DDR_ALIGNMENT => "NONE", INIT => '0', SRTYPE => "SYNC")
        port map (Q => DDROut(0), C0 => CLK90, C1 => not CLK90, CE => '1',
                  D0 => DataOut(16), D1 => DataOut(0), R => '0', S => '0');

    -- Copy tristate signal and make sure its preserved
    TristateCopyp :
        for i in 0 to 15 generate
        begin
            FDCE_inst : FDCE
                generic map (
                    INIT => '0'
                )
                port map (
                    Q => nTristate_p(i),
                    C => not CLK,
                    CE => '1',
                    CLR => '0',
                    D => DataTri_P
                );            
        end generate;

    TristateCopyn :
        for i in 0 to 15 generate
        begin
            FDCE_inst : FDCE
                generic map (
                    INIT => '0'
                )
                port map (
                    Q => nTristate_n(i),
                    C => CLK,
                    CE => '1',
                    CLR => '0',
                    D => DataTri_N
                );            
        end generate;

    -- Instantiate FFs for tristate signals
    TristateDDR :
        for i in 0 to 15 generate
        begin
            TristateDDRInst : ODDR2
                generic map (
                    DDR_ALIGNMENT => "NONE",
                    INIT => '0',
                    SRTYPE => "SYNC"
                )
                port map (
                    Q => DDRTri(i), C0 => CLK90, C1 => not CLK90, CE => '1',
                    D0 => nTristate_p(i), D1 => nTristate_n(i), R => '0', S => '0'
                );
        end generate;
    
    -- Clock incoming data using DQS strobe
    process (DQSBuf0) begin
        if (DQSBuf0'event and DQSBuf0='1') then
            DataInDQS(7 downto 0) <= DDRIn7 & DDRIn6 & DDRIn5 & DDRIn4 & DDRIn3 & DDRIn2 & DDRIn1 & DDRIn0;
        end if;
    end process;
    process (DQSBuf0) begin
        if (DQSBuf0'event and DQSBuf0='0') then
            DataInDQS(23 downto 16) <= DDRIn7 & DDRIn6 & DDRIn5 & DDRIn4 & DDRIn3 & DDRIn2 & DDRIn1 & DDRIn0;
        end if;
    end process;
    process (DQSBuf1) begin
        if (DQSBuf1'event and DQSBuf1='1') then
            DataInDQS(15 downto 8) <= DDRIn15 & DDRIn14 & DDRIn13 & DDRIn12 & DDRIn11 & DDRIn10 & DDRIn9 & DDRIn8;
        end if;
    end process;
    process (DQSBuf1) begin
        if (DQSBuf1'event and DQSBuf1='0') then
            DataInDQS(31 downto 24) <= DDRIn15 & DDRIn14 & DDRIn13 & DDRIn12 & DDRIn11 & DDRIn10 & DDRIn9 & DDRIn8;
        end if;
    end process;

    -- Align all incoming data to negative DQS edge
    process (DQSBuf0) begin
        if (DQSBuf0'event and DQSBuf0='0') then
            DataInAlignDQS(7 downto 0) <= DataInDQS(7 downto 0);
        end if;
    end process;
    process (DQSBuf1) begin
        if (DQSBuf1'event and DQSBuf1='0') then
            DataInAlignDQS(15 downto 8) <= DataInDQS(15 downto 8);
        end if;
    end process;
    
    -- Mark valid read data
    process (RST, CLK) begin
        if (RST='1') then
            ReadHistory <= (others=>'0');
            DataValid <= '0';
        elsif (CLK'event and CLK='1') then
            ReadHistory <= ReadHistory(6 downto 0) & ReadVal;
            DataValid <= ReadHistory(3);
        end if;
    end process;
    
    -- Align data to user clock
    -- Need to clock to negedge of clock first
    process (CLK) begin
        if (CLK'event and CLK='0') then
            ReclockDQ <= DataInDQS(31 downto 16) & DataInAlignDQS;
        end if;
    end process;
    process (CLK) begin
        if (CLK'event and CLK='1') then
            DataIn <= ReclockDQ(15 downto 0) & ReclockDQ(31 downto 16);
        end if;
    end process;

    -- Introduce phase shift on DQS here
    IBUF_DLY_ADJ_inst0 : IBUF
        generic map (
            IBUF_DELAY_VALUE => DQS_DELAY,
            IOSTANDARD => "DEFAULT"
        )
        port map (
            O => DQSDelayed0(0),
            I => DS_DQS(0)
        );
    DQS0Buffer0 : LUT1
        generic map (
            INIT => "01"
        )
        port map (
            I0 => DQSDelayed0(0),
            O => DQSDelayed0(1)
        );
    DQS0Buffer1 : LUT1
        generic map (
            INIT => "01"
        )
        port map (
            I0 => DQSDelayed0(1),
            O => DQSBuf0
    );
    Inst2_BB0 : OBUFT port map (I => DQSOut(0), T => DQSTri(0), O => DS_DQS(0));
    
    IBUF_DLY_ADJ_inst1 : IBUF
        generic map (
            IBUF_DELAY_VALUE => DQS_DELAY,
            IOSTANDARD => "DEFAULT"
        )
        port map (
            I => DS_DQS(1),
            O => DQSDelayed1(0)
        );
    DQS1Buffer0 : LUT1
        generic map (
            INIT => "01"
        )
        port map (
            I0 => DQSDelayed1(0),
            O => DQSDelayed1(1)
        );
    DQS1Buffer1 : LUT1
        generic map (
            INIT => "01"
        )
        port map (
            I0 => DQSDelayed1(1),
            O => DQSBuf1
        );
    DQSOBuf1 : OBUFT port map (I => DQSOut(1), T => DQSTri(1), O => DS_DQS(1));

    -- Data bus buffers
    DMBuf1 : OBUF port map (I => DDRMask(1), O => DS_DM(1));
    DMBuf0 : OBUF port map (I => DDRMask(0), O => DS_DM(0));
    DQIBuf15 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn15, I => DS_DQ(15));
    DQIBuf14 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn14, I => DS_DQ(14));
    DQIBuf13 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn13, I => DS_DQ(13));
    DQIBuf12 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn12, I => DS_DQ(12));
    DQIBuf11 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn11, I => DS_DQ(11));
    DQIBuf10 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn10, I => DS_DQ(10));
    DQIBuf9 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn9, I => DS_DQ(9));
    DQIBuf8 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn8, I => DS_DQ(8));
    DQIBuf7 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn7, I => DS_DQ(7));
    DQIBuf6 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn6, I => DS_DQ(6));
    DQIBuf5 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn5, I => DS_DQ(5));
    DQIBuf4 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn4, I => DS_DQ(4));
    DQIBuf3 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn3, I => DS_DQ(3));
    DQIBuf2 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn2, I => DS_DQ(2));
    DQIBuf1 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn1, I => DS_DQ(1));
    DQIBuf0 : IBUF generic map (IBUF_DELAY_VALUE => DQ_DELAY) 
                    port map (O => DDRIn0, I => DS_DQ(0));
   
    DOutBuf :
        for i in 0 to 15 generate
        begin
            DQOBuf : OBUFT 
                port map (
                    I => DDROut(i),
                    T => DDRTri(i),
                    O => DS_DQ(i)
                );
        end generate;
    
end arch;
